@@ -1,4 +1,4 @@
-#     Copyright 2015, Kay Hayen, mailto:kay.hayen@gmail.com
+#     Copyright 2016, Kay Hayen, mailto:kay.hayen@gmail.com
 #
 #     Part of "Nuitka", an optimizing Python compiler that is compatible and
 #     integrates with CPython, but also works on its own.
@@ -30,7 +30,7 @@ from logging import info, warning
 
 from nuitka.importing import Importing, Recursion
 from nuitka.plugins.PluginBase import Plugins
-from nuitka.PythonVersions import isUninstalledPython
+from nuitka.PythonVersions import isUninstalledPython, python_version
 from nuitka.tree import SyntaxErrors
 from nuitka.utils import InstanceCounters, Utils
 
@@ -38,12 +38,7 @@ from . import ModuleRegistry, Options, Tracing, TreeXML
 from .build import SconsInterface
 from .codegen import CodeGeneration, ConstantCodes, MainCodes
 from .finalizations import Finalization
-from .freezer.BytecodeModuleFreezer import (
-    addFrozenModule,
-    generateBytecodeFrozenCode,
-    getFrozenModuleCount,
-    removeFrozenModule
-)
+from .freezer.BytecodeModuleFreezer import generateBytecodeFrozenCode
 from .freezer.Standalone import (
     copyUsedDLLs,
     detectEarlyImports,
@@ -111,7 +106,7 @@ def dumpTreeXML(tree):
     TreeXML.dump(xml_root)
 
 
-def displayTree(tree):
+def displayTree(tree): # pragma: no cover
     # Import only locally so the Qt4 dependency doesn't normally come into play
     # when it's not strictly needed.
     from .gui import TreeDisplay
@@ -124,7 +119,7 @@ def getTreeFilenameWithSuffix(tree, suffix):
 
 
 def getSourceDirectoryPath(main_module):
-    assert main_module.isPythonModule()
+    assert main_module.isCompiledPythonModule()
 
     return Options.getOutputPath(
         path = Utils.basename(
@@ -142,7 +137,7 @@ def getStandaloneDirectoryPath(main_module):
 
 
 def getResultBasepath(main_module):
-    assert main_module.isPythonModule()
+    assert main_module.isCompiledPythonModule()
 
     if Options.isStandaloneMode():
         return Utils.joinpath(
@@ -267,25 +262,46 @@ def pickSourceFilenames(source_dir, modules):
 standalone_entry_points = []
 
 def makeSourceDirectory(main_module):
+    """ Get the full list of modules imported, create code for all of them.
+
+    """
     # We deal with a lot of details here, but rather one by one, and split makes
     # no sense, pylint: disable=R0912,R0914
 
-    assert main_module.isPythonModule()
+    assert main_module.isCompiledPythonModule()
 
     # The global context used to generate code.
     global_context = CodeGeneration.makeGlobalContext()
 
-    # Get the full list of modules imported, create code for all of them.
-    modules = ModuleRegistry.getDoneModules()
-    assert main_module in modules
+    assert main_module in ModuleRegistry.getDoneModules()
 
-    # Sometimes we need to talk about all modules except main module.
-    other_modules = ModuleRegistry.getDoneUserModules()
+    # We might have chosen to include it as bytecode, and only compiled it for
+    # fun, and to find its imports. In this case, now we just can drop it. Or
+    # a module may shadow a frozen module, but be a different one, then we can
+    # drop the frozen one.
+    # TODO: This really should be done when the compiled module comes into
+    # existence.
+    for module in ModuleRegistry.getDoneUserModules():
+        if module.isCompiledPythonModule():
+            uncompiled_module = ModuleRegistry.getUncompiledModule(
+                module_name     = module.getFullName(),
+                module_filename = module.getCompileTimeFilename()
+            )
+
+            if uncompiled_module is not None:
+                # We now need to decide which one to keep, compiled or uncompiled
+                # module. Some uncompiled modules may have been asked by the user
+                # or technically required. By default, frozen code if it exists
+                # is preferred, as it will be from standalone mode adding it.
+                if uncompiled_module.isUserProvided():
+                    ModuleRegistry.removeDoneModule(module)
+                else:
+                    ModuleRegistry.removeUncompiledModule(uncompiled_module)
 
     # Lets check if the recurse-to modules are actually present, and warn the
-    # user if one was not found.
+    # user if one of those was not found.
     for any_case_module in Options.getShallFollowModules():
-        for module in other_modules:
+        for module in ModuleRegistry.getDoneUserModules():
             if module.getFullName() == any_case_module:
                 break
         else:
@@ -295,8 +311,8 @@ def makeSourceDirectory(main_module):
             )
 
     # Prepare code generation, i.e. execute finalization for it.
-    for module in sorted(modules, key = lambda x : x.getFullName()):
-        if module.isPythonModule():
+    for module in ModuleRegistry.getDoneModules():
+        if module.isCompiledPythonModule():
             Finalization.prepareCodeGeneration(module)
 
     # Pick filenames.
@@ -304,7 +320,7 @@ def makeSourceDirectory(main_module):
 
     module_filenames = pickSourceFilenames(
         source_dir = source_dir,
-        modules    = modules
+        modules    = ModuleRegistry.getDoneModules()
     )
 
     # First pass, generate code and use constants doing so, but prepare the
@@ -312,32 +328,23 @@ def makeSourceDirectory(main_module):
     # end only.
     prepared_modules = {}
 
-    for module in sorted(modules, key = lambda x : x.getFullName()):
-        if module.isPythonModule():
+    for module in ModuleRegistry.getDoneModules():
+        if module.isCompiledPythonModule():
             cpp_filename = module_filenames[module]
 
             prepared_modules[cpp_filename] = CodeGeneration.prepareModuleCode(
                 global_context = global_context,
                 module         = module,
                 module_name    = module.getFullName(),
-                other_modules  = other_modules
-                                   if module is main_module else
-                                 ()
             )
 
             # Main code constants need to be allocated already too.
             if module is main_module and not Options.shallMakeModule():
                 prepared_modules[cpp_filename][1].getConstantCode(0)
 
-
-    for module in sorted(modules, key = lambda x : x.getFullName()):
-        if module.isPythonModule():
-            if removeFrozenModule(module.getFullName()):
-                warning(
-                    """\
-Compiled module shadows standard library module '%s'.""" % module.getFullName()
-                )
-
+    # Second pass, generate the actual module code into the files.
+    for module in ModuleRegistry.getDoneModules():
+        if module.isCompiledPythonModule():
             cpp_filename = module_filenames[module]
 
             template_values, module_context = prepared_modules[cpp_filename]
@@ -397,7 +404,9 @@ Compiled module shadows standard library module '%s'.""" % module.getFullName()
         )
     )
 
-    helper_decl_code, helper_impl_code = CodeGeneration.generateHelpersCode()
+    helper_decl_code, helper_impl_code = CodeGeneration.generateHelpersCode(
+        ModuleRegistry.getDoneUserModules()
+    )
 
     writeSourceCode(
         filename    = Utils.joinpath(source_dir, "__helpers.hpp"),
@@ -414,17 +423,17 @@ def runScons(main_module, quiet):
     # Scons gets transported many details, that we express as variables, and
     # have checks for them, leading to many branches, pylint: disable=R0912
 
-    python_version = "%d.%d" % (sys.version_info[0], sys.version_info[1])
+    python_version_str = "%d.%d" % (sys.version_info[0], sys.version_info[1])
 
     if hasattr(sys, "abiflags"):
         if Options.isPythonDebug() or \
            hasattr(sys, "getobjects"):
             if sys.abiflags.startswith('d'):
-                python_version += sys.abiflags
+                python_version_str += sys.abiflags
             else:
-                python_version += 'd' + sys.abiflags
+                python_version_str += 'd' + sys.abiflags
         else:
-            python_version += sys.abiflags
+            python_version_str += sys.abiflags
 
     def asBoolStr(value):
         return "true" if value else "false"
@@ -443,7 +452,7 @@ def runScons(main_module, quiet):
         "full_compat"     : asBoolStr(Options.isFullCompat()),
         "experimental"    : asBoolStr(Options.isExperimental()),
         "trace_mode"      : asBoolStr(Options.shallTraceExecution()),
-        "python_version"  : python_version,
+        "python_version"  : python_version_str,
         "target_arch"     : Utils.getArchitecture(),
         "python_prefix"   : sys.prefix,
         "nuitka_src"      : SconsInterface.getSconsDataPath(),
@@ -469,11 +478,11 @@ def runScons(main_module, quiet):
     if not Options.isStandaloneMode() and \
        not Options.shallMakeModule() and \
        isUninstalledPython():
-        options["win_copy_dll"] = "true"
+        options["uninstalled_python"] = "true"
 
-    if getFrozenModuleCount():
+    if ModuleRegistry.getUncompiledModules():
         options["frozen_modules"] = str(
-            getFrozenModuleCount()
+            len(ModuleRegistry.getUncompiledModules())
         )
 
     if Options.isShowScons():
@@ -497,6 +506,9 @@ def runScons(main_module, quiet):
     if Options.getIconPath():
         options["icon_path"] = Options.getIconPath()
 
+    if Options.isProfile():
+        options["profile_mode"] = "true"
+
     return SconsInterface.runScons(options, quiet), options
 
 
@@ -505,7 +517,7 @@ def writeSourceCode(filename, source_code):
     # or something else has failed.
     assert not Utils.isFile(filename), filename
 
-    if Utils.python_version >= 300:
+    if python_version >= 300:
         with open(filename, "wb") as output_file:
             output_file.write(source_code.encode("latin1"))
     else:
@@ -541,24 +553,16 @@ def callExec(args, clean_path, add_path):
     sys.stderr.flush()
 
     # Add the main arguments, previous separated.
-    args += Options.getMainArgs()
+    args += Options.getPositionalArgs()[1:] + Options.getMainArgs()
 
     Utils.callExec(args)
 
 
-def executeMain(binary_filename, tree, clean_path):
-    main_filename = tree.getFilename()
+def executeMain(binary_filename, clean_path):
+    args = (binary_filename, binary_filename)
 
-    if Options.isStandaloneMode():
-        name = binary_filename
-    elif main_filename.endswith(".py"):
-        name = main_filename[:-3]
-    else:
-        name = main_filename
-
-    name = Utils.abspath(name)
-
-    args = (binary_filename, name)
+    if Options.shallRunInDebugger():
+        args = ("/usr/bin/gdb", "gdb", "-ex=run", "-ex=where", "--args", binary_filename)
 
     callExec(
         clean_path = clean_path,
@@ -594,11 +598,12 @@ def compileTree(main_module):
         )
 
         if Options.isStandaloneMode():
-            for late_import in detectLateImports():
-                addFrozenModule(late_import)
+            for module in detectLateImports():
+                ModuleRegistry.addUncompiledModule(module)
 
-        if getFrozenModuleCount():
-            frozen_code = generateBytecodeFrozenCode()
+        frozen_code = generateBytecodeFrozenCode()
+
+        if frozen_code is not None:
 
             writeSourceCode(
                 filename    = Utils.joinpath(
@@ -670,12 +675,12 @@ def main():
     # Detect to be frozen modules if any, so we can consider to not recurse
     # to them.
     if Options.isStandaloneMode():
-        for early_import in detectEarlyImports():
-            addFrozenModule(early_import)
+        for module in detectEarlyImports():
+            ModuleRegistry.addUncompiledModule(module)
 
-            if early_import[0] == "site":
+            if module.getName() == "site":
                 origin_prefix_filename = Utils.joinpath(
-                    Utils.dirname(early_import[3]),
+                    Utils.dirname(module.getCompileTimeFilename()),
                     "orig-prefix.txt"
                 )
 
@@ -771,6 +776,5 @@ def main():
             else:
                 executeMain(
                     binary_filename = getResultFullpath(main_module),
-                    tree            = main_module,
                     clean_path      = Options.shallClearPythonPathEnvironment()
                 )
