@@ -1,4 +1,4 @@
-#     Copyright 2015, Kay Hayen, mailto:kay.hayen@gmail.com
+#     Copyright 2016, Kay Hayen, mailto:kay.hayen@gmail.com
 #
 #     Part of "Nuitka", an optimizing Python compiler that is compatible and
 #     integrates with CPython, but also works on its own.
@@ -18,8 +18,8 @@
 """ Options module """
 
 version_string = """\
-Nuitka V0.5.14
-Copyright (C) 2015 Kay Hayen."""
+Nuitka V0.5.18.1
+Copyright (C) 2016 Kay Hayen."""
 
 import logging
 import sys
@@ -90,7 +90,8 @@ parser.add_option(
     default = True,
     help    = """\
 In standalone mode by default all modules of standard library will be frozen
-as bytecode. As a result compilation time will increase very much.
+as bytecode. This compiles them all and as a result compilation time will
+increase very much.
 """,
     )
 
@@ -243,6 +244,16 @@ Defaults to %s.""" %
        ("on" if is_nuitka_run else "off")
 )
 
+execute_group.add_option(
+    "--debugger", "--gdb",
+    action  = "store_true",
+    dest    = "debugger",
+    default = False,
+    help    = """\
+Execute inside "gdb" to automatically get a stack trace.
+Defaults to off."""
+)
+
 execute_group.add_option(
     "--execute-with-pythonpath", "--keep-pythonpath",
     action  = "store_true",
@@ -397,6 +408,24 @@ Keep debug info in the resulting object file for better debugger interaction.
 Defaults to off."""
 )
 
+debug_group.add_option(
+    "--profile",
+    action  = "store_true",
+    dest    = "profile",
+    default = False,
+    help    = """\
+Enable vmprof based profiling of time spent. Defaults to off."""
+)
+
+debug_group.add_option(
+    "--graph",
+    action  = "store_true",
+    dest    = "graph",
+    default = False,
+    help    = """\
+Create graph of optimization process. Defaults to off."""
+)
+
 debug_group.add_option(
     "--trace-execution",
     action  = "store_true",
@@ -597,6 +626,45 @@ windows_group.add_option(
 
 parser.add_option_group(windows_group)
 
+plugin_group = OptionGroup(
+    parser,
+    "Plugin control"
+)
+
+plugin_group.add_option(
+    "--plugin-enable", "--enable-plugin",
+    action  = "append",
+    dest    = "plugins_enabled",
+    default = [],
+    help    = """\
+Enabled plugins. Must be plug-in names. Use --plugin-list to query the
+full list and exit. Default empty."""
+)
+
+plugin_group.add_option(
+    "--plugin-disable", "--disable-plugin",
+    action  = "append",
+    dest    = "plugins_disabled",
+    default = [],
+    help    = """\
+Disabled plugins. Must be plug-in names. Use --plugin-list to query the
+full list and exit. Default empty."""
+)
+
+plugin_group.add_option(
+    "--plugin-no-detection",
+    action  = "store_false",
+    dest    = "detect_missing_plugins",
+    default = True,
+    help    = """\
+Plugins can detect if they might be used, and the you can disable the warning
+via --plugin-disable=plugin-that-warned, or you can use this option to disable
+the mechanism entirely, which also speeds up compilation slightly of course as
+this detection code is run in vain once you are certain of which plug-ins to
+use. Defaults to off."""
+)
+
+parser.add_option_group(plugin_group)
 
 # First, isolate the first non-option arguments.
 if is_nuitka_run:
@@ -628,6 +696,12 @@ if not positional_args:
     sys.exit("""
 Error, need positional argument with python module or main program.""")
 
+if not options.immediate_execution and len(positional_args) > 1:
+    parser.print_help()
+
+    sys.exit("""
+Error, need only one positional argument unless --run is specified.""")
+
 if options.verbose:
     logging.getLogger().setLevel(logging.DEBUG)
 else:
@@ -639,7 +713,6 @@ else:
 if options.is_standalone:
     options.executable = True
     options.recurse_all = True
-    options.recurse_stdlib = True
 
 def shallTraceExecution():
     return options.trace_execution
@@ -647,6 +720,9 @@ def shallTraceExecution():
 def shallExecuteImmediately():
     return options.immediate_execution
 
+def shallRunInDebugger():
+    return options.debugger
+
 def shallDumpBuiltTreeXML():
     return options.dump_xml
 
@@ -736,7 +812,7 @@ def shallWarnImplicitRaises():
     return options.warn_implicit_exceptions
 
 def isDebug():
-    return options.debug
+    return options.debug or options.debugger
 
 def isPythonDebug():
     return options.python_debug or sys.flags.debug
@@ -745,7 +821,13 @@ def isOptimize():
     return not options.no_optimize
 
 def isUnstripped():
-    return options.unstripped
+    return options.unstripped or options.profile
+
+def isProfile():
+    return options.profile
+
+def shouldCreateGraph():
+    return options.graph
 
 def getOutputPath(path):
     if options.output_dir:
@@ -836,5 +918,18 @@ def getPythonFlags():
 
     return result
 
-def freezeAllStdlib():
+def shallFreezeAllStdlib():
     return options.freeze_stdlib
+
+def getPluginsEnabled():
+    return options.plugins_enabled
+
+def getPluginsDisabled():
+    return options.plugins_disabled
+
+def shallDetectMissingPlugins():
+    return options.detect_missing_plugins
+
+def getPluginOptions(plugin_name):
+    # TODO: This should come from command line, pylint: disable=W0613
+    return {}
