@@ -1,4 +1,4 @@
-#     Copyright 2015, Kay Hayen, mailto:kay.hayen@gmail.com
+#     Copyright 2016, Kay Hayen, mailto:kay.hayen@gmail.com
 #
 #     Part of "Nuitka", an optimizing Python compiler that is compatible and
 #     integrates with CPython, but also works on its own.
@@ -42,11 +42,11 @@ from nuitka.Constants import (
     isMutable
 )
 
-from . import CodeTemplates
 from .BlobCodes import StreamData
 from .Emission import SourceCodeCollector
 from .Indentation import indented
 from .Pickling import getStreamedConstant
+from .templates.CodeTemplatesConstants import template_constants_reading
 
 
 def generateConstantReferenceCode(to_name, expression, emit, context):
@@ -328,7 +328,7 @@ def __addConstantInitCode(context, emit, check, constant_type, constant_value,
         constants belong into the same scope.
     """
     # This has many cases, that all return, and do a lot.
-    # pylint: disable=R0911,R0912,R0915,R0914
+    # pylint: disable=R0911,R0912,R0914,R0915
 
     # For the module level, we only mean to create constants that are used only
     # inside of it. For the global level, it must must be single use.
@@ -372,7 +372,7 @@ def __addConstantInitCode(context, emit, check, constant_type, constant_value,
                 """\
 %s = PyLong_FromLong( %sl ); // To be corrected with -1 in-place next lines.
 CHECK_OBJECT( const_int_pos_1 );
-%s = PyNumber_InPlaceSubtract( %s, const_int_pos_1 );""" % (
+%s = PyNumber_InPlaceSubtract( %s, PyLong_FromLong( 1 ) );""" % (
                     constant_identifier,
                     min_signed_long,
                     constant_identifier,
@@ -404,7 +404,7 @@ CHECK_OBJECT( const_int_pos_1 );
             emit(
                 """\
 %s = PyInt_FromLong( %sl );  // To be corrected in next line.
-%s = PyNumber_InPlaceSubtract( %s, const_int_pos_1 );""" % (
+%s = PyNumber_InPlaceSubtract( %s, PyInt_FromLong( 1 ) );""" % (
                     constant_identifier,
                     min_signed_long,
                     constant_identifier,
@@ -1012,7 +1012,7 @@ def getConstantsDefinitionCode(context):
         context = context
     )
 
-    return CodeTemplates.template_constants_reading % {
+    return template_constants_reading % {
         "constant_declarations" : '\n'.join(constant_declarations),
         "constant_inits"        : indented(constant_inits),
         "constant_checks"       : indented(constant_checks)
