@@ -1,4 +1,4 @@
-#     Copyright 2014, Kay Hayen, mailto:kay.hayen@gmail.com
+#     Copyright 2016, Kay Hayen, mailto:kay.hayen@gmail.com
 #
 #     Part of "Nuitka", an optimizing Python compiler that is compatible and
 #     integrates with CPython, but also works on its own.
@@ -24,7 +24,7 @@ import hashlib
 from nuitka import Options
 from nuitka.__past__ import iterItems
 from nuitka.Constants import constant_builtin_types
-from nuitka.Utils import python_version
+from nuitka.PythonVersions import python_version
 
 from .Namify import namifyConstant
 
@@ -32,6 +32,9 @@ from .Namify import namifyConstant
 # Many methods won't use self, but it's the interface. pylint: disable=R0201
 
 class TempMixin:
+    # Lots of details, everything gets to store bits here, to indicate
+    # code generation states, and there are many, pylint: disable=R0902
+
     def __init__(self):
         self.tmp_names = {}
         self.tmp_types = {}
@@ -42,20 +45,17 @@ class TempMixin:
         # For exception handling
         self.needs_exception_variables = False
         self.exception_escape = None
-        self.exception_ok = None
         self.loop_continue = None
         self.loop_break = None
-        # Python3 frame exception stack
-        self.frame_preservation_stack = []
-
-        # For exception handlers visibility of caught exception
-        self.exception_published = True
 
         # For branches
         self.true_target = None
         self.false_target = None
 
         self.keeper_variable_count = 0
+        self.exception_keepers = (None, None, None, None)
+
+        self.preserver_variable_counts = set()
 
     def formatTempName(self, base_name, number):
         if number is None:
@@ -68,23 +68,25 @@ class TempMixin:
                 number = number
             )
 
-    def allocateTempName(self, base_name, type_name, unique):
+    def allocateTempName(self, base_name, type_name = "PyObject *",
+                         unique = False):
         if unique:
-            result = None
+            number = None
         else:
-            result = self.tmp_names.get(base_name, 0)
-            result += 1
+            number = self.tmp_names.get(base_name, 0)
+            number += 1
 
-        self.tmp_names[base_name] = result
+        self.tmp_names[base_name] = number
 
-        if result == 1 or result == None:
+        if base_name not in self.tmp_types:
             self.tmp_types[base_name] = type_name
         else:
-            assert self.tmp_types[base_name] == type_name, type_name
+            assert self.tmp_types[base_name] == type_name, \
+                (self.tmp_types[base_name], type_name)
 
         return self.formatTempName(
             base_name = base_name,
-            number    = result
+            number    = number
         )
 
     def hasTempName(self, base_name):
@@ -131,37 +133,25 @@ class TempMixin:
         return self.exception_escape
 
     def setExceptionEscape(self, label):
+        result = self.exception_escape
         self.exception_escape = label
-
-    def getExceptionNotOccured(self):
-        return self.exception_ok
-
-    def setExceptionNotOccured(self, label):
-        self.exception_ok = label
-
-    def isExceptionPublished(self):
-        return self.exception_published
-
-    def setExceptionPublished(self, value):
-        self.exception_published = value
+        return result
 
     def getLoopBreakTarget(self):
         return self.loop_break
 
-    def setLoopBreakTarget(self, label, name = None):
-        if name is None:
-            self.loop_break = label
-        else:
-            self.loop_break = label, name
+    def setLoopBreakTarget(self, label):
+        result = self.loop_break
+        self.loop_break = label
+        return result
 
     def getLoopContinueTarget(self):
         return self.loop_continue
 
-    def setLoopContinueTarget(self, label, name = None):
-        if name is None:
-            self.loop_continue = label
-        else:
-            self.loop_continue = label, name
+    def setLoopContinueTarget(self, label):
+        result = self.loop_continue
+        self.loop_continue = label
+        return result
 
     def allocateLabel(self, label):
         result = self.labels.get(label, 0)
@@ -179,18 +169,34 @@ class TempMixin:
     def markAsNeedsExceptionVariables(self):
         self.needs_exception_variables = True
 
-    def getExceptionKeeperVariables(self):
+    def allocateExceptionKeeperVariables(self):
         self.keeper_variable_count += 1
 
         return (
             "exception_keeper_type_%d" % self.keeper_variable_count,
             "exception_keeper_value_%d" % self.keeper_variable_count,
-            "exception_keeper_tb_%d" % self.keeper_variable_count
+            "exception_keeper_tb_%d" % self.keeper_variable_count,
+            "exception_keeper_lineno_%s" % self.keeper_variable_count
         )
 
     def getKeeperVariableCount(self):
         return self.keeper_variable_count
 
+    def getExceptionKeeperVariables(self):
+        return self.exception_keepers
+
+    def setExceptionKeeperVariables(self, keeper_vars):
+        result = self.exception_keepers
+        self.exception_keepers = tuple(keeper_vars)
+        return result
+
+    def getExceptionPreserverCounts(self):
+        return self.preserver_variable_counts
+
+    def addExceptionPreserverVariables(self, count):
+        assert count != 0
+        self.preserver_variable_counts.add(count)
+
     def getTrueBranchTarget(self):
         return self.true_target
 
@@ -203,23 +209,6 @@ class TempMixin:
     def setFalseBranchTarget(self, label):
         self.false_target = label
 
-    def pushFrameExceptionPreservationDepth(self):
-        if self.frame_preservation_stack:
-            self.frame_preservation_stack.append(
-                self.getExceptionKeeperVariables()
-            )
-        else:
-            self.frame_preservation_stack.append(
-                None
-            )
-
-        return self.frame_preservation_stack[-1]
-
-    def popFrameExceptionPreservationDepth(self):
-        result = self.frame_preservation_stack[-1]
-        del self.frame_preservation_stack[-1]
-        return result
-
 
 class CodeObjectsMixin:
     def __init__(self):
@@ -229,28 +218,22 @@ class CodeObjectsMixin:
     def getCodeObjects(self):
         return sorted(iterItems(self.code_objects))
 
-    # Sad but true, code objects have these many details that actually are fed
-    # from all different sources, pylint: disable=R0913
-    def getCodeObjectHandle(self, filename, code_name, line_number, var_names,
-                            arg_count, kw_only_count, is_generator,
-                            is_optimized, has_starlist, has_stardict,
-                            has_closure, future_flags):
-        var_names = tuple(var_names)
-
-        assert type(has_starlist) is bool
-        assert type(has_stardict) is bool
+    def getCodeObjectHandle(self, code_object, filename, line_number,
+                            is_optimized, new_locals, has_closure,
+                            future_flags):
 
         key = (
             filename,
-            code_name,
+            code_object.getCodeObjectName(),
             line_number,
-            var_names,
-            arg_count,
-            kw_only_count,
-            is_generator,
+            code_object.getVarNames(),
+            code_object.getArgumentCount(),
+            code_object.getKwOnlyParameterCount(),
+            code_object.getKind(),
             is_optimized,
-            has_starlist,
-            has_stardict,
+            new_locals,
+            code_object.hasStarListArg(),
+            code_object.hasStarDictArg(),
             has_closure,
             future_flags
         )
@@ -260,44 +243,40 @@ class CodeObjectsMixin:
 
         return self.code_objects[key]
 
-
-    # False alarms about "hashlib.md5" due to its strange way of defining what
-    # is exported, PyLint won't understand it. pylint: disable=E1101
     if python_version < 300:
         def _calcHash(self, key):
             hash_value = hashlib.md5(
-                "%s%s%d%s%d%d%s%s%s%s%s%s" % key
+                "%s%s%d%s%d%d%s%s%s%s%s%s%s" % key
             )
 
             return hash_value.hexdigest()
     else:
         def _calcHash(self, key):
             hash_value = hashlib.md5(
-                ("%s%s%d%s%d%d%s%s%s%s%s%s" % key).encode("utf-8")
+                ("%s%s%d%s%d%d%s%s%s%s%s%s%s" % key).encode("utf-8")
             )
 
             return hash_value.hexdigest()
 
 
-
 class PythonContextBase:
     def __init__(self):
         self.temp_counts = {}
 
         self.source_ref = None
 
-    def isPythonModule(self):
+    def isCompiledPythonModule(self):
         return False
 
     def allocateTempNumber(self, tmp_scope):
-        result = self.temp_counts.get( tmp_scope, 0 ) + 1
+        result = self.temp_counts.get(tmp_scope, 0) + 1
         self.temp_counts[ tmp_scope ] = result
         return result
 
 
 class PythonChildContextBase(PythonContextBase):
     def __init__(self, parent):
-        PythonContextBase.__init__( self )
+        PythonContextBase.__init__(self)
 
         self.parent = parent
 
@@ -311,14 +290,21 @@ class PythonChildContextBase(PythonContextBase):
         return self.parent.getModuleName()
 
     def addHelperCode(self, key, code):
-        self.parent.addHelperCode( key, code )
+        return self.parent.addHelperCode(key, code)
+
+    def hasHelperCode(self, key):
+        return self.parent.hasHelperCode(key)
 
     def addDeclaration(self, key, code):
-        self.parent.addDeclaration( key, code )
+        self.parent.addDeclaration(key, code)
+
 
 
 def _getConstantDefaultPopulation():
-    result = (
+    # Note: Can't work with set here, because we need to put in some values that
+    # cannot be hashed.
+
+    result = [
         # Basic values that the helper code uses all the times.
         (),
         {},
@@ -345,54 +331,60 @@ def _getConstantDefaultPopulation():
         "__builtins__",
         "__all__",
         "__cmp__",
+        "__iter__",
 
         # Patched module name.
         "inspect",
 
-        # Names of builtins used in helper code.
+        # Names of built-ins used in helper code.
         "compile",
         "range",
         "open",
-        "__import__",
-    )
+        "__import__"
+    ]
 
-    # For Python3 modules
     if python_version >= 300:
-        result += (
-            "__cached__",
+        # For Python3 modules
+        result.append(
+            "__cached__"
         )
 
-    # For Python3 print
-    if python_version >= 300:
+        # For Python3 print
         result += (
             "print",
             "end",
-            "file",
+            "file"
+        )
+
+    if python_version >= 330:
+        # Modules have that attribute starting with 3.3
+        result.append(
+            "__loader__"
+        )
+
+    if python_version >= 340:
+        result.append(
+            # YIELD_FROM uses this starting 3.4, with 3.3 other code is used.
+            "send"
         )
 
     if python_version >= 330:
-        result += (
-            # Modules have that attribute.
-            "__loader__",
-        )
-
-    if python_version >= 340:
         result += (
             # YIELD_FROM uses this
-            "send",
+            "throw",
+            "close",
         )
 
 
-    # For patching Python2 internal class type
     if python_version < 300:
+        # For patching Python2 internal class type
         result += (
             "__getattr__",
             "__setattr__",
             "__delattr__",
         )
 
-    # For patching Python2 sys attributes for current exception
-    if python_version < 300:
+        # For patching Python2 "sys" attributes for current exception
         result += (
             "exc_type",
             "exc_value",
@@ -401,41 +393,47 @@ def _getConstantDefaultPopulation():
 
     # The xrange built-in is Python2 only.
     if python_version < 300:
-        result += (
-            "xrange",
+        result.append(
+            "xrange"
         )
 
     # Executables only
     if not Options.shallMakeModule():
-        result += (
-            "__main__",
+        result.append(
+            "__main__"
         )
 
         # The "site" module is referenced in inspect patching.
-        result += (
-            "site",
+        result.append(
+            "site"
         )
 
-    # Builtin original values
+    # Built-in original values
     if not Options.shallMakeModule():
-        result += (
+        result += [
             "type",
             "len",
             "range",
             "repr",
             "int",
             "iter",
-        )
+        ]
 
         if python_version < 300:
-            result += (
+            result.append(
                 "long",
             )
 
     # Disabling warnings at startup
     if "no_warnings" in Options.getPythonFlags():
-        result += (
-            "ignore",
+        result.append(
+            "ignore"
+        )
+
+    if python_version >= 350:
+        # Patching the types module.
+        result.append(
+            "types"
         )
 
     return result
@@ -500,23 +498,33 @@ class PythonGlobalContext:
         return self.constants
 
 
+class FrameDeclarationsMixin:
+    def __init__(self):
+        self.frame_declarations = []
 
-class PythonModuleContext(PythonContextBase, TempMixin, CodeObjectsMixin):
-    # Plent of attributes, because it's storing so many different things.
+    def addFrameDeclaration(self, frame_decl):
+        self.frame_declarations.append(frame_decl)
+
+    def getFrameDeclarations(self):
+        return self.frame_declarations
+
+
+class PythonModuleContext(PythonContextBase, TempMixin, CodeObjectsMixin,
+                          FrameDeclarationsMixin):
+    # Plenty of attributes, because it's storing so many different things.
     # pylint: disable=R0902
 
-    def __init__(self, module, module_name, code_name, filename, is_empty,
-                global_context):
+    def __init__(self, module, module_name, code_name, filename, global_context):
         PythonContextBase.__init__(self)
 
         TempMixin.__init__(self)
         CodeObjectsMixin.__init__(self)
+        FrameDeclarationsMixin.__init__(self)
 
         self.module = module
         self.name = module_name
         self.code_name = code_name
         self.filename = filename
-        self.is_empty = is_empty
 
         self.global_context = global_context
 
@@ -525,15 +533,23 @@ class PythonModuleContext(PythonContextBase, TempMixin, CodeObjectsMixin):
 
         self.constants = set()
 
+        self.return_release_mode = False
+
         self.frame_handle = None
 
+        self.return_exit = True
+
+        self.return_name = None
+
+        self.needs_module_filename_object = False
+
     def __repr__(self):
         return "<PythonModuleContext instance for module %s>" % self.filename
 
     def getOwner(self):
         return self.module
 
-    def isPythonModule(self):
+    def isCompiledPythonModule(self):
         return True
 
     def hasLocalsDict(self):
@@ -551,8 +567,10 @@ class PythonModuleContext(PythonContextBase, TempMixin, CodeObjectsMixin):
     def getFilename(self):
         return self.filename
 
-    def isEmptyModule(self):
-        return self.is_empty
+    def mayRaiseException(self):
+        body = self.module.getBody()
+
+        return body is not None and body.mayRaiseException(BaseException)
 
     getModuleName = getName
 
@@ -561,9 +579,6 @@ class PythonModuleContext(PythonContextBase, TempMixin, CodeObjectsMixin):
 
     # There cannot be local variable in modules no need to consider the name.
     # pylint: disable=W0613
-    def hasLocalVariable(self, var_name):
-        return False
-
     def hasClosureVariable(self, var_name):
         return False
     # pylint: enable=W0613
@@ -576,6 +591,9 @@ class PythonModuleContext(PythonContextBase, TempMixin, CodeObjectsMixin):
 
         self.helper_codes[ key ] = code
 
+    def hasHelperCode(self, key):
+        return key in self.helper_codes
+
     def getHelperCodes(self):
         return self.helper_codes
 
@@ -587,17 +605,29 @@ class PythonModuleContext(PythonContextBase, TempMixin, CodeObjectsMixin):
     def getDeclarations(self):
         return self.declaration_codes
 
+    def getReturnValueName(self):
+        return self.return_name
+
+    def setReturnValueName(self, value):
+        result = self.return_name
+        self.return_name = value
+        return result
+
     def setReturnReleaseMode(self, value):
-        pass
+        result = self.return_release_mode
+        self.return_release_mode = value
+        return result
 
     def getReturnReleaseMode(self):
-        pass
+        return self.return_release_mode
 
     def getReturnTarget(self):
-        return None
+        return self.return_exit
 
     def setReturnTarget(self, label):
-        pass
+        result = self.return_exit
+        self.return_exit = label
+        return result
 
     def mayRecurse(self):
         return False
@@ -614,8 +644,15 @@ class PythonModuleContext(PythonContextBase, TempMixin, CodeObjectsMixin):
     def getConstants(self):
         return self.constants
 
+    def markAsNeedsModuleFilenameObject(self):
+        self.needs_module_filename_object = True
 
-class PythonFunctionContext(PythonChildContextBase, TempMixin):
+    def needsModuleFilenameObject(self):
+        return self.needs_module_filename_object
+
+
+class PythonFunctionContext(PythonChildContextBase, TempMixin,
+                            FrameDeclarationsMixin):
     def __init__(self, parent, function):
         PythonChildContextBase.__init__(
             self,
@@ -623,6 +660,7 @@ class PythonFunctionContext(PythonChildContextBase, TempMixin):
         )
 
         TempMixin.__init__(self)
+        FrameDeclarationsMixin.__init__(self)
 
         self.function = function
 
@@ -632,12 +670,13 @@ class PythonFunctionContext(PythonChildContextBase, TempMixin):
         self.setReturnTarget("function_return_exit")
 
         self.return_release_mode = False
+        self.return_name = None
 
         self.frame_handle = None
 
     def __repr__(self):
         return "<PythonFunctionContext for %s '%s'>" % (
-            "function" if not self.function.isClassDictCreation() else "class",
+            "function" if not self.function.isExpressionClassBody() else "class",
             self.function.getName()
         )
 
@@ -650,9 +689,6 @@ class PythonFunctionContext(PythonChildContextBase, TempMixin):
     def hasLocalsDict(self):
         return self.function.hasLocalsDict()
 
-    def hasLocalVariable(self, var_name):
-        return var_name in self.function.getLocalVariableNames()
-
     def hasClosureVariable(self, var_name):
         return var_name in self.function.getClosureVariableNames()
 
@@ -662,20 +698,35 @@ class PythonFunctionContext(PythonChildContextBase, TempMixin):
     def setFrameHandle(self, frame_handle):
         self.frame_handle = frame_handle
 
+    def getReturnValueName(self):
+        if self.return_name is None:
+            self.return_name = self.allocateTempName("return_value", unique = True)
+
+        return self.return_name
+
+    def setReturnValueName(self, value):
+        result = self.return_name
+        self.return_name = value
+        return result
+
     def getReturnTarget(self):
         return self.return_exit
 
     def setReturnTarget(self, label):
+        result = self.return_exit
         self.return_exit = label
+        return result
 
     def setReturnReleaseMode(self, value):
+        result = self.return_release_mode
         self.return_release_mode = value
+        return result
 
     def getReturnReleaseMode(self):
         return self.return_release_mode
 
     def mayRecurse(self):
-        # TODO: Determine this at compile time.
+        # TODO: Determine this at compile time for enhanced optimizations.
         return True
 
     def getCodeObjectHandle(self, **kw):
@@ -692,6 +743,20 @@ class PythonFunctionDirectContext(PythonFunctionContext):
     def isForCreatedFunction(self):
         return False
 
+class PythonGeneratorObjectContext(PythonFunctionContext):
+    def isForDirectCall(self):
+        return False
+
+    def isForCrossModuleUsage(self):
+        return self.function.isCrossModuleUsed()
+
+    def isForCreatedFunction(self):
+        return False
+
+
+class PythonCoroutineObjectContext(PythonGeneratorObjectContext):
+    pass
+
 
 class PythonFunctionCreatedContext(PythonFunctionContext):
     def isForDirectCall(self):
@@ -716,8 +781,8 @@ class PythonStatementCContext(PythonChildContextBase):
     def getOwner(self):
         return self.parent.getOwner()
 
-    def isPythonModule(self):
-        return self.parent.isPythonModule()
+    def isCompiledPythonModule(self):
+        return self.parent.isCompiledPythonModule()
 
     def getFunction(self):
         return self.parent.getFunction()
@@ -739,7 +804,10 @@ class PythonStatementCContext(PythonChildContextBase):
         return self.allocateTempName("result", "bool", unique = True)
 
     def getReturnValueName(self):
-        return self.allocateTempName("return_value", unique = True)
+        return self.parent.getReturnValueName()
+
+    def setReturnValueName(self, value):
+        return self.parent.setReturnValueName(value)
 
     def getGeneratorReturnValueName(self):
         if python_version >= 330:
@@ -759,55 +827,43 @@ class PythonStatementCContext(PythonChildContextBase):
         return self.parent.getExceptionEscape()
 
     def setExceptionEscape(self, label):
-        self.parent.setExceptionEscape(label)
-
-    def getExceptionNotOccured(self):
-        return self.parent.getExceptionNotOccured()
-
-    def setExceptionNotOccured(self, label):
-        self.parent.setExceptionNotOccured(label)
-
-    def isExceptionPublished(self):
-        return self.parent.isExceptionPublished()
-
-    def setExceptionPublished(self, value):
-        self.parent.setExceptionPublished(value)
+        return self.parent.setExceptionEscape(label)
 
     def getLoopBreakTarget(self):
         return self.parent.getLoopBreakTarget()
 
-    def setLoopBreakTarget(self, label, name = None):
-        self.parent.setLoopBreakTarget(label, name)
+    def setLoopBreakTarget(self, label):
+        return self.parent.setLoopBreakTarget(label)
 
     def getLoopContinueTarget(self):
         return self.parent.getLoopContinueTarget()
 
-    def setLoopContinueTarget(self, label, name = None):
-        self.parent.setLoopContinueTarget(label, name)
+    def setLoopContinueTarget(self, label):
+        return self.parent.setLoopContinueTarget(label)
 
     def getTrueBranchTarget(self):
         return self.parent.getTrueBranchTarget()
 
     def setTrueBranchTarget(self, label):
-        self.parent.setTrueBranchTarget(label)
+        return self.parent.setTrueBranchTarget(label)
 
     def getFalseBranchTarget(self):
         return self.parent.getFalseBranchTarget()
 
     def setFalseBranchTarget(self, label):
-        self.parent.setFalseBranchTarget(label)
+        return self.parent.setFalseBranchTarget(label)
 
     def getReturnTarget(self):
         return self.parent.getReturnTarget()
 
     def setReturnTarget(self, label):
-        self.parent.setReturnTarget(label)
+        return self.parent.setReturnTarget(label)
 
     def getReturnReleaseMode(self):
         return self.parent.getReturnReleaseMode()
 
     def setReturnReleaseMode(self, value):
-        self.parent.setReturnReleaseMode(value)
+        return self.parent.setReturnReleaseMode(value)
 
     def allocateLabel(self, label):
         return self.parent.allocateLabel(label)
@@ -831,7 +887,7 @@ class PythonStatementCContext(PythonChildContextBase):
             return True
 
     def forgetTempName(self, tmp_name):
-        return self.parent.forgetTempName(tmp_name)
+        self.parent.forgetTempName(tmp_name)
 
     def getCleanupTempnames(self):
         return self.cleanup_names
@@ -842,24 +898,30 @@ class PythonStatementCContext(PythonChildContextBase):
     def setFrameHandle(self, frame_handle):
         return self.parent.setFrameHandle(frame_handle)
 
+    def allocateExceptionKeeperVariables(self):
+        return self.parent.allocateExceptionKeeperVariables()
+
     def getExceptionKeeperVariables(self):
         return self.parent.getExceptionKeeperVariables()
 
+    def setExceptionKeeperVariables(self, keeper_vars):
+        return self.parent.setExceptionKeeperVariables(keeper_vars)
+
+    def addExceptionPreserverVariables(self, count):
+        self.parent.addExceptionPreserverVariables(count)
+
     def needsExceptionVariables(self):
         return self.parent.needsExceptionVariables()
 
     def markAsNeedsExceptionVariables(self):
         self.parent.markAsNeedsExceptionVariables()
 
+    def addFrameDeclaration(self, frame_decl):
+        self.parent.addFrameDeclaration(frame_decl)
+
     def mayRecurse(self):
         return self.parent.mayRecurse()
 
-    def pushFrameExceptionPreservationDepth(self):
-        return self.parent.pushFrameExceptionPreservationDepth()
-
-    def popFrameExceptionPreservationDepth(self):
-        return self.parent.popFrameExceptionPreservationDepth()
-
     def getCodeObjectHandle(self, **kw):
         return self.parent.getCodeObjectHandle(**kw)
 
@@ -879,3 +941,6 @@ class PythonStatementCContext(PythonChildContextBase):
         result = self.last_source_ref
         # self.last_source_ref = None
         return result
+
+    def markAsNeedsModuleFilenameObject(self):
+        self.parent.markAsNeedsModuleFilenameObject()
