@@ -1,4 +1,4 @@
-#     Copyright 2015, Kay Hayen, mailto:kay.hayen@gmail.com
+#     Copyright 2016, Kay Hayen, mailto:kay.hayen@gmail.com
 #
 #     Part of "Nuitka", an optimizing Python compiler that is compatible and
 #     integrates with CPython, but also works on its own.
@@ -26,10 +26,13 @@ from .ErrorCodes import (
     getReleaseCode,
     getReleaseCodes
 )
-from .Helpers import generateChildExpressionsCode
+from .Helpers import generateChildExpressionsCode, generateExpressionCode
+from .PythonAPICodes import generateCAPIObjectCode
 
 
-def generateListCreationCode(to_name, elements, emit, context):
+def generateListCreationCode(to_name, expression, emit, context):
+    elements = expression.getElements()
+
     emit(
         "%s = PyList_New( %d );" % (
             to_name,
@@ -37,8 +40,6 @@ def generateListCreationCode(to_name, elements, emit, context):
         )
     )
 
-    from .CodeGeneration import generateExpressionCode
-
     context.addCleanupTempName(to_name)
 
     element_name = context.allocateTempName("list_element")
@@ -64,6 +65,7 @@ def generateListCreationCode(to_name, elements, emit, context):
             )
         )
 
+
 def generateListOperationAppendCode(to_name, expression, emit, context):
     res_name = context.getIntResName()
 
@@ -74,6 +76,7 @@ def generateListOperationAppendCode(to_name, expression, emit, context):
     )
 
 
+    emit("assert( PyList_Check( %s ) );" % list_arg_name)
     emit(
         "%s = PyList_Append( %s, %s );" % (
             res_name,
@@ -103,6 +106,37 @@ def generateListOperationAppendCode(to_name, expression, emit, context):
         context.forgetTempName(to_name)
 
 
+def generateListOperationExtendCode(to_name, expression, emit, context):
+    list_arg_name, value_arg_name = generateChildExpressionsCode(
+        expression = expression,
+        emit       = emit,
+        context    = context
+    )
+
+    emit("assert( PyList_Check( %s ) );" % list_arg_name)
+    emit(
+        "%s = _PyList_Extend( (PyListObject *)%s, %s );" % (
+            to_name,
+            list_arg_name,
+            value_arg_name
+        )
+    )
+
+    getReleaseCodes(
+        release_names = (list_arg_name, value_arg_name),
+        emit          = emit,
+        context       = context
+    )
+
+    getErrorExitCode(
+        check_name = to_name,
+        emit       = emit,
+        context    = context
+    )
+
+    context.addCleanupTempName(to_name)
+
+
 def generateListOperationPopCode(to_name, expression, emit, context):
     list_arg_name, = generateChildExpressionsCode(
         expression = expression,
@@ -110,8 +144,8 @@ def generateListOperationPopCode(to_name, expression, emit, context):
         context    = context
     )
 
-
     # TODO: Have a dedicated helper instead, this could be more efficient.
+    emit("assert( PyList_Check( %s ) );" % list_arg_name)
     emit(
         '%s = PyObject_CallMethod(  %s, (char *)"pop", NULL );' % (
             to_name,
@@ -132,3 +166,17 @@ def generateListOperationPopCode(to_name, expression, emit, context):
     )
 
     context.addCleanupTempName(to_name)
+
+
+def generateBuiltinListCode(to_name, expression, emit, context):
+    generateCAPIObjectCode(
+        to_name    = to_name,
+        capi       = "PySequence_List",
+        arg_desc   = (
+            ("list_arg", expression.getValue()),
+        ),
+        may_raise  = expression.mayRaiseException(BaseException),
+        source_ref = expression.getCompatibleSourceReference(),
+        emit       = emit,
+        context    = context
+    )
