@@ -1,4 +1,4 @@
-#     Copyright 2014, Kay Hayen, mailto:kay.hayen@gmail.com
+#     Copyright 2016, Kay Hayen, mailto:kay.hayen@gmail.com
 #
 #     Part of "Nuitka", an optimizing Python compiler that is compatible and
 #     integrates with CPython, but also works on its own.
@@ -21,6 +21,8 @@ These are mostly used to look up the Python C/API from operations or a wrapper u
 
 """
 
+from nuitka.PythonVersions import python_version
+
 binary_operator_codes = {
 # Those commented out in this section have fully specialized variants already.
 
@@ -29,8 +31,8 @@ binary_operator_codes = {
 #    "Div"       : "PyNumber_Divide",
 #    "Mult"      : "PyNumber_Multiply",
 #    "Mod"       : "PyNumber_Remainder",
-# These have their own variants only to make sure the generic code is inlined
-# but the CPython code is not inlined.
+# These have their own variants only to make sure the generic code is in-lined
+# but the CPython code is not in-lined.
 
 #    "Pow"       : "PyNumber_Power",
 #    "IPow"      : "PyNumber_InPlacePower",
@@ -57,12 +59,17 @@ binary_operator_codes = {
     "IBitXor"   : "PyNumber_InPlaceXor",
 }
 
+# Python 3.5 only operator
+if python_version >= 350:
+    binary_operator_codes["MatMult"] = "PyNumber_MatrixMultiply"
+    binary_operator_codes["IMatMult"] = "PyNumber_InPlaceMatrixMultiply"
+
 unary_operator_codes = {
-    "UAdd"   : ( "PyNumber_Positive", 1 ),
-    "USub"   : ( "PyNumber_Negative", 1 ),
-    "Invert" : ( "PyNumber_Invert", 1 ),
-    "Repr"   : ( "PyObject_Repr", 1 ),
-    "Not"    : ( "UNARY_NOT", 0 )
+    "UAdd"   : ("PyNumber_Positive", 1),
+    "USub"   : ("PyNumber_Negative", 1),
+    "Invert" : ("PyNumber_Invert", 1),
+    "Repr"   : ("PyObject_Repr", 1),
+    "Not"    : ("UNARY_NOT", 0)
 }
 
 rich_comparison_codes = {
