@@ -1,4 +1,4 @@
-#     Copyright 2015, Kay Hayen, mailto:kay.hayen@gmail.com
+#     Copyright 2016, Kay Hayen, mailto:kay.hayen@gmail.com
 #
 #     Part of "Nuitka", an optimizing Python compiler that is compatible and
 #     integrates with CPython, but also works on its own.
@@ -15,7 +15,7 @@
 #     See the License for the specific language governing permissions and
 #     limitations under the License.
 #
-""" Normal function (no yield) related templates.
+""" Normal function (no generator, not yielding) related templates.
 
 """
 
@@ -27,15 +27,19 @@ template_function_direct_declaration = """\
 %(file_scope)s PyObject *impl_%(function_identifier)s( %(direct_call_arg_spec)s );
 """
 
-make_function_with_context_template = """
+template_function_closure_making = """\
+    PyCellObject **closure = (PyCellObject **)malloc(%(closure_count)d * sizeof(PyCellObject *));
+%(closure_copy)s
+"""
+
+template_make_function_with_context_template = """
 static PyObject *MAKE_FUNCTION_%(function_identifier)s( %(function_creation_args)s )
 {
     // Copy the parameter default values and closure values over.
-%(context_copy)s
+%(closure_making)s
 
     PyObject *result = Nuitka_Function_New(
-        %(fparse_function_identifier)s,
-        %(dparse_function_identifier)s,
+        %(function_impl_identifier)s,
         %(function_name_obj)s,
 #if PYTHON_VERSION >= 330
         %(function_qualname_obj)s,
@@ -56,12 +60,11 @@ static PyObject *MAKE_FUNCTION_%(function_identifier)s( %(function_creation_args
 }
 """
 
-make_function_without_context_template = """
+template_make_function_without_context_template = """
 static PyObject *MAKE_FUNCTION_%(function_identifier)s( %(function_creation_args)s )
 {
     PyObject *result = Nuitka_Function_New(
-        %(fparse_function_identifier)s,
-        %(dparse_function_identifier)s,
+        %(function_impl_identifier)s,
         %(function_name_obj)s,
 #if PYTHON_VERSION >= 330
         %(function_qualname_obj)s,
@@ -99,9 +102,6 @@ static PyObject *impl_%(function_identifier)s( %(parameter_objects_decl)s )
 """
 
 template_function_exception_exit = """\
-    // Return statement must have exited already.
-    NUITKA_CANNOT_GET_HERE( %(function_identifier)s );
-
 function_exception_exit:
 %(function_cleanup)s\
     assert( exception_type );
@@ -110,19 +110,12 @@ function_exception_exit:
     return NULL;
 """
 
-template_function_noexception_exit = """\
-    // Return statement must have exited already.
-    NUITKA_CANNOT_GET_HERE( %(function_identifier)s );
-
-    return NULL;
-"""
-
 template_function_return_exit = """\
-    function_return_exit:
-%(function_cleanup)s\
-    CHECK_OBJECT( tmp_return_value );
-    assert( had_error || !ERROR_OCCURRED() );
-    return tmp_return_value;
+function_return_exit:
+%(function_cleanup)s
+CHECK_OBJECT( tmp_return_value );
+assert( had_error || !ERROR_OCCURRED() );
+return tmp_return_value;
 """
 
 function_direct_body_template = """\
@@ -143,8 +136,10 @@ function_direct_body_template = """\
 }
 """
 
-
 function_dict_setup = """\
 // Locals dictionary setup.
 PyObject *locals_dict = PyDict_New();
 """
+
+from . import TemplateDebugWrapper # isort:skip
+TemplateDebugWrapper.checkDebug(globals())
