@@ -1,4 +1,4 @@
-#     Copyright 2015, Kay Hayen, mailto:kay.hayen@gmail.com
+#     Copyright 2016, Kay Hayen, mailto:kay.hayen@gmail.com
 #
 #     Part of "Nuitka", an optimizing Python compiler that is compatible and
 #     integrates with CPython, but also works on its own.
@@ -32,11 +32,16 @@ import marshal
 from nuitka import Options, SourceCodeReferences, Tracing
 from nuitka.__past__ import iterItems
 from nuitka.codegen.ConstantCodes import needsPickleInit
+from nuitka.importing import ImportCache
 from nuitka.importing.StandardLibrary import (
     getStandardLibraryPaths,
     isStandardLibraryPath
 )
-from nuitka.nodes.ModuleNodes import PythonShlibModule
+from nuitka.nodes.ModuleNodes import (
+    PythonShlibModule,
+    makeUncompiledPythonModule
+)
+from nuitka.PythonVersions import python_version
 from nuitka.tree.SourceReading import readSourceCodeFromFilename
 from nuitka.utils import Utils
 
@@ -51,14 +56,14 @@ def loadCodeObjectData(precompiled_filename):
 
 module_names = set()
 
-def _detectedPrecompiledFile(filename, module_name, result, is_late):
+def _detectedPrecompiledFile(filename, module_name, result, user_provided):
     if filename.endswith(".pyc"):
         if Utils.isFile(filename[:-1]):
             return _detectedSourceFile(
-                filename    = filename[:-1],
-                module_name = module_name,
-                result      = result,
-                is_late     = is_late
+                filename      = filename[:-1],
+                module_name   = module_name,
+                result        = result,
+                user_provided = user_provided
             )
 
     if module_name in module_names:
@@ -71,30 +76,30 @@ def _detectedPrecompiledFile(filename, module_name, result, is_late):
     )
 
     result.append(
-        (
-            module_name,
-            loadCodeObjectData(
+        makeUncompiledPythonModule(
+            module_name   = module_name,
+            bytecode      = loadCodeObjectData(
                 precompiled_filename = filename
             ),
-            "__init__" in filename,
-            filename,
-            is_late
-        ),
+            is_package    = "__init__" in filename,
+            filename      = filename,
+            user_provided = user_provided
+        )
     )
 
     module_names.add(module_name)
 
 
-def _detectedSourceFile(filename, module_name, result, is_late):
+def _detectedSourceFile(filename, module_name, result, user_provided):
     if module_name in module_names:
         return
 
     if module_name == "collections.abc":
         _detectedSourceFile(
-            filename    = filename,
-            module_name = "_collections_abc",
-            result      = result,
-            is_late     = is_late
+            filename      = filename,
+            module_name   = "_collections_abc",
+            result        = result,
+            user_provided = user_provided
         )
 
     source_code = readSourceCodeFromFilename(module_name, filename)
@@ -117,14 +122,14 @@ __file__ = (__nuitka_binary_dir + '%s%s') if '__nuitka_binary_dir' in dict(__bui
     )
 
     result.append(
-        (
-            module_name,
-            marshal.dumps(
+        makeUncompiledPythonModule(
+            module_name   = module_name,
+            bytecode      = marshal.dumps(
                 compile(source_code, filename, "exec")
             ),
-            Utils.basename(filename) == "__init__.py",
-            filename,
-            is_late
+            is_package    = Utils.basename(filename) == "__init__.py",
+            filename      = filename,
+            user_provided = user_provided
         )
     )
 
@@ -136,6 +141,10 @@ def _detectedShlibFile(filename, module_name):
     if module_name == "__main__":
         return
 
+    from nuitka import ModuleRegistry
+    if ModuleRegistry.hasRootModule(module_name):
+        return
+
     parts = module_name.split('.')
     if len(parts) == 1:
         package_name = None
@@ -154,18 +163,18 @@ def _detectedShlibFile(filename, module_name):
         source_ref   = source_ref
     )
 
-    from nuitka import ModuleRegistry
     ModuleRegistry.addRootModule(shlib_module)
+    ImportCache.addImportedModule(shlib_module)
 
     module_names.add(module_name)
 
 
-def _detectImports(command, is_late):
+def _detectImports(command, user_provided):
     # This is pretty complicated stuff, with variants to deal with.
     # pylint: disable=R0912,R0914,R0915
 
     # Print statements for stuff to show, the modules loaded.
-    if Utils.python_version >= 300:
+    if python_version >= 300:
         command += '\nimport sys\nprint("\\n".join(sorted("import " + module.__name__ + " # sourcefile " + ' \
                    'module.__file__ for module in sys.modules.values() if hasattr(module, "__file__") and ' \
                    'module.__file__ != "<frozen>")), file = sys.stderr)'  # do not read it
@@ -192,7 +201,7 @@ def _detectImports(command, is_late):
     tmp_file, tmp_filename = tempfile.mkstemp()
 
     try:
-        if Utils.python_version >= 300:
+        if python_version >= 300:
             command = command.encode("ascii")
         os.write(tmp_file, command)
         os.close(tmp_file)
@@ -226,14 +235,14 @@ def _detectImports(command, is_late):
             module_name = parts[0].split(b" ", 2)[1]
             origin = parts[1].split()[0]
 
-            if Utils.python_version >= 300:
+            if python_version >= 300:
                 module_name = module_name.decode("utf-8")
 
             if origin == b"precompiled":
                 # This is a ".pyc" file that was imported, even before we have a
                 # chance to do anything, we need to preserve it.
                 filename = parts[1][len(b"precompiled from "):]
-                if Utils.python_version >= 300:
+                if python_version >= 300:
                     filename = filename.decode("utf-8")
 
                 # Do not leave standard library when freezing.
@@ -241,14 +250,14 @@ def _detectImports(command, is_late):
                     continue
 
                 _detectedPrecompiledFile(
-                    filename    = filename,
-                    module_name = module_name,
-                    result      = result,
-                    is_late     = is_late
+                    filename      = filename,
+                    module_name   = module_name,
+                    result        = result,
+                    user_provided = user_provided
                 )
             elif origin == b"sourcefile":
                 filename = parts[1][len(b"sourcefile "):]
-                if Utils.python_version >= 300:
+                if python_version >= 300:
                     filename = filename.decode("utf-8")
 
                 # Do not leave standard library when freezing.
@@ -257,10 +266,10 @@ def _detectImports(command, is_late):
 
                 if filename.endswith(".py"):
                     _detectedSourceFile(
-                        filename    = filename,
-                        module_name = module_name,
-                        result      = result,
-                        is_late     = is_late
+                        filename      = filename,
+                        module_name   = module_name,
+                        result        = result,
+                        user_provided = user_provided
                     )
                 elif not filename.endswith("<frozen>"):
                     _detectedShlibFile(
@@ -271,7 +280,7 @@ def _detectImports(command, is_late):
                 # Shared library in early load, happens on RPM based systems and
                 # or self compiled Python installations.
                 filename = parts[1][len(b"dynamically loaded from "):]
-                if Utils.python_version >= 300:
+                if python_version >= 300:
                     filename = filename.decode("utf-8")
 
                 # Do not leave standard library when freezing.
@@ -292,11 +301,11 @@ def detectLateImports():
     # we need to make sure it will be available as well.
     if needsPickleInit():
         command += "import {pickle};".format(
-            pickle = "pickle" if Utils.python_version >= 300 else "cPickle"
+            pickle = "pickle" if python_version >= 300 else "cPickle"
         )
 
     # For Python3 we patch inspect without knowing if it is used.
-    if Utils.python_version >= 300:
+    if python_version >= 300:
         command += "import inspect;"
 
     if command:
@@ -306,7 +315,7 @@ def detectLateImports():
 
         return result
     else:
-        return ""
+        return ()
 
 # Some modules we want to blacklist.
 ignore_modules = [
@@ -338,7 +347,13 @@ def scanStandardLibraryPath(stdlib_dir):
             if "turtledemo" in dirs:
                 dirs.remove("turtledemo")
 
-        if import_path in ("tkinter", "importlib", "ctypes"):
+            if "ensurepip" in filenames:
+                filenames.remove("ensurepip")
+            if "ensurepip" in dirs:
+                dirs.remove("ensurepip")
+
+        if import_path in ("tkinter", "importlib", "ctypes", "unittest",
+                           "sqlite3", "distutils"):
             if "test" in dirs:
                 dirs.remove("test")
 
@@ -346,7 +361,7 @@ def scanStandardLibraryPath(stdlib_dir):
             if "tests" in dirs:
                 dirs.remove("tests")
 
-        if Utils.python_version >= 340 and Utils.getOS() == "Windows":
+        if python_version >= 340 and Utils.getOS() == "Windows":
             if import_path == "multiprocessing":
                 filenames.remove("popen_fork.py")
                 filenames.remove("popen_forkserver.py")
@@ -364,7 +379,7 @@ def scanStandardLibraryPath(stdlib_dir):
                 else:
                     yield import_path + '.' + module_name
 
-        if Utils.python_version >= 300:
+        if python_version >= 300:
             if "__pycache__" in dirs:
                 dirs.remove("__pycache__")
 
@@ -376,7 +391,21 @@ def scanStandardLibraryPath(stdlib_dir):
 
 
 def detectEarlyImports():
-    if Options.freezeAllStdlib():
+    # TODO: Should recursively include all of encodings module.
+    import_code = "import encodings.utf_8;import encodings.ascii;import encodings.idna;"
+
+    if Utils.getOS() == "Windows":
+        import_code += "import encodings.mbcs;import encodings.cp437;"
+
+    # String method hex depends on it.
+    if python_version < 300:
+        import_code += "import encodings.hex_codec;"
+
+    import_code += "import locale;"
+
+    result = _detectImports(import_code, False)
+
+    if Options.shallFreezeAllStdlib():
         stdlib_modules = set()
 
         # Scan the standard library paths (multiple in case of virtualenv.
@@ -388,22 +417,20 @@ def detectEarlyImports():
                       "for imp in imports:\n" \
                       "    try:\n" \
                       "        __import__(imp)\n" \
-                      "    except ImportError:\n" \
+                      "    except (ImportError, SyntaxError):\n" \
                       "        pass\n"
-    else:
-        # TODO: Should recursively include all of encodings module.
-        import_code = "import encodings.utf_8;import encodings.ascii;import encodings.idna;"
 
-        if Utils.getOS() == "Windows":
-            import_code += "import encodings.mbcs;import encodings.cp437;"
+        early_names = [
+            module.getFullName()
+            for module in result
+        ]
 
-        # String method hex depends on it.
-        if Utils.python_version < 300:
-            import_code += "import encodings.hex_codec;"
+        result += [
+            module
+            for module in _detectImports(import_code, False)
+            if module.getFullName() not in early_names
+        ]
 
-        import_code += "import locale;"
-
-    result = _detectImports(import_code, False)
     debug("Finished detecting early imports.")
 
     return result
@@ -442,9 +469,13 @@ def _detectBinaryPathDLLsLinuxBSD(binary_filename):
         if not filename:
             continue
 
-        if Utils.python_version >= 300:
+        if python_version >= 300:
             filename = filename.decode("utf-8")
 
+        # Sometimes might use stuff not found.
+        if filename == "not found":
+            continue
+
         result.add(filename)
 
     return result
@@ -476,7 +507,7 @@ def _detectBinaryPathDLLsMacOS(binary_filename):
                 stop = True
                 break
         if not stop:
-            if Utils.python_version >= 300:
+            if python_version >= 300:
                 filename = filename.decode("utf-8")
 
             # print "adding", filename
