@@ -1,4 +1,4 @@
-#     Copyright 2015, Kay Hayen, mailto:kay.hayen@gmail.com
+#     Copyright 2016, Kay Hayen, mailto:kay.hayen@gmail.com
 #
 #     Part of "Nuitka", an optimizing Python compiler that is compatible and
 #     integrates with CPython, but also works on its own.
@@ -41,6 +41,8 @@ from logging import warning
 
 from nuitka import Options
 from nuitka.containers import oset
+from nuitka.plugins.PluginBase import Plugins
+from nuitka.PythonVersions import python_version
 from nuitka.utils import Utils
 
 from .PreloadedPackages import getPreloadedPackagePath, isPreloadedPackagePath
@@ -74,18 +76,49 @@ def isPackageDir(dirname):
 
     return Utils.isDir(dirname) and \
            (
-               Utils.python_version >= 330 or
+               python_version >= 330 or
                Utils.isFile(Utils.joinpath(dirname, "__init__.py")) or
                isPreloadedPackagePath(dirname)
            )
 
+
 def getPackageNameFromFullName(full_name):
     if '.' in full_name:
         return full_name[:full_name.rfind('.')]
     else:
         return None
 
-def warnAbout(module_name, parent_package, level, source_ref):
+def getExtensionModuleSuffixes():
+    for suffix, _mode, kind in imp.get_suffixes():
+        if kind == imp.C_EXTENSION:
+            yield suffix
+
+
+def getModuleNameAndKindFromFilename(module_filename):
+    if Utils.isDir(module_filename):
+        module_name = Utils.basename(module_filename)
+        module_kind = "py"
+    elif module_filename.endswith(".py"):
+        module_name = Utils.basename(module_filename)[:-3]
+        module_kind = "py"
+    else:
+        for suffix, _mode, kind in imp.get_suffixes():
+            if kind != imp.C_EXTENSION:
+                continue
+
+            if module_filename.endswith(suffix):
+                module_name = Utils.basename(module_filename)[:-len(suffix)]
+                module_kind = "shlib"
+
+                break
+        else:
+            module_kind = None
+            module_name = None
+
+    return module_name, module_kind
+
+
+def warnAbout(importing, module_name, parent_package, level):
     # This probably should not be dealt with here.
     if module_name == "":
         return
@@ -96,6 +129,14 @@ def warnAbout(module_name, parent_package, level, source_ref):
         if key not in warned_about:
             warned_about.add(key)
 
+            if parent_package is None:
+                full_name = module_name
+            else:
+                full_name = module_name
+
+            if Plugins.suppressUnknownImportWarning(importing, full_name):
+                return
+
             if level == 0:
                 level_desc = "as absolute import"
             elif level == -1:
@@ -108,7 +149,7 @@ def warnAbout(module_name, parent_package, level, source_ref):
             if parent_package is not None:
                 warning(
                     "%s: Cannot find '%s' in package '%s' %s.",
-                    source_ref.getAsString(),
+                    importing.getSourceReference().getAsString(),
                     module_name,
                     parent_package,
                     level_desc
@@ -116,7 +157,7 @@ def warnAbout(module_name, parent_package, level, source_ref):
             else:
                 warning(
                     "%s: Cannot find '%s' %s.",
-                    source_ref.getAsString(),
+                    importing.getSourceReference().getAsString(),
                     module_name,
                     level_desc
                 )
@@ -131,7 +172,7 @@ def normalizePackageName(module_name):
     return module_name
 
 
-def findModule(source_ref, module_name, parent_package, level, warn):
+def findModule(importing, module_name, parent_package, level, warn):
     """ Find a module with given package name as parent.
 
         The package name can be None of course. Level is the same
@@ -156,6 +197,8 @@ def findModule(source_ref, module_name, parent_package, level, warn):
     # that name, but it would be wasteful.
     assert module_name != '*'
 
+    tried_names = []
+
     if level > 1:
         # TODO: Should give a warning and return not found if the levels
         # exceed the package name.
@@ -174,7 +217,7 @@ def findModule(source_ref, module_name, parent_package, level, warn):
         if full_name.endswith('.'):
             full_name = full_name[:-1]
 
-        package_name = getPackageNameFromFullName(full_name)
+        tried_names.append(full_name)
 
         try:
             module_filename = _findModule(
@@ -184,22 +227,20 @@ def findModule(source_ref, module_name, parent_package, level, warn):
             # For relative import, that is OK, we will still try absolute.
             pass
         else:
-            package_name = getPackageNameFromFullName(full_name)
-            found = "relative"
-
             if _debug_module_finding:
                 print(
-                    "findModule: Relative imported module '%s' as '%s' in filename '%s'    ':" % (
+                    "findModule: Relative imported module '%s' as '%s' in filename '%s':" % (
                         module_name,
                         full_name,
                         module_filename
                     )
                 )
 
-            return package_name, module_filename, found
+            return getPackageNameFromFullName(full_name), module_filename, "relative"
 
     if level <= 1 and module_name != "":
         module_name = normalizePackageName(module_name)
+        tried_names.append(module_name)
 
         package_name = getPackageNameFromFullName(module_name)
 
@@ -221,8 +262,6 @@ def findModule(source_ref, module_name, parent_package, level, warn):
             # For relative import, that is OK, we will still try absolute.
             pass
         else:
-            found = "absolute"
-
             if _debug_module_finding:
                 print(
                     "findModule: Found absolute imported module '%s' in filename '%s':" % (
@@ -231,15 +270,14 @@ def findModule(source_ref, module_name, parent_package, level, warn):
                     )
                 )
 
-            return package_name, module_filename, found
-
+            return package_name, module_filename, "absolute"
 
     if warn:
         warnAbout(
-            module_name,
-            parent_package,
-            level,
-            source_ref
+            importing      = importing,
+            module_name    = module_name,
+            parent_package = parent_package,
+            level          = level,
         )
 
     return None, None, "not-found"
@@ -287,7 +325,7 @@ def _findModuleInPath2(module_name, search_path):
                     )
                     break
             else:
-                if Utils.python_version >= 330:
+                if python_version >= 330:
                     candidates.add(
                         (entry, 2, package_directory)
                     )
@@ -448,8 +486,13 @@ def _findModule(module_name):
     try:
         module_search_cache[key] = _findModule2(module_name)
     except ImportError:
-        module_search_cache[key] = ImportError
-        raise
+        new_module_name = Plugins.considerFailedImportReferrals(module_name)
+
+        if new_module_name is None:
+            module_search_cache[key] = ImportError
+            raise
+        else:
+            module_search_cache[key] = _findModule(new_module_name)
 
     return module_search_cache[key]
 
