@@ -1,4 +1,4 @@
-#     Copyright 2012, Kay Hayen, mailto:kayhayen@gmx.de
+#     Copyright 2016, Kay Hayen, mailto:kay.hayen@gmail.com
 #
 #     Part of "Nuitka", an optimizing Python compiler that is compatible and
 #     integrates with CPython, but also works on its own.
@@ -23,37 +23,41 @@ answer quite a few questions at compile time.
 
 """
 
+from nuitka.optimizations import BuiltinOptimization
+
 from .NodeBases import (
-    CPythonExpressionBuiltinSingleArgBase,
-    CPythonExpressionBuiltinNoArgBase
+    ExpressionBuiltinNoArgBase,
+    ExpressionBuiltinSingleArgBase
 )
 
-from nuitka.transform.optimizations import BuiltinOptimization
 
-
-class CPythonExpressionBuiltinOrd0( CPythonExpressionBuiltinNoArgBase ):
+class ExpressionBuiltinOrd0(ExpressionBuiltinNoArgBase):
     kind = "EXPRESSION_BUILTIN_ORD0"
 
-    def __init__( self, source_ref ):
-        CPythonExpressionBuiltinNoArgBase.__init__(
+    def __init__(self, source_ref):
+        ExpressionBuiltinNoArgBase.__init__(
             self,
             builtin_function = ord,
             source_ref       = source_ref
         )
 
-class CPythonExpressionBuiltinOrd( CPythonExpressionBuiltinSingleArgBase ):
+
+class ExpressionBuiltinOrd(ExpressionBuiltinSingleArgBase):
     kind = "EXPRESSION_BUILTIN_ORD"
 
     builtin_spec = BuiltinOptimization.builtin_ord_spec
 
-    def isKnownToBeIterable( self, count ):
+    def isKnownToBeIterable(self, count):
         return False
 
 
-class CPythonExpressionBuiltinChr( CPythonExpressionBuiltinSingleArgBase ):
+class ExpressionBuiltinChr(ExpressionBuiltinSingleArgBase):
     kind = "EXPRESSION_BUILTIN_CHR"
 
     builtin_spec = BuiltinOptimization.builtin_chr_spec
 
-    def isKnownToBeIterable( self, count ):
+    def isKnownToBeIterable(self, count):
+        if self.mayRaiseException(BaseException):
+            return None
+
         return count is None or count == 1
