@@ -1,4 +1,4 @@
-#     Copyright 2015, Kay Hayen, mailto:kay.hayen@gmail.com
+#     Copyright 2016, Kay Hayen, mailto:kay.hayen@gmail.com
 #
 #     Part of "Nuitka", an optimizing Python compiler that is compatible and
 #     integrates with CPython, but also works on its own.
@@ -15,7 +15,7 @@
 #     See the License for the specific language governing permissions and
 #     limitations under the License.
 #
-""" Node the calls to the 'dict' builtin.
+""" Node for the calls to the 'dict' built-in.
 
 """
 
@@ -25,11 +25,10 @@ from nuitka.nodes.DictionaryNodes import (
 )
 from nuitka.nodes.NodeMakingHelpers import wrapExpressionWithNodeSideEffects
 from nuitka.optimizations.BuiltinOptimization import builtin_dict_spec
-from nuitka.utils.Utils import python_version
 
+from .BuiltinIteratorNodes import ExpressionBuiltinIter1
 from .ConstantRefNodes import ExpressionConstantRef
 from .NodeBases import ExpressionChildrenHavingBase
-from .NodeMakingHelpers import makeConstantReplacementNode
 
 
 class ExpressionBuiltinDict(ExpressionChildrenHavingBase):
@@ -80,72 +79,70 @@ class ExpressionBuiltinDict(ExpressionChildrenHavingBase):
 
     def computeExpression(self, constraint_collection):
         pos_arg = self.getPositionalArgument()
+        pairs = self.getNamedArgumentPairs()
 
-        if self.hasOnlyConstantArguments():
+        if pos_arg is None:
+            new_node = ExpressionMakeDict(
+                pairs      = self.getNamedArgumentPairs(),
+                source_ref = self.source_ref
+            )
+
+            # This cannot raise anymore than its arguments, as the keys will
+            # be known as hashable, due to being Python parameters before.
+
+            return (
+                new_node,
+                "new_expression",
+                "Replace 'dict' built-in call dictionary creation from arguments."
+            )
+
+        pos_iteration_length = pos_arg.getIterationLength()
+
+        if pos_iteration_length == 0:
+            new_node = ExpressionMakeDict(
+                pairs      = self.getNamedArgumentPairs(),
+                source_ref = self.source_ref
+            )
+
+            # Maintain potential side effects from the positional arguments.
+            new_node = wrapExpressionWithNodeSideEffects(
+                old_node = ExpressionBuiltinIter1(
+                    value      = pos_arg,
+                    source_ref = self.source_ref
+                ),
+                new_node = new_node
+            )
+
+            # Just in case, the iteration may do that.
+            if pos_arg.mayRaiseExceptionIter(BaseException):
+                constraint_collection.onExceptionRaiseExit(BaseException)
+
+            return (
+                new_node,
+                "new_expression",
+                "Replace 'dict' built-in call dictionary creation from arguments."
+            )
+
+        if pos_iteration_length is not None and \
+           pos_iteration_length + len(pairs) < 256 and \
+           self.hasOnlyConstantArguments():
             if pos_arg is not None:
                 pos_args = (
                     pos_arg,
                 )
-            elif python_version >= 340:
-                # Doing this here, because calling dict built-in apparently
-                # mutates existing dictionaries in Python 3.4
-                result = {}
-
-                for pair in reversed(self.getNamedArgumentPairs()):
-                    arg_name = pair.getKey().getCompileTimeConstant()
-                    arg_value = pair.getValue().getCompileTimeConstant()
-
-                    result[arg_name] = arg_value
-
-                new_node = makeConstantReplacementNode(
-                    constant = result,
-                    node     = self
-                )
-
-                return new_node, "new_expression", "Replace 'dict' built-in call with constant arguments."
             else:
                 pos_args = None
 
-            from .NodeMakingHelpers import getComputationResult
-
-            return getComputationResult(
+            return constraint_collection.getCompileTimeComputationResult(
                 node        = self,
                 computation = lambda : builtin_dict_spec.simulateCall(
                     (pos_args, self.getNamedArgumentPairs())
                 ),
                 description = "Replace 'dict' call with constant arguments."
             )
-        elif pos_arg is None:
-            new_node = ExpressionMakeDict(
-                pairs      = self.getNamedArgumentPairs(),
-                lazy_order = False,
-                source_ref = self.source_ref
-            )
-
-            return (
-                new_node,
-                "new_expression",
-                "Replace 'dict' built-in call dictionary creation from arguments."
-            )
-        elif pos_arg.getIterationLength() == 0:
-            new_node = ExpressionMakeDict(
-                pairs      = self.getNamedArgumentPairs(),
-                lazy_order = False,
-                source_ref = self.source_ref
-            )
-
-            # Maintain potential side effects from the positional arguments.
-            new_node = wrapExpressionWithNodeSideEffects(
-                old_node = pos_arg,
-                new_node = new_node
-            )
-
-            return (
-                new_node,
-                "new_expression",
-                "Replace 'dict' built-in call dictionary creation from arguments."
-            )
         else:
+            constraint_collection.onExceptionRaiseExit(BaseException)
+
             return self, None, None
 
     def mayRaiseException(self, exception_type):
