@@ -1,4 +1,4 @@
-#     Copyright 2015, Kay Hayen, mailto:kay.hayen@gmail.com
+#     Copyright 2016, Kay Hayen, mailto:kay.hayen@gmail.com
 #
 #     Part of "Nuitka", an optimizing Python compiler that is compatible and
 #     integrates with CPython, but also works on its own.
@@ -32,9 +32,10 @@ from nuitka.Builtins import (
     builtin_names
 )
 from nuitka.optimizations import BuiltinOptimization
-from nuitka.utils.Utils import python_version
+from nuitka.PythonVersions import python_version
 
 from .ConstantRefNodes import ExpressionConstantRef
+from .ExceptionNodes import ExpressionBuiltinMakeException
 from .NodeBases import CompileTimeConstantExpressionMixin, NodeBase
 
 
@@ -45,9 +46,6 @@ class ExpressionBuiltinRefBase(CompileTimeConstantExpressionMixin, NodeBase):
 
         self.builtin_name = builtin_name
 
-    def makeCloneAt(self, source_ref):
-        return self.__class__(self.builtin_name, source_ref)
-
     def getDetails(self):
         return {
             "builtin_name" : self.builtin_name
@@ -59,6 +57,19 @@ class ExpressionBuiltinRefBase(CompileTimeConstantExpressionMixin, NodeBase):
     def isKnownToBeHashable(self):
         return True
 
+    def mayRaiseException(self, exception_type):
+        return False
+
+    def mayHaveSideEffects(self):
+        return False
+
+    def getStrValue(self):
+        return ExpressionConstantRef(
+            constant      = str(self.getCompileTimeConstant()),
+            user_provided = True,
+            source_ref    = self.getSourceReference(),
+        )
+
 
 class ExpressionBuiltinRef(ExpressionBuiltinRefBase):
     kind = "EXPRESSION_BUILTIN_REF"
@@ -94,18 +105,27 @@ class ExpressionBuiltinRef(ExpressionBuiltinRefBase):
             )
 
             return new_node, "new_constant", """\
-Built-in constant %s resolved""" % self.builtin_name
+Built-in constant '%s' resolved.""" % self.builtin_name
 
         return self, None, None
 
-    def computeExpressionCall(self, call_node, constraint_collection):
+    def computeExpressionCall(self, call_node, call_args, call_kw,
+                              constraint_collection):
         from nuitka.optimizations.OptimizeBuiltinCalls import computeBuiltinCall
 
+        # Anything may happen. On next pass, if replaced, we might be better
+        # but not now.
+        constraint_collection.onExceptionRaiseExit(BaseException)
+
         new_node, tags, message = computeBuiltinCall(
             call_node = call_node,
             called    = self
         )
 
+        if self.builtin_name in ("dir", "eval", "exec", "execfile", "locals", "vars"):
+            # Just inform the collection that all has escaped.
+            constraint_collection.onLocalsUsage()
+
         return new_node, tags, message
 
     def getStringValue(self):
@@ -134,7 +154,7 @@ class ExpressionBuiltinAnonymousRef(ExpressionBuiltinRefBase):
     kind = "EXPRESSION_BUILTIN_ANONYMOUS_REF"
 
     def __init__(self, builtin_name, source_ref):
-        assert builtin_name in builtin_anon_names
+        assert builtin_name in builtin_anon_names, builtin_name
 
         ExpressionBuiltinRefBase.__init__(
             self,
@@ -187,21 +207,18 @@ class ExpressionBuiltinExceptionRef(ExpressionBuiltinRefBase):
         # Not much that can be done here.
         return self, None, None
 
-    def computeExpressionCall(self, call_node, constraint_collection):
+    def computeExpressionCall(self, call_node, call_args, call_kw,
+                              constraint_collection):
         exception_name = self.getExceptionName()
 
         # TODO: Keyword only arguments of it, are not properly handled yet by
         # the built-in call code.
         if exception_name == "ImportError" and python_version >= 330:
-            kw = call_node.getCallKw()
-
-            if kw is not None and \
-               (not kw.isExpressionConstantRef() or kw.getConstant() != {}):
+            if call_kw is not None and \
+               (not call_kw.isExpressionConstantRef() or call_kw.getConstant() != {}):
                 return call_node, None, None
 
         def createBuiltinMakeException(args, source_ref):
-            from .ExceptionNodes import ExpressionBuiltinMakeException
-
             return ExpressionBuiltinMakeException(
                 exception_name = exception_name,
                 args           = args,
