@@ -1,4 +1,4 @@
-#     Copyright 2015, Kay Hayen, mailto:kay.hayen@gmail.com
+#     Copyright 2016, Kay Hayen, mailto:kay.hayen@gmail.com
 #
 #     Part of "Nuitka", an optimizing Python compiler that is compatible and
 #     integrates with CPython, but also works on its own.
@@ -55,6 +55,37 @@ class ExpressionListOperationAppend(ExpressionChildrenHavingBase):
         return self, None, None
 
 
+class ExpressionListOperationExtend(ExpressionChildrenHavingBase):
+    kind = "EXPRESSION_LIST_OPERATION_EXTEND"
+
+    named_children = (
+        "list",
+        "value"
+    )
+
+    @calledWithBuiltinArgumentNamesDecorator
+    def __init__(self, list_arg, value, source_ref):
+        assert list_arg is not None
+        assert value is not None
+
+        ExpressionChildrenHavingBase.__init__(
+            self,
+            values     = {
+                "list"  : list_arg,
+                "value" : value
+            },
+            source_ref = source_ref
+        )
+
+    getList = ExpressionChildrenHavingBase.childGetter("list")
+    getValue = ExpressionChildrenHavingBase.childGetter("value")
+
+    def computeExpression(self, constraint_collection):
+        constraint_collection.removeKnowledge(self.getList())
+
+        return self, None, None
+
+
 class ExpressionListOperationPop(ExpressionChildrenHavingBase):
     kind = "EXPRESSION_LIST_OPERATION_POP"
 
@@ -92,6 +123,41 @@ class ExpressionSetOperationAdd(ExpressionChildrenHavingBase):
         "value"
     )
 
+    @calledWithBuiltinArgumentNamesDecorator
+    def __init__(self, set_arg, value, source_ref):
+        assert set_arg is not None
+        assert value is not None
+
+        ExpressionChildrenHavingBase.__init__(
+            self,
+            values     = {
+                "set"   : set_arg,
+                "value" : value
+            },
+            source_ref = source_ref
+        )
+
+    getSet = ExpressionChildrenHavingBase.childGetter(
+        "set"
+    )
+    getValue = ExpressionChildrenHavingBase.childGetter(
+        "value"
+    )
+
+    def computeExpression(self, constraint_collection):
+        constraint_collection.removeKnowledge(self.getSet())
+
+        return self, None, None
+
+
+class ExpressionSetOperationUpdate(ExpressionChildrenHavingBase):
+    kind = "EXPRESSION_SET_OPERATION_UPDATE"
+
+    named_children = (
+        "set",
+        "value"
+    )
+
     @calledWithBuiltinArgumentNamesDecorator
     def __init__(self, set_arg, value, source_ref):
         assert set_arg is not None
@@ -244,3 +310,38 @@ class ExpressionDictOperationGet(ExpressionChildrenHavingBase):
 
     def computeExpression(self, constraint_collection):
         return self, None, None
+
+
+class ExpressionDictOperationUpdate(ExpressionChildrenHavingBase):
+    kind = "EXPRESSION_DICT_OPERATION_UPDATE"
+
+    named_children = (
+        "dict",
+        "value"
+    )
+
+    @calledWithBuiltinArgumentNamesDecorator
+    def __init__(self, dict_arg, value, source_ref):
+        assert dict_arg is not None
+        assert value is not None
+
+        ExpressionChildrenHavingBase.__init__(
+            self,
+            values     = {
+                "dict"  : dict_arg,
+                "value" : value
+            },
+            source_ref = source_ref
+        )
+
+    getSet = ExpressionChildrenHavingBase.childGetter(
+        "dict"
+    )
+    getValue = ExpressionChildrenHavingBase.childGetter(
+        "value"
+    )
+
+    def computeExpression(self, constraint_collection):
+        constraint_collection.removeKnowledge(self.getSet())
+
+        return self, None, None
