@@ -1,4 +1,4 @@
-#     Copyright 2015, Kay Hayen, mailto:kay.hayen@gmail.com
+#     Copyright 2016, Kay Hayen, mailto:kay.hayen@gmail.com
 #
 #     Part of "Nuitka", an optimizing Python compiler that is compatible and
 #     integrates with CPython, but also works on its own.
@@ -23,7 +23,7 @@ to do.
 """
 
 from nuitka.Builtins import calledWithBuiltinArgumentNamesDecorator
-from nuitka.utils import Utils
+from nuitka.PythonVersions import python_version
 
 from .NodeBases import ExpressionChildrenHavingBase, StatementChildrenHavingBase
 from .NodeMakingHelpers import (
@@ -62,7 +62,7 @@ class ExpressionBuiltinEval(ExpressionChildrenHavingBase):
 
 
 # Note: Python3 only so far.
-if Utils.python_version >= 300:
+if python_version >= 300:
     class ExpressionBuiltinExec(ExpressionBuiltinEval):
         kind = "EXPRESSION_BUILTIN_EXEC"
 
@@ -76,7 +76,7 @@ if Utils.python_version >= 300:
             )
 
         def needsLocalsDict(self):
-            return False
+            return True
 
         def computeExpression(self, constraint_collection):
             # TODO: Attempt for constant values to do it.
@@ -92,13 +92,13 @@ if Utils.python_version >= 300:
                 )
 
                 return result, "new_statements", """\
-Replaced builtin exec call to exec statement in early closure context."""
+Replaced built-in exec call to exec statement in early closure context."""
             else:
                 return statement, None, None
 
 
 # Note: Python2 only
-if Utils.python_version < 300:
+if python_version < 300:
     class ExpressionBuiltinExecfile(ExpressionBuiltinEval):
         kind = "EXPRESSION_BUILTIN_EXECFILE"
 
@@ -121,8 +121,7 @@ if Utils.python_version < 300:
             # correctly by the code for exec statements.
             provider = self.getParentVariableProvider()
 
-            if provider.isExpressionFunctionBody() and \
-               provider.isClassDictCreation():
+            if provider.isExpressionClassBody():
                 result = StatementExec(
                     source_code = self.getSourceCode(),
                     globals_arg = self.getGlobals(),
@@ -131,7 +130,7 @@ if Utils.python_version < 300:
                 )
 
                 return result, "new_statements", """\
-Changed execfile to exec on class level."""
+Changed 'execfile' with unused result to 'exec' on class level."""
             else:
                 return statement, None, None
 
@@ -175,6 +174,11 @@ class StatementExec(StatementChildrenHavingBase):
         )
         source_code = self.getSourceCode()
 
+        if source_code.mayRaiseException(BaseException):
+            constraint_collection.onExceptionRaiseExit(
+                BaseException
+            )
+
         if source_code.willRaiseException(BaseException):
             result = source_code
 
@@ -191,6 +195,11 @@ Exec statement raises implicitly when determining source code argument."""
         )
         globals_arg = self.getGlobals()
 
+        if globals_arg is not None and globals_arg.mayRaiseException(BaseException):
+            constraint_collection.onExceptionRaiseExit(
+                BaseException
+            )
+
         if globals_arg is not None and \
            globals_arg.willRaiseException(BaseException):
             result = makeStatementOnlyNodesFromExpressions(
@@ -213,6 +222,11 @@ Exec statement raises implicitly when determining globals argument."""
         )
         locals_arg = self.getLocals()
 
+        if locals_arg is not None and locals_arg.mayRaiseException(BaseException):
+            constraint_collection.onExceptionRaiseExit(
+                BaseException
+            )
+
         if locals_arg is not None and \
            locals_arg.willRaiseException(BaseException):
             result = makeStatementOnlyNodesFromExpressions(
@@ -230,10 +244,15 @@ Exec statement raises implicitly when determining globals argument."""
 Exec statement raises implicitly when determining locals argument."""
             )
 
+        constraint_collection.onExceptionRaiseExit(
+            BaseException
+        )
+
         str_value = self.getSourceCode().getStrValue()
 
         if False and str_value is not None:
-            # TODO: Don't forget to consider side effects of source code.
+            # TODO: Don't forget to consider side effects of source code,
+            # locals_arg, and globals_arg.
 
             # TODO: This needs to be re-done.
             exec_body = None
@@ -265,7 +284,7 @@ class StatementLocalsDictSync(StatementChildrenHavingBase):
         )
 
     def computeStatement(self, constraint_collection):
-        if self.getParentVariableProvider().isPythonModule():
+        if self.getParentVariableProvider().isCompiledPythonModule():
             return None, "new_statements", "Removed sync back to locals without locals."
 
         constraint_collection.removeAllKnowledge()
@@ -313,5 +332,7 @@ class ExpressionBuiltinCompile(ExpressionChildrenHavingBase):
     getOptimize = ExpressionChildrenHavingBase.childGetter("optimize")
 
     def computeExpression(self, constraint_collection):
+        constraint_collection.onExceptionRaiseExit(BaseException)
+
         # TODO: Attempt for constant values to do it.
         return self, None, None
