@@ -1,4 +1,4 @@
-#     Copyright 2015, Kay Hayen, mailto:kay.hayen@gmail.com
+#     Copyright 2016, Kay Hayen, mailto:kay.hayen@gmail.com
 #
 #     Part of "Nuitka", an optimizing Python compiler that is compatible and
 #     integrates with CPython, but also works on its own.
@@ -25,14 +25,17 @@ only be used to be called directly, while knowing exactly what it is. So
 the "ExpressionFunctionCreation" might be used to provide that kind of
 CPython reference, and may escape.
 
-
+Coroutines and generators live in their dedicated module and share base
+classes.
 """
 
-from nuitka import Variables
-from nuitka.utils import Utils
+from nuitka import Options, VariableRegistry, Variables
+from nuitka.optimizations.FunctionInlining import convertFunctionCallToOutline
+from nuitka.PythonVersions import python_version
+from nuitka.tree.Extractions import updateVariableUsage
 
+from .Checkers import checkStatementsSequenceOrNone
 from .IndicatorMixins import (
-    MarkGeneratorIndicator,
     MarkLocalsDictIndicator,
     MarkUnoptimizedFunctionIndicator
 )
@@ -44,79 +47,20 @@ from .NodeBases import (
     ExpressionChildrenHavingBase,
     ExpressionMixin,
     NodeBase,
-    SideEffectsFromChildrenMixin,
-    checkStatementsSequenceOrNone
+    SideEffectsFromChildrenMixin
+)
+from .NodeMakingHelpers import (
+    makeConstantReplacementNode,
+    makeRaiseExceptionReplacementExpressionFromInstance,
+    wrapExpressionWithSideEffects
 )
 from .ParameterSpecs import TooManyArguments, matchCall
 
 
-class ExpressionFunctionBody(ClosureTakerMixin, ChildrenHavingMixin,
-                             ClosureGiverNodeBase, ExpressionMixin,
-                             MarkGeneratorIndicator,
-                             MarkLocalsDictIndicator,
-                             MarkUnoptimizedFunctionIndicator):
-    # We really want these many ancestors, as per design, we add properties via
-    # base class mix-ins a lot, leading to many instance attributes, and
-    # methods, pylint: disable=R0901,R0902
-
-    kind = "EXPRESSION_FUNCTION_BODY"
-
-    named_children = (
-        "body",
-    )
-
-    checkers = {
-        "body" : checkStatementsSequenceOrNone
-    }
-
-    if Utils.python_version >= 340:
-        qualname_setup = None
-
-    def __init__(self, provider, name, doc, parameters, source_ref,
-                 is_class = False):
-        # These got too many details to cover, pylint: disable=R0915
-
-        if is_class:
-            code_prefix = "class"
-        else:
-            code_prefix = "function"
-
-        if name == "<lambda>":
-            name = "lambda"
-            code_prefix = name
-
-            self.is_lambda = True
-        else:
-            self.is_lambda = False
-
-        if name == "<listcontraction>":
-            assert Utils.python_version >= 300
-
-            code_prefix = "listcontr"
-            name = ""
-
-            self.local_locals = Utils.python_version >= 300
-        else:
-            self.local_locals = True
-
-        if name == "<setcontraction>":
-            code_prefix = "setcontr"
-            name = ""
-
-        if name == "<dictcontraction>":
-            code_prefix = "dictcontr"
-            name = ""
-
-        if name == "<genexpr>":
-            code_prefix = "genexpr"
-            name = ""
-
-            self.is_genexpr = True
-        else:
-            self.is_genexpr = False
-
-        self.non_local_declarations = []
+class ExpressionFunctionBodyBase(ClosureTakerMixin, ChildrenHavingMixin,
+                                 ClosureGiverNodeBase, ExpressionMixin):
 
+    def __init__(self, provider, name, code_prefix, is_class, flags, source_ref):
         ClosureTakerMixin.__init__(
             self,
             provider      = provider,
@@ -137,114 +81,56 @@ class ExpressionFunctionBody(ClosureTakerMixin, ChildrenHavingMixin,
             }
         )
 
-        MarkGeneratorIndicator.__init__(self)
+        self.flags = flags
 
-        MarkLocalsDictIndicator.__init__(self)
-
-        MarkUnoptimizedFunctionIndicator.__init__(self)
-
-        self.is_class = is_class
-        self.doc = doc
-
-        # Indicator, if this is a function that uses "super", because if it
-        # does, it would like to get the final "__class__" attached.
-        self.has_super = False
-
-        # Indicator if the return value exception might be required.
-        self.return_exception = False
-
-        # Indicator if the function needs to be created as a function object.
-        self.needs_creation = False
-
-        # Indicator if the function is called directly.
-        self.needs_direct = False
-
-        # Indicator if the function is used outside of where it's defined.
-        self.cross_module_use = False
+        # Hack: This allows some APIs to work although this is not yet
+        # officially a child yet. Important during building.
+        self.parent = provider
 
         # Python3.4: Might be overridden by global statement on the class name.
-        if Utils.python_version >= 340:
+        # TODO: Make this class only code.
+        if python_version >= 340:
             self.qualname_provider = provider
 
-        self.parameters = parameters
-        self.parameters.setOwner(self)
+        # Non-local declarations.
+        self.non_local_declarations = []
 
         # Register ourselves immediately with the module.
         provider.getParentModule().addFunction(self)
 
-        self.registerProvidedVariables(
-            *self.parameters.getVariables()
-        )
-
         self.constraint_collection = None
 
-
-    def getDetails(self):
-        return {
-            "name"       : self.getFunctionName(),
-            "ref_name"   : self.getCodeName(),
-            "parameters" : self.getParameters(),
-            "provider"   : self.provider.getCodeName(),
-            "doc"        : self.doc
-        }
-
-    def getDetail(self):
-        return "named %s with %s" % (self.getFunctionName(), self.parameters)
-
-    def getParent(self):
-        assert False
-
-    def isClassDictCreation(self):
-        return self.is_class
+    @staticmethod
+    def isExpressionFunctionBodyBase():
+        return True
 
     def getContainingClassDictCreation(self):
         current = self
 
-        while not current.isPythonModule():
-            if current.isClassDictCreation():
+        while not current.isCompiledPythonModule():
+            if current.isExpressionClassBody():
                 return current
 
             current = current.getParentVariableProvider()
 
         return None
 
-    def getFunctionName(self):
-        if self.is_lambda:
-            return "<lambda>"
-        elif self.is_genexpr:
-            return "<genexpr>"
+    def hasFlag(self, flag):
+        return flag in self.flags
+
+    def getLocalsMode(self):
+        if python_version >= 300:
+            return "updated"
+        elif self.isEarlyClosure() or self.isUnoptimized():
+            return "updated"
         else:
-            return self.name
+            return "copy"
 
-    def getFunctionQualname(self):
-        """ Function __qualname__ new in CPython3.3
+    def hasVariableName(self, variable_name):
+        return variable_name in self.providing or variable_name in self.temp_variables
 
-        Should contain some kind of full name descriptions for the closure to
-        recognize and will be used for outputs.
-        """
-
-        function_name = self.getFunctionName()
-
-        if Utils.python_version < 340:
-            provider = self.getParentVariableProvider()
-        else:
-            provider = self.qualname_provider
-
-        if provider.isPythonModule():
-            return function_name
-        elif provider.isClassDictCreation():
-            return provider.getFunctionQualname() + '.' + function_name
-        else:
-            return provider.getFunctionQualname() + ".<locals>." + function_name
-
-    def getDoc(self):
-        return self.doc
-
-    def getParameters(self):
-        return self.parameters
-
-    def getLocalVariableNames(self):
-        return Variables.getNames(self.getLocalVariables())
+    def getVariables(self):
+        return self.providing.values()
 
     def getLocalVariables(self):
         return [
@@ -263,9 +149,6 @@ class ExpressionFunctionBody(ClosureTakerMixin, ChildrenHavingMixin,
             if variable.getOwner() is self
         )
 
-    def getVariables(self):
-        return self.providing.values()
-
     def removeClosureVariable(self, variable):
         assert variable in self.providing.values(), (self.providing, variable)
 
@@ -276,6 +159,31 @@ class ExpressionFunctionBody(ClosureTakerMixin, ChildrenHavingMixin,
 
         self.taken.remove(variable)
 
+        VariableRegistry.removeVariableUsage(variable, self)
+
+    def demoteClosureVariable(self, variable):
+        assert variable.isLocalVariable()
+
+        self.taken.remove(variable)
+
+        assert variable.getOwner() is not self
+
+        new_variable = Variables.LocalVariable(
+            owner         = self,
+            variable_name = variable.getName()
+        )
+
+        self.providing[variable.getName()] = new_variable
+
+        updateVariableUsage(
+            provider     = self,
+            old_variable = variable,
+            new_variable = new_variable
+        )
+
+        VariableRegistry.removeVariableUsage(variable, self)
+        VariableRegistry.addVariableUsage(new_variable, self)
+
     def removeUserVariable(self, variable):
         assert variable in self.providing.values(), (self.providing, variable)
 
@@ -284,9 +192,10 @@ class ExpressionFunctionBody(ClosureTakerMixin, ChildrenHavingMixin,
         assert not variable.isParameterVariable() or \
                variable.getOwner() is not self
 
+        VariableRegistry.removeVariableUsage(variable, self)
 
     def getVariableForAssignment(self, variable_name):
-        # print ("ASS func", self, variable_name)
+        # print("ASS func", self, variable_name)
 
         if self.hasTakenVariable(variable_name):
             result = self.getTakenVariable(variable_name)
@@ -296,7 +205,7 @@ class ExpressionFunctionBody(ClosureTakerMixin, ChildrenHavingMixin,
         return result
 
     def getVariableForReference(self, variable_name):
-        # print ( "REF func", self, variable_name )
+        # print( "REF func", self, variable_name )
 
         if self.hasProvidedVariable(variable_name):
             result = self.getProvidedVariable(variable_name)
@@ -313,7 +222,7 @@ class ExpressionFunctionBody(ClosureTakerMixin, ChildrenHavingMixin,
             # For "exec" containing/star import containing, we get a
             # closure variable already, but if it is a module variable,
             # only then make it a maybe local variable.
-            if self.isUnoptimized() and result.isModuleVariable():
+            if not self.isExpressionClassBody() and self.isUnoptimized() and result.isModuleVariable():
                 result = Variables.MaybeLocalVariable(
                     owner          = self,
                     maybe_variable = result
@@ -326,27 +235,6 @@ class ExpressionFunctionBody(ClosureTakerMixin, ChildrenHavingMixin,
     def getVariableForClosure(self, variable_name):
         # print( "getVariableForClosure", self, variable_name )
 
-        # The class bodies provide no closure, except under CPython3.x, there
-        # they provide "__class__" but nothing else.
-
-        if self.isClassDictCreation():
-            if variable_name == "__class__":
-                if Utils.python_version < 300:
-                    return self.provider.getVariableForClosure(
-                        variable_name
-                    )
-                elif Utils.python_version >= 340 and False: # TODO: Temporarily reverted
-                    result = self.getTempVariable(
-                        temp_scope = None,
-                        name       = "__class__"
-                    )
-
-                    return result
-            else:
-                return self.provider.getVariableForClosure(
-                    variable_name
-                )
-
         if self.hasProvidedVariable(variable_name):
             return self.getProvidedVariable(variable_name)
         else:
@@ -355,21 +243,10 @@ class ExpressionFunctionBody(ClosureTakerMixin, ChildrenHavingMixin,
     def createProvidedVariable(self, variable_name):
         # print("createProvidedVariable", self, variable_name)
 
-        if self.local_locals:
-            return Variables.LocalVariable(
-                owner         = self,
-                variable_name = variable_name
-            )
-        else:
-            # Make sure the provider knows it has to provide a variable of this
-            # name for the assignment.
-            self.provider.getVariableForAssignment(
-                variable_name = variable_name
-            )
-
-            return self.getClosureVariable(
-                variable_name = variable_name
-            )
+        return Variables.LocalVariable(
+            owner         = self,
+            variable_name = variable_name
+        )
 
     def addNonlocalsDeclaration(self, names, source_ref):
         self.non_local_declarations.append(
@@ -379,12 +256,132 @@ class ExpressionFunctionBody(ClosureTakerMixin, ChildrenHavingMixin,
     def getNonlocalDeclarations(self):
         return self.non_local_declarations
 
+    def getFunctionName(self):
+        return self.name
+
+    def getFunctionQualname(self):
+        """ Function __qualname__ new in CPython3.3
+
+        Should contain some kind of full name descriptions for the closure to
+        recognize and will be used for outputs.
+        """
+
+        function_name = self.getFunctionName()
+
+        if python_version < 340:
+            provider = self.getParentVariableProvider()
+        else:
+            provider = self.qualname_provider
+
+        if provider.isCompiledPythonModule():
+            return function_name
+        elif provider.isExpressionClassBody():
+            return provider.getFunctionQualname() + '.' + function_name
+        else:
+            return provider.getFunctionQualname() + ".<locals>." + function_name
+
+    def computeExpression(self, constraint_collection):
+        assert False
+
+        # Function body is quite irreplaceable.
+        return self, None, None
+
+    def mayRaiseException(self, exception_type):
+        body = self.getBody()
+
+        if body is None:
+            return False
+        else:
+            return self.getBody().mayRaiseException(exception_type)
+
+
+class ExpressionFunctionBody(ExpressionFunctionBodyBase,
+                             MarkLocalsDictIndicator,
+                             MarkUnoptimizedFunctionIndicator):
+    # We really want these many ancestors, as per design, we add properties via
+    # base class mix-ins a lot, leading to many methods, pylint: disable=R0901
+
+    kind = "EXPRESSION_FUNCTION_BODY"
+
+    named_children = (
+        "body",
+    )
+
+    checkers = {
+        # TODO: Is "None" really an allowed value.
+        "body" : checkStatementsSequenceOrNone
+    }
+
+    if python_version >= 340:
+        qualname_setup = None
+
+    def __init__(self, provider, name, doc, parameters, flags, source_ref):
+        while provider.isExpressionOutlineBody():
+            provider = provider.getParentVariableProvider()
+
+        if name == "<listcontraction>":
+            assert python_version >= 300
+
+        ExpressionFunctionBodyBase.__init__(
+            self,
+            provider    = provider,
+            name        = name,
+            code_prefix = "function",
+            is_class    = False,
+            flags       = flags,
+            source_ref  = source_ref
+        )
+
+        MarkLocalsDictIndicator.__init__(self)
+
+        MarkUnoptimizedFunctionIndicator.__init__(self)
+
+        self.doc = doc
+
+        # Indicator if the return value exception might be required.
+        self.return_exception = False
+
+        # Indicator if the function needs to be created as a function object.
+        self.needs_creation = False
+
+        # Indicator if the function is called directly.
+        self.needs_direct = False
+
+        # Indicator if the function is used outside of where it's defined.
+        self.cross_module_use = False
+
+        self.parameters = parameters
+        self.parameters.setOwner(self)
+
+        self.registerProvidedVariables(
+            *self.parameters.getAllVariables()
+        )
+
+    def getDetails(self):
+        return {
+            "name"       : self.getFunctionName(),
+            "ref_name"   : self.getCodeName(),
+            "parameters" : self.getParameters(),
+            "provider"   : self.provider.getCodeName(),
+            "doc"        : self.doc
+        }
+
+    def getDetail(self):
+        return "named %s with %s" % (self.getFunctionName(), self.parameters)
+
+    def getParent(self):
+        assert False
+
+    def getDoc(self):
+        return self.doc
+
+    def getParameters(self):
+        return self.parameters
+
     getBody = ChildrenHavingMixin.childGetter("body")
     setBody = ChildrenHavingMixin.childSetter("body")
 
     def needsCreation(self):
-        # TODO: This looks kind of arbitrary, the users should decide, if they
-        # need it.
         return self.needs_creation
 
     def markAsNeedsCreation(self):
@@ -402,21 +399,8 @@ class ExpressionFunctionBody(ClosureTakerMixin, ChildrenHavingMixin,
     def markAsCrossModuleUsed(self):
         self.cross_module_use = True
 
-    def computeExpression(self, constraint_collection):
-        assert False
-
-        # Function body is quite irreplaceable.
-        return self, None, None
-
-    def getLocalsMode(self):
-        if Utils.python_version >= 300:
-            return "updated"
-        elif self.isEarlyClosure() or self.isUnoptimized():
-            return "updated"
-        else:
-            return "copy"
-
-    def computeExpressionCall(self, call_node, constraint_collection):
+    def computeExpressionCall(self, call_node, call_args, call_kw,
+                              constraint_collection):
         # TODO: Until we have something to re-order the arguments, we need to
         # skip this. For the immediate need, we avoid this complexity, as a
         # re-ordering will be needed.
@@ -435,35 +419,12 @@ class ExpressionFunctionBody(ClosureTakerMixin, ChildrenHavingMixin,
     def mayRaiseException(self, exception_type):
         return self.getBody().mayRaiseException(exception_type)
 
-    def markAsClassClosureTaker(self):
-        self.has_super = True
-
-    def isClassClosureTaker(self):
-        return self.has_super
-
-    def makeCloneAt(self, source_ref):
-        result = self.__class__(
-            provider   = self.provider,
-            name       = self.name,
-            doc        = self.name,
-            # TODO: Clone parameters too, when we start to mutate them.
-            parameters = self.parameters,
-            source_ref =  source_ref
-        )
-
-        result.setBody(
-            self.getBody().makeCloneAt(source_ref)
-        )
-
-        return result
-
     def markAsExceptionReturnValue(self):
         self.return_exception = True
 
     def needsExceptionReturnValue(self):
         return self.return_exception
 
-
 def convertNoneConstantOrEmptyDictToNone(node):
     if node is None:
         return None
@@ -486,7 +447,7 @@ class ExpressionFunctionCreation(SideEffectsFromChildrenMixin,
     # all kinds go last.
 
     # A bug of CPython3.x not fixed before version 3.4, see bugs.python.org/issue16967
-    kw_defaults_before_defaults = Utils.python_version < 340
+    kw_defaults_before_defaults = python_version < 340
 
     if kw_defaults_before_defaults:
         named_children = (
@@ -501,8 +462,8 @@ class ExpressionFunctionCreation(SideEffectsFromChildrenMixin,
         "kw_defaults" : convertNoneConstantOrEmptyDictToNone,
     }
 
-    def __init__(self, function_ref, defaults, kw_defaults, annotations,
-                 source_ref):
+    def __init__(self, function_ref, code_object, defaults, kw_defaults,
+                 annotations, source_ref):
         assert kw_defaults is None or kw_defaults.isExpression()
         assert annotations is None or annotations.isExpression()
         assert function_ref.isExpressionFunctionRef()
@@ -518,9 +479,60 @@ class ExpressionFunctionCreation(SideEffectsFromChildrenMixin,
             source_ref = source_ref
         )
 
+        self.code_object = code_object
+
+    def getName(self):
+        return self.getFunctionRef().getName()
+
+    def getCodeObject(self):
+        return self.code_object
+
+    def getDetails(self):
+        return {
+            "code_object" : self.code_object
+        }
+
     def computeExpression(self, constraint_collection):
-        # TODO: Function body may know something, creation of defaults may
-        # raise, etc.
+        defaults = self.getDefaults()
+
+        side_effects = []
+
+        for default in defaults:
+            if default.willRaiseException(BaseException):
+                result = wrapExpressionWithSideEffects(
+                    side_effects = side_effects,
+                    old_node     = self,
+                    new_node     = default
+                )
+
+                return result, "new_raise", "Default value contains raise."
+
+        kw_defaults = self.getKwDefaults()
+
+        if kw_defaults is not None:
+            if kw_defaults.willRaiseException(BaseException):
+                result = wrapExpressionWithSideEffects(
+                    side_effects = side_effects,
+                    old_node     = self,
+                    new_node     = kw_defaults
+                )
+
+                return result, "new_raise", "Keyword default values contain raise."
+
+            side_effects.append(kw_defaults)
+
+        annotations = self.getAnnotations()
+
+        if annotations is not None and annotations.willRaiseException(BaseException):
+            result = wrapExpressionWithSideEffects(
+                side_effects = side_effects,
+                old_node     = self,
+                new_node     = annotations
+            )
+
+            return result, "new_raise", "Annotation values contain raise."
+
+        # TODO: Function body may know something too.
         return self, None, None
 
     getFunctionRef = ExpressionChildrenHavingBase.childGetter("function_ref")
@@ -529,19 +541,36 @@ class ExpressionFunctionCreation(SideEffectsFromChildrenMixin,
     getAnnotations = ExpressionChildrenHavingBase.childGetter("annotations")
 
     def mayRaiseException(self, exception_type):
-        return True
+        for default in self.getDefaults():
+            if default.mayRaiseException(exception_type):
+                return True
+
+        kw_defaults = self.getKwDefaults()
+
+        if kw_defaults is not None and kw_defaults.mayRaiseException(exception_type):
+            return True
+
+        annotations = self.getAnnotations()
+
+        if annotations is not None and annotations.mayRaiseException(exception_type):
+            return True
+
+        return False
+
+    def computeExpressionCall(self, call_node, call_args, call_kw,
+                              constraint_collection):
+
+        constraint_collection.onExceptionRaiseExit(BaseException)
 
-    def computeExpressionCall(self, call_node, constraint_collection):
         # TODO: Until we have something to re-order the keyword arguments, we
         # need to skip this. For the immediate need, we avoid this complexity,
         # as a re-ordering will be needed.
-        call_kw = call_node.getCallKw()
-
-        # TODO: empty constant can happen too
-        if call_kw is not None:
+        if call_kw is not None and \
+           (not call_kw.isExpressionConstantRef() or call_kw.getConstant() != {}):
             return call_node, None, None
 
-        call_args = call_node.getCallArgs()
+        if call_kw is not None:
+            return call_node, None, None
 
         if call_args is None:
             args_tuple = ()
@@ -555,12 +584,13 @@ class ExpressionFunctionCreation(SideEffectsFromChildrenMixin,
 
             return call_node, None, None
 
+        function_body = self.getFunctionRef().getFunctionBody()
 
         # TODO: Actually the above disables it entirely, as it is at least
         # the empty dictionary node in any case. We will need some enhanced
         # interfaces for "matchCall" to work on.
 
-        call_spec = self.getFunctionRef().getFunctionBody().getParameters()
+        call_spec = function_body.getParameters()
 
         try:
             args_dict = matchCall(
@@ -573,15 +603,11 @@ class ExpressionFunctionCreation(SideEffectsFromChildrenMixin,
                 pairs         = ()
             )
 
-            values = []
-
-            for positional_arg in args_tuple:
-                for arg_value in args_dict.values():
-                    if arg_value is positional_arg:
-                        values.append(arg_value)
-                        break
-                else:
-                    assert False
+            values = [
+                args_dict[name]
+                for name in
+                call_spec.getParameterNames()
+            ]
 
             result = ExpressionFunctionCall(
                 function   = self,
@@ -598,11 +624,6 @@ function call.""" % self.getName()
             )
 
         except TooManyArguments as e:
-            from .NodeMakingHelpers import (
-                makeRaiseExceptionReplacementExpressionFromInstance,
-                wrapExpressionWithSideEffects
-            )
-
             result = wrapExpressionWithSideEffects(
                 new_node     = makeRaiseExceptionReplacementExpressionFromInstance(
                     expression = call_node,
@@ -614,17 +635,53 @@ function call.""" % self.getName()
 
             return (
                 result,
-                "new_statements,new_raise", # TODO: More appropriate tag maybe.
+                "new_raise", # TODO: More appropriate tag maybe.
                 """Replaced call to created function body '%s' to argument \
 error""" % self.getName()
             )
 
+    def getCallCost(self, values):
+        # TODO: Ought to use values. If they are all constant, how about we
+        # assume no cost, pylint: disable=W0613
+
+        if not Options.isExperimental():
+            return None
+
+        function_body = self.getFunctionRef().getFunctionBody()
+
+        if function_body.isExpressionGeneratorObjectBody():
+            # TODO: That's not even allowed, is it?
+            assert False
+
+            return None
+
+        if function_body.isExpressionClassBody():
+            return None
+
+        # TODO: Lying for the demo, this is too limiting, but needs frames to
+        # be allowed twice in a context.
+        if function_body.mayRaiseException(BaseException):
+            return 60
+
+        return 20
+
+    def createOutlineFromCall(self, provider, values):
+        return convertFunctionCallToOutline(
+            provider     = provider,
+            function_ref = self.getFunctionRef(),
+            values       = values
+        )
+
+
 
 class ExpressionFunctionRef(NodeBase, ExpressionMixin):
     kind = "EXPRESSION_FUNCTION_REF"
 
     def __init__(self, function_body, source_ref):
-        assert function_body.isExpressionFunctionBody()
+        assert function_body.isExpressionFunctionBody() or \
+               function_body.isExpressionClassBody() or \
+               function_body.isExpressionGeneratorObjectBody() or \
+               function_body.isExpressionCoroutineObjectBody()
 
         NodeBase.__init__(
             self,
@@ -633,17 +690,19 @@ class ExpressionFunctionRef(NodeBase, ExpressionMixin):
 
         self.function_body = function_body
 
+    def getName(self):
+        return self.function_body.getName()
+
     def getDetails(self):
+        return {
+            "function_body" : self.function_body
+        }
+
+    def getDetailsForDisplay(self):
         return {
             "function" : self.function_body.getCodeName()
         }
 
-    def makeCloneAt(self, source_ref):
-        return ExpressionFunctionRef(
-            function_body = self.function_body,
-            source_ref    = source_ref
-        )
-
     def getFunctionBody(self):
         return self.function_body
 
@@ -652,6 +711,9 @@ class ExpressionFunctionRef(NodeBase, ExpressionMixin):
 
         owning_module = function_body.getParentModule()
 
+        # Make sure the owning module is added to the used set. This is most
+        # important for helper functions, or modules, which otherwise have
+        # become unused.
         from nuitka.ModuleRegistry import addUsedModule
         addUsedModule(owning_module)
 
@@ -697,6 +759,13 @@ class ExpressionFunctionRef(NodeBase, ExpressionMixin):
 
 
 class ExpressionFunctionCall(ExpressionChildrenHavingBase):
+    """ Shared function call.
+
+        This is for calling created function bodies with multiple users. Not
+        clear if such a thing should exist. But what this will do is to have
+        respect for the fact that there are multiple such calls.
+    """
+
     kind = "EXPRESSION_FUNCTION_CALL"
 
     named_children = (
@@ -719,24 +788,37 @@ class ExpressionFunctionCall(ExpressionChildrenHavingBase):
     def computeExpression(self, constraint_collection):
         function = self.getFunction()
 
-        if function.willRaiseException(BaseException):
-            return function, "new_raise", "Called function is a raise"
+        values = self.getArgumentValues()
+
+        # TODO: This needs some design.
+        cost = function.getCallCost(values)
+
+        if function.getFunctionRef().getFunctionBody().mayRaiseException(BaseException):
+            constraint_collection.onExceptionRaiseExit(BaseException)
+
+        if cost is not None and cost < 50:
+            result = function.createOutlineFromCall(
+                provider = self.getParentVariableProvider(),
+                values   = values
+            )
+
+            return result, "new_statements", "Function call in-lined."
+
+        return self, None, None
+
+    def mayRaiseException(self, exception_type):
+        function = self.getFunction()
+
+        if function.getFunctionRef().getFunctionBody().mayRaiseException(exception_type):
+            return True
 
         values = self.getArgumentValues()
 
-        for count, value in enumerate(values):
-            if value.willRaiseException(BaseException):
-                from .NodeMakingHelpers import wrapExpressionWithSideEffects
+        for value in values:
+            if value.mayRaiseException(exception_type):
+                return True
 
-                result = wrapExpressionWithSideEffects(
-                    side_effects = [function] + list(values[:count]),
-                    new_node     = value,
-                    old_node     = self
-                )
-
-                return result, "new_raise", "Called function arguments raise"
-
-        return self, None, None
+        return False
 
     getFunction = ExpressionChildrenHavingBase.childGetter("function")
     getArgumentValues = ExpressionChildrenHavingBase.childGetter("values")
@@ -753,8 +835,6 @@ class ExpressionFunctionQualnameRef(CompileTimeConstantExpressionMixin,
         self.function_body = function_body
 
     def computeExpression(self, constraint_collection):
-        from .NodeMakingHelpers import makeConstantReplacementNode
-
         result = makeConstantReplacementNode(
             node     = self,
             constant = self.function_body.getFunctionQualname()
