@@ -1,4 +1,4 @@
-#     Copyright 2015, Kay Hayen, mailto:kay.hayen@gmail.com
+#     Copyright 2016, Kay Hayen, mailto:kay.hayen@gmail.com
 #
 #     Part of "Nuitka", an optimizing Python compiler that is compatible and
 #     integrates with CPython, but also works on its own.
@@ -29,7 +29,10 @@ compile time constant.
 
 from logging import warning
 
-from nuitka.importing.Importing import findModule
+from nuitka.importing.Importing import (
+    findModule,
+    getModuleNameAndKindFromFilename
+)
 from nuitka.importing.Recursion import decideRecursion, recurseTo
 from nuitka.importing.Whitelisting import getModuleWhiteList
 from nuitka.utils import Utils
@@ -51,6 +54,8 @@ class ExpressionImportModule(NodeBase, ExpressionMixin):
     _warned_about = set()
 
     def __init__(self, module_name, import_list, level, source_ref):
+        assert type(module_name) is str, type(module_name)
+
         NodeBase.__init__(
             self,
             source_ref = source_ref
@@ -102,21 +107,7 @@ class ExpressionImportModule(NodeBase, ExpressionMixin):
 
         module_filename = Utils.normpath(module_filename)
 
-        if Utils.isDir(module_filename):
-            module_name = Utils.basename(module_filename)
-            module_kind = "py"
-        elif module_filename.endswith(".py"):
-            module_name = Utils.basename(module_filename)[:-3]
-            module_kind = "py"
-        elif module_filename.endswith(".so"):
-            module_kind = "shlib"
-            module_name = Utils.basename(module_filename)[:-3]
-        elif module_filename.endswith(".pyd"):
-            module_kind = "shlib"
-            module_name = Utils.basename(module_filename)[:-4]
-        else:
-            module_kind = None
-            module_name = None
+        module_name, module_kind = getModuleNameAndKindFromFilename(module_filename)
 
         if module_kind is not None:
             decision, reason = decideRecursion(
@@ -172,13 +163,13 @@ Not recursing to '%(full_path)s' (%(filename)s), please specify \
 
         parent_module = self.getParentModule()
 
-        if parent_module.isPythonPackage():
+        if parent_module.isCompiledPythonPackage():
             parent_package = parent_module.getFullName()
         else:
             parent_package = self.getParentModule().getPackage()
 
         module_package, module_filename, _finding = findModule(
-            source_ref     = self.source_ref,
+            importing      = self,
             module_name    = self.getModuleName(),
             parent_package = parent_package,
             level          = self.getLevel(),
@@ -198,13 +189,13 @@ Not recursing to '%(full_path)s' (%(filename)s), please specify \
 
                 import_list = self.getImportList()
 
-                if import_list and imported_module.isPythonPackage():
+                if import_list and imported_module.isCompiledPythonPackage():
                     for import_item in import_list:
                         if import_item == '*':
                             continue
 
                         module_package, module_filename, _finding = findModule(
-                            source_ref     = self.source_ref,
+                            importing      = self,
                             module_name    = import_item,
                             parent_package = imported_module.getFullName(),
                             level          = -1,
@@ -238,8 +229,14 @@ Not recursing to '%(full_path)s' (%(filename)s), please specify \
             for found_module in self.found_modules:
                 addUsedModule(found_module)
 
-        # TODO: May return a module reference of some sort in the future with
-        # embedded modules.
+
+        # When a module is recursed to and included, we know it won't raise,
+        # right? But even if you import, that successful import may still raise
+        # and we don't know how to check yet.
+        constraint_collection.onExceptionRaiseExit(
+            BaseException
+        )
+
         return self, None, None
 
 
@@ -339,18 +336,39 @@ class ExpressionBuiltinImport(ExpressionChildrenHavingBase):
         if module_name.isExpressionConstantRef() and \
            fromlist.isExpressionConstantRef() and \
            level.isExpressionConstantRef():
-            new_node = ExpressionImportModule(
-                module_name = module_name.getConstant(),
-                import_list = fromlist.getConstant(),
-                level       = level.getConstant(),
-                source_ref  = self.getSourceReference()
-            )
 
-            return (
-                new_node,
-                "new_import",
-                "Replaced __import__ call with module import expression."
-            )
+            if module_name.isStringConstant():
+                new_node = ExpressionImportModule(
+                    module_name = module_name.getConstant(),
+                    import_list = fromlist.getConstant(),
+                    level       = level.getConstant(),
+                    source_ref  = self.getSourceReference()
+                )
+
+                # Importing may raise an exception obviously.
+                constraint_collection.onExceptionRaiseExit(BaseException)
+
+
+                return (
+                    new_node,
+                    "new_import",
+                    "Replaced __import__ call with module import expression."
+                )
+            else:
+                # Non-strings is going to raise an error.
+                new_node, change_tags, message = constraint_collection.getCompileTimeComputationResult(
+                    node        = self,
+                    computation = lambda : __import__(module_name.getConstant()),
+                    description = "Replaced __import__ call with non-string module name argument."
+                )
+
+                # Must fail, must not go on when it doesn't.
+                assert change_tags == "new_raise"
+
+                return new_node, change_tags, message
+
+        # Importing may raise an exception obviously.
+        constraint_collection.onExceptionRaiseExit(BaseException)
 
         # TODO: May return a module or module variable reference of some sort in
         # the future with embedded modules.
