@@ -1,4 +1,4 @@
-#     Copyright 2015, Kay Hayen, mailto:kay.hayen@gmail.com
+#     Copyright 2016, Kay Hayen, mailto:kay.hayen@gmail.com
 #
 #     Part of "Nuitka", an optimizing Python compiler that is compatible and
 #     integrates with CPython, but also works on its own.
@@ -25,12 +25,18 @@ import re
 
 from nuitka import Options, Variables
 from nuitka.containers.oset import OrderedSet
-from nuitka.importing import Importing
+from nuitka.importing.Importing import (
+    findModule,
+    getModuleNameAndKindFromFilename
+)
+from nuitka.importing.Recursion import decideRecursion, recurseTo
 from nuitka.optimizations.TraceCollections import ConstraintCollectionModule
-from nuitka.SourceCodeReferences import SourceCodeReference
+from nuitka.PythonVersions import python_version
+from nuitka.SourceCodeReferences import SourceCodeReference, fromFilename
 from nuitka.utils import Utils
 
 from .Checkers import checkStatementsSequenceOrNone
+from .CodeObjectSpecs import CodeObjectSpec
 from .ConstantRefNodes import ExpressionConstantRef
 from .FutureSpecs import FutureSpec
 from .NodeBases import (
@@ -74,40 +80,49 @@ class PythonModuleMixin:
 
     def attemptRecursion(self):
         # Make sure the package is recursed to.
-        from nuitka.importing import Recursion
 
         # Return the list of newly added modules.
         result = []
 
         if self.package_name is not None and self.package is None:
-            package_package, package_filename, _finding = \
-              Importing.findModule(
-                source_ref     = self.getSourceReference(),
+            package_package, package_filename, _finding = findModule(
+                importing      = self,
                 module_name    = self.package_name,
                 parent_package = None,
                 level          = 1,
-                warn           = Utils.python_version < 330
+                warn           = python_version < 330
             )
 
             # TODO: Temporary, if we can't find the package for Python3.3 that
             # is semi-OK, maybe.
-            if Utils.python_version >= 330 and not package_filename:
+            if python_version >= 330 and not package_filename:
                 return []
 
             assert package_filename is not None, self.package_name
 
-            imported_module, is_added = Recursion.recurseTo(
-                module_package  = package_package,
+            _package_name, package_kind = getModuleNameAndKindFromFilename(package_filename)
+            # assert _package_name == self.package_name, (package_filename, _package_name, self.package_name)
+
+            decision, _reason = decideRecursion(
                 module_filename = package_filename,
-                module_relpath  = Utils.relpath(package_filename),
-                module_kind     = "py",
-                reason          = "Containing package of recursed module.",
+                module_name     = self.package_name,
+                module_package  = package_package,
+                module_kind     = package_kind
             )
 
-            self.package = imported_module
+            if decision is not None:
+                imported_module, is_added = recurseTo(
+                    module_package  = package_package,
+                    module_filename = package_filename,
+                    module_relpath  = Utils.relpath(package_filename),
+                    module_kind     = "py",
+                    reason          = "Containing package of recursed module '%s'." % self.getFullName(),
+                )
 
-            if is_added:
-                result.append(imported_module)
+                self.package = imported_module
+
+                if is_added:
+                    result.append(imported_module)
 
         if self.package:
             from nuitka.ModuleRegistry import addUsedModule
@@ -142,7 +157,7 @@ class PythonModuleMixin:
             current = filename
 
             levels = full_name.count('.')
-            if self.isPythonPackage():
+            if self.isCompiledPythonPackage():
                 levels += 1
 
             for _i in range(levels):
@@ -152,15 +167,13 @@ class PythonModuleMixin:
             return result
 
 
-class PythonModule(PythonModuleMixin, ChildrenHavingMixin,
-                   ClosureGiverNodeBase):
-    """ Module
+class CompiledPythonModule(PythonModuleMixin, ChildrenHavingMixin,
+                           ClosureGiverNodeBase):
+    """ Compiled Python Module
 
-        The module is the only possible root of a tree. When there are many
-        modules they form a forest.
     """
 
-    kind = "PYTHON_MODULE"
+    kind = "COMPILED_PYTHON_MODULE"
 
     named_children = (
         "body",
@@ -184,7 +197,6 @@ class PythonModule(PythonModuleMixin, ChildrenHavingMixin,
             package_name = package_name
         )
 
-
         ChildrenHavingMixin.__init__(
             self,
             values = {
@@ -192,7 +204,7 @@ class PythonModule(PythonModuleMixin, ChildrenHavingMixin,
             },
         )
 
-        self.variables = set()
+        self.variables = {}
 
         # The list functions contained in that module.
         self.functions = OrderedSet()
@@ -203,6 +215,18 @@ class PythonModule(PythonModuleMixin, ChildrenHavingMixin,
         # SSA trace based information about the module.
         self.constraint_collection = None
 
+        self.code_object = CodeObjectSpec(
+            code_name     = "<module>" if self.isMainModule() else self.getName(),
+            code_kind     = "Module",
+            arg_names     = (),
+            kw_only_count = 0,
+            has_stardict  = False,
+            has_starlist  = False,
+        )
+
+    def getCodeObject(self):
+        return self.code_object
+
     def getDetails(self):
         return {
             "filename" : self.source_ref.getFilename(),
@@ -211,18 +235,60 @@ class PythonModule(PythonModuleMixin, ChildrenHavingMixin,
         }
 
     def asXml(self):
-        result = super(PythonModule, self).asXml()
+        result = super(CompiledPythonModule, self).asXml()
 
         for function_body in self.active_functions:
             result.append(function_body.asXml())
 
         return result
 
+    def asGraph(self, computation_counter):
+        from graphviz import Digraph # @UnresolvedImport pylint: disable=F0401,I0021
+
+        graph = Digraph("cluster_%d" % computation_counter, comment = "Graph for %s" % self.getName())
+        graph.body.append("style=filled")
+        graph.body.append("color=lightgrey")
+        graph.body.append("label=Iteration_%d" % computation_counter)
+
+
+        def makeTraceNodeName(variable_trace):
+            return "%d/ %s %s %s" % (
+                computation_counter,
+                variable_trace.getVariable(),
+                variable_trace.getVersion(),
+                variable_trace.__class__.__name__
+            )
+
+        for function_body in self.active_functions:
+            constraint_collection = function_body.constraint_collection
+
+            for (_variable, _version), variable_trace in constraint_collection.getVariableTracesAll().items():
+                node = makeTraceNodeName(variable_trace)
+
+                previous = variable_trace.getPrevious()
+
+                if variable_trace.hasDefiniteUsages():
+                    graph.attr("node", style = "filled", color = "blue")
+                elif variable_trace.hasPotentialUsages():
+                    graph.attr("node", style = "filled", color = "yellow")
+                else:
+                    graph.attr("node", style = "filled", color = "red")
+
+                graph.node(node)
+
+                if type(previous) is tuple:
+                    for previous in previous:
+                        graph.edge(makeTraceNodeName(previous), node)
+                elif previous is not None:
+                    graph.edge(makeTraceNodeName(previous), node)
+
+        return graph
+
     getBody = ChildrenHavingMixin.childGetter("body")
     setBody = ChildrenHavingMixin.childSetter("body")
 
     @staticmethod
-    def isPythonModule():
+    def isCompiledPythonModule():
         return True
 
     def getParent(self):
@@ -231,8 +297,11 @@ class PythonModule(PythonModuleMixin, ChildrenHavingMixin,
     def getParentVariableProvider(self):
         return None
 
+    def hasVariableName(self, variable_name):
+        return variable_name in self.variables or variable_name in self.temp_variables
+
     def getVariables(self):
-        return self.variables
+        return self.variables.values()
 
     def getFilename(self):
         return self.source_ref.getFilename()
@@ -249,13 +318,14 @@ class PythonModule(PythonModuleMixin, ChildrenHavingMixin,
         )
 
     def createProvidedVariable(self, variable_name):
+        assert variable_name not in self.variables
+
         result = Variables.ModuleVariable(
             module        = self,
             variable_name = variable_name
         )
 
-        assert result not in self.variables
-        self.variables.add(result)
+        self.variables[variable_name] = result
 
         return result
 
@@ -296,7 +366,10 @@ class PythonModule(PythonModuleMixin, ChildrenHavingMixin,
     def addUsedFunction(self, function_body):
         assert function_body in self.functions
 
-        assert function_body.isExpressionFunctionBody()
+        assert function_body.isExpressionFunctionBody() or \
+               function_body.isExpressionClassBody() or \
+               function_body.isExpressionGeneratorObjectBody() or \
+               function_body.isExpressionCoroutineObjectBody()
 
         if function_body not in self.active_functions:
             self.active_functions.add(function_body)
@@ -374,6 +447,95 @@ class PythonModule(PythonModuleMixin, ChildrenHavingMixin,
 
         return False
 
+
+class CompiledPythonPackage(CompiledPythonModule):
+    kind = "COMPILED_PYTHON_PACKAGE"
+
+    def __init__(self, name, package_name, source_ref):
+        assert name
+
+        CompiledPythonModule.__init__(
+            self,
+            name         = name,
+            package_name = package_name,
+            source_ref   = source_ref
+        )
+
+    def getOutputFilename(self):
+        return Utils.dirname(self.getFilename())
+
+
+def makeUncompiledPythonModule(module_name, filename, bytecode, is_package,
+                               user_provided):
+    parts = module_name.rsplit('.', 1)
+    name = parts[-1]
+
+    package_name = parts[0] if len(parts) == 2 else None
+    source_ref = fromFilename(filename)
+
+    if is_package:
+        return UncompiledPythonPackage(
+            name          = name,
+            package_name  = package_name,
+            bytecode      = bytecode,
+            filename      = filename,
+            user_provided = user_provided,
+            source_ref    = source_ref
+        )
+    else:
+        return UncompiledPythonModule(
+            name          = name,
+            package_name  = package_name,
+            bytecode      = bytecode,
+            filename      = filename,
+            user_provided = user_provided,
+            source_ref    = source_ref
+        )
+
+
+class UncompiledPythonModule(PythonModuleMixin, NodeBase):
+    """ Compiled Python Module
+
+    """
+
+    kind = "UNCOMPILED_PYTHON_MODULE"
+
+    def __init__(self, name, package_name, bytecode, filename, user_provided,
+                 source_ref):
+        NodeBase.__init__(
+            self,
+            source_ref = source_ref
+        )
+
+        PythonModuleMixin.__init__(
+            self,
+            name         = name,
+            package_name = package_name
+        )
+
+        self.bytecode = bytecode
+        self.filename = filename
+
+        self.user_provided = user_provided
+
+    def isUserProvided(self):
+        return self.user_provided
+
+    def getByteCode(self):
+        return self.bytecode
+
+    @staticmethod
+    def isPackage():
+        return False
+
+class UncompiledPythonPackage(UncompiledPythonModule):
+    kind = "UNCOMPILED_PYTHON_PACKAGE"
+
+    @staticmethod
+    def isPackage():
+        return True
+
+
 class SingleCreationMixin:
     created = set()
 
@@ -382,11 +544,11 @@ class SingleCreationMixin:
         self.created.add(self.__class__)
 
 
-class PythonMainModule(PythonModule, SingleCreationMixin):
+class PythonMainModule(CompiledPythonModule, SingleCreationMixin):
     kind = "PYTHON_MAIN_MODULE"
 
     def __init__(self, main_added, source_ref):
-        PythonModule.__init__(
+        CompiledPythonModule.__init__(
             self,
             name         = "__main__",
             package_name = None,
@@ -405,14 +567,14 @@ class PythonMainModule(PythonModule, SingleCreationMixin):
         if self.main_added:
             return Utils.dirname(self.getFilename())
         else:
-            return PythonModule.getOutputFilename(self)
+            return CompiledPythonModule.getOutputFilename(self)
 
 
-class PythonInternalModule(PythonModule, SingleCreationMixin):
+class PythonInternalModule(CompiledPythonModule, SingleCreationMixin):
     kind = "PYTHON_INTERNAL_MODULE"
 
     def __init__(self):
-        PythonModule.__init__(
+        CompiledPythonModule.__init__(
             self,
             name         = "__internal__",
             package_name = None,
@@ -433,26 +595,11 @@ class PythonInternalModule(PythonModule, SingleCreationMixin):
         return "__internal"
 
 
-class PythonPackage(PythonModule):
-    kind = "PYTHON_PACKAGE"
-
-    def __init__(self, name, package_name, source_ref):
-        assert name
-
-        PythonModule.__init__(
-            self,
-            name         = name,
-            package_name = package_name,
-            source_ref   = source_ref
-        )
-
-    def getOutputFilename(self):
-        return Utils.dirname(self.getFilename())
-
-
 class PythonShlibModule(PythonModuleMixin, NodeBase):
     kind = "PYTHON_SHLIB_MODULE"
 
+    avoid_duplicates = set()
+
     def __init__(self, name, package_name, source_ref):
         NodeBase.__init__(
             self,
@@ -465,11 +612,17 @@ class PythonShlibModule(PythonModuleMixin, NodeBase):
             package_name = package_name
         )
 
+        # That would be a mistake we just made.
         assert Utils.basename(source_ref.getFilename()) != "<frozen>"
 
         # That is too likely a bug.
         assert name != "__main__"
 
+        # Duplicates should be avoided by us caching elsewhere before creating
+        # the object.
+        assert self.getFullName() not in self.avoid_duplicates, self.getFullName()
+        self.avoid_duplicates.add(self.getFullName())
+
     def getDetails(self):
         return {
             "name"         : self.name,
