@@ -1,4 +1,4 @@
-#     Copyright 2014, Kay Hayen, mailto:kay.hayen@gmail.com
+#     Copyright 2016, Kay Hayen, mailto:kay.hayen@gmail.com
 #
 #     Part of "Nuitka", an optimizing Python compiler that is compatible and
 #     integrates with CPython, but also works on its own.
@@ -21,7 +21,9 @@ These are for use in optimizations and computations, and therefore cover
 mostly exceptions and constants.
 
 Often cyclic dependencies kicks in, which is why this module is mostly only
-imported locally.
+imported locally. Note: It's intended to be reversed, this module will make
+the local imports instead, as these local imports look ugly everywhere else,
+making it more difficult to use.
 """
 
 from logging import warning
@@ -30,19 +32,10 @@ from nuitka.Builtins import builtin_names
 from nuitka.Constants import isConstant
 from nuitka.Options import isDebug, shallWarnImplicitRaises
 
-from .BuiltinRefNodes import ExpressionBuiltinExceptionRef, ExpressionBuiltinRef
-from .ComparisonNodes import (
-    ExpressionComparison,
-    ExpressionComparisonIs,
-    ExpressionComparisonIsNOT
-)
-from .ConstantRefNodes import ExpressionConstantRef
-from .ExceptionNodes import ExpressionRaiseException
-from .SideEffectNodes import ExpressionSideEffects
-from .StatementNodes import StatementExpressionOnly, StatementsSequence
-
 
 def makeConstantReplacementNode(constant, node):
+    from .ConstantRefNodes import ExpressionConstantRef
+
     return ExpressionConstantRef(
         constant   = constant,
         source_ref = node.getSourceReference()
@@ -50,14 +43,19 @@ def makeConstantReplacementNode(constant, node):
 
 def makeRaiseExceptionReplacementExpression(expression, exception_type,
                                             exception_value):
+    from .ExceptionNodes import ExpressionRaiseException
+    from .BuiltinRefNodes import ExpressionBuiltinExceptionRef
+
     source_ref = expression.getSourceReference()
 
-    assert type( exception_type ) is str
+    assert type(exception_type) is str
 
     if shallWarnImplicitRaises():
         warning(
-            "%s: Static exception raise",
+            "%s: Will always raise exception '%s(%s)'",
             expression.getSourceReference().getAsString(),
+            exception_type,
+            exception_value
         )
 
     result = ExpressionRaiseException(
@@ -74,11 +72,12 @@ def makeRaiseExceptionReplacementExpression(expression, exception_type,
 
     return result
 
+
 def makeRaiseExceptionReplacementExpressionFromInstance(expression, exception):
     assert isinstance(exception, Exception)
 
     args = exception.args
-    if type(args) is tuple and len( args ) == 1:
+    if type(args) is tuple and len(args) == 1:
         value = args[0]
     else:
         assert type(args) is tuple
@@ -90,27 +89,21 @@ def makeRaiseExceptionReplacementExpressionFromInstance(expression, exception):
         exception_value = value
     )
 
-def isCompileTimeConstantValue(value):
-    # This needs to match code in makeCompileTimeConstantReplacementNode
-    if isConstant( value ):
-        return True
-    elif type( value ) is type:
-        return True
-    else:
-        return False
 
 def makeCompileTimeConstantReplacementNode(value, node):
     # This needs to match code in isCompileTimeConstantValue
-    if isConstant( value ):
+    if isConstant(value):
         return makeConstantReplacementNode(
             constant = value,
             node     = node
         )
-    elif type( value ) is type:
+    elif type(value) is type:
         if value.__name__ in builtin_names:
+            from .BuiltinRefNodes import ExpressionBuiltinRef
+
             return ExpressionBuiltinRef(
                 builtin_name = value.__name__,
-                source_ref    = node.getSourceReference()
+                source_ref   = node.getSourceReference()
             )
         else:
             return node
@@ -133,7 +126,7 @@ def getComputationResult(node, computation, description):
         )
 
         change_tags = "new_raise"
-        change_desc = description + " Was predicted to raise an exception."
+        change_desc = description + " Predicted to raise an exception."
     else:
         new_node = makeCompileTimeConstantReplacementNode(
             value = result,
@@ -141,29 +134,37 @@ def getComputationResult(node, computation, description):
         )
 
         if isDebug():
-            assert new_node is not node, ( node, result )
+            assert new_node is not node, (node, result)
 
         if new_node is not node:
             change_tags = "new_constant"
-            change_desc = description + " Was predicted to constant result."
+            change_desc = description + " Predicted constant result."
         else:
             change_tags = None
             change_desc = None
 
     return new_node, change_tags, change_desc
 
+
 def makeStatementExpressionOnlyReplacementNode(expression, node):
+    from .StatementNodes import StatementExpressionOnly
+
     return StatementExpressionOnly(
         expression = expression,
         source_ref = node.getSourceReference()
     )
 
-def mergeStatements(statements):
+
+def mergeStatements(statements, allow_none = False):
     """ Helper function that merges nested statement sequences. """
     merged_statements = []
 
     for statement in statements:
-        if statement.isStatement() or statement.isStatementsFrame():
+        if statement is None and allow_none:
+            pass
+        elif type(statement) in (tuple, list):
+            merged_statements += mergeStatements(statement, allow_none)
+        elif statement.isStatement() or statement.isStatementsFrame():
             merged_statements.append(statement)
         elif statement.isStatementsSequence():
             merged_statements.extend(mergeStatements(statement.getStatements()))
@@ -172,7 +173,10 @@ def mergeStatements(statements):
 
     return merged_statements
 
+
 def makeStatementsSequenceReplacementNode(statements, node):
+    from .StatementNodes import StatementsSequence
+
     return StatementsSequence(
         statements = mergeStatements(statements),
         source_ref = node.getSourceReference()
@@ -189,6 +193,8 @@ def convertNoneConstantToNone(node):
 def wrapExpressionWithSideEffects(side_effects, old_node, new_node):
     assert new_node.isExpression()
 
+    from .SideEffectNodes import ExpressionSideEffects
+
     if side_effects:
         new_node = ExpressionSideEffects(
             expression   = new_node,
@@ -211,6 +217,8 @@ def wrapStatementWithSideEffects(new_node, old_node, allow_none = False):
     side_effects = old_node.extractSideEffects()
 
     if side_effects:
+        from .StatementNodes import StatementExpressionOnly
+
         side_effects = tuple(
             StatementExpressionOnly(
                 expression = side_effect,
@@ -221,7 +229,7 @@ def wrapStatementWithSideEffects(new_node, old_node, allow_none = False):
 
         if new_node is not None:
             new_node = makeStatementsSequenceReplacementNode(
-                statements = side_effects + ( new_node, ),
+                statements = side_effects + (new_node,),
                 node       = old_node
             )
         else:
@@ -233,6 +241,8 @@ def wrapStatementWithSideEffects(new_node, old_node, allow_none = False):
     return new_node
 
 def makeStatementOnlyNodesFromExpressions(expressions):
+    from .StatementNodes import StatementExpressionOnly, StatementsSequence
+
     statements = tuple(
         StatementExpressionOnly(
             expression = expression,
@@ -243,31 +253,80 @@ def makeStatementOnlyNodesFromExpressions(expressions):
 
     if not statements:
         return None
-    elif len( statements ) == 1:
-        return statements[ 0 ]
+    elif len(statements) == 1:
+        return statements[0]
     else:
         return StatementsSequence(
             statements = statements,
             source_ref = statements[0].getSourceReference()
         )
 
+
 def makeComparisonNode(left, right, comparator, source_ref):
+    from .ComparisonNodes import (
+        ExpressionComparison,
+        ExpressionComparisonIs,
+        ExpressionComparisonIsNOT
+    )
+
     if comparator == "Is":
-        return ExpressionComparisonIs(
+        result = ExpressionComparisonIs(
             left       = left,
             right      = right,
             source_ref = source_ref
         )
     elif comparator == "IsNot":
-        return ExpressionComparisonIsNOT(
+        result = ExpressionComparisonIsNOT(
                 left       = left,
                 right      = right,
                 source_ref = source_ref
             )
     else:
-        return ExpressionComparison(
+        result = ExpressionComparison(
             left       = left,
             right      = right,
             comparator = comparator,
             source_ref = source_ref
         )
+
+    result.setCompatibleSourceReference(
+        source_ref = right.getCompatibleSourceReference()
+    )
+
+    return result
+
+
+def makeVariableRefNode(variable, source_ref):
+    if variable.isTempVariable():
+        from .VariableRefNodes import ExpressionTempVariableRef
+
+        return ExpressionTempVariableRef(
+            variable   = variable,
+            source_ref = source_ref
+        )
+    else:
+        from .VariableRefNodes import ExpressionVariableRef
+
+        return ExpressionVariableRef(
+            variable_name = variable.getName(),
+            variable      = variable,
+            source_ref    = source_ref
+        )
+
+
+def makeVariableTargetRefNode(variable, source_ref):
+    if variable.isTempVariable():
+        from .AssignNodes import ExpressionTargetTempVariableRef
+
+        return ExpressionTargetTempVariableRef(
+            variable   = variable,
+            source_ref = source_ref
+        )
+    else:
+        from .AssignNodes import ExpressionTargetVariableRef
+
+        return ExpressionTargetVariableRef(
+            variable_name = variable.getName(),
+            variable      = variable,
+            source_ref    = source_ref
+        )
