@@ -1,4 +1,4 @@
-#     Copyright 2015, Kay Hayen, mailto:kay.hayen@gmail.com
+#     Copyright 2016, Kay Hayen, mailto:kay.hayen@gmail.com
 #
 #     Part of "Nuitka", an optimizing Python compiler that is compatible and
 #     integrates with CPython, but also works on its own.
@@ -30,7 +30,7 @@ flexible.
 """
 
 from nuitka import Variables
-from nuitka.utils import Utils
+from nuitka.PythonVersions import python_version
 
 
 class TooManyArguments(Exception):
@@ -43,105 +43,7 @@ class TooManyArguments(Exception):
         return self.real_exception
 
 
-class ParameterSpecTuple:
-    def __init__(self, normal_args, nest_count = 1):
-        self.normal_args = tuple(normal_args)
-        self.nest_count = nest_count
-
-        self.owner = None
-        self.normal_variables = None
-
-    def getNormalParameters(self):
-        return self.normal_variables
-
-    def setOwner(self, owner):
-        assert self.owner is None
-
-        self.owner = owner
-
-        self.normal_variables = []
-
-        for count, normal_arg in enumerate(self.normal_args):
-            if type(normal_arg) == str:
-                normal_variable = Variables.ParameterVariable(
-                    owner          = self.owner,
-                    parameter_name = normal_arg,
-                    kw_only        = False
-                )
-            elif type(normal_arg) == tuple:
-                sub_parameter_spec = ParameterSpecTuple(
-                    normal_args = normal_arg,
-                    nest_count  = self.nest_count + 1
-                )
-                sub_parameter_spec.setOwner(self.owner)
-
-                sub_parameter_name = "Unpackable_%s_%s" % (
-                    self.nest_count,
-                    count+1
-                )
-
-                normal_variable = Variables.NestedParameterVariable(
-                    owner          = self.owner,
-                    parameter_name = sub_parameter_name,
-                    parameter_spec = sub_parameter_spec
-                )
-            else:
-                assert False, normal_arg
-
-            self.normal_variables.append(normal_variable)
-
-
-    def getVariables(self):
-        result = []
-
-        for variable in self.normal_variables:
-            if variable.isNestedParameterVariable():
-                result += variable.getVariables()
-            else:
-                result.append(variable)
-
-        return result
-
-    def hasNestedParameterVariables(self):
-        for variable in self.normal_variables:
-            if variable.isNestedParameterVariable():
-                return True
-        return False
-
-    def getAllVariables(self):
-        result = self.normal_variables[:]
-
-        for variable in self.normal_variables:
-            if variable.isNestedParameterVariable():
-                result += variable.getAllVariables()
-
-        return result
-
-    def getAllNames(self):
-        result = []
-
-        def extractArg(normal_arg):
-            if type(normal_arg) is str:
-                result.append(normal_arg)
-            elif type(normal_arg) is tuple:
-                for normal_arg in normal_arg:
-                    extractArg(normal_arg)
-            else:
-                assert False
-
-        for normal_arg in self.normal_args:
-            extractArg(normal_arg)
-
-        return result
-
-    def getTopLevelVariables(self):
-        return self.normal_variables
-
-    def getParameterNames(self):
-        return Variables.getNames(self.getVariables())
-
-
-class ParameterSpec(ParameterSpecTuple):
+class ParameterSpec:
     # These got many attributes, in part duplicating name and instance of
     # variables, pylint: disable=R0902
 
@@ -149,11 +51,11 @@ class ParameterSpec(ParameterSpecTuple):
                  dict_star_arg, default_count):
         assert None not in normal_args
 
+        self.owner = None
+
         self.name = name
-
-        self.nest_count = 1
-
-        ParameterSpecTuple.__init__(self, normal_args)
+        self.normal_args = tuple(normal_args)
+        self.normal_variables = None
 
         assert list_star_arg is None or type(list_star_arg) is str, \
           list_star_arg
@@ -171,17 +73,32 @@ class ParameterSpec(ParameterSpecTuple):
         self.kw_only_args = tuple(kw_only_args)
         self.kw_only_variables = None
 
+    def makeClone(self):
+        return ParameterSpec(
+            name          = self.name,
+            normal_args   = self.normal_args,
+            kw_only_args  = self.kw_only_args,
+            list_star_arg = self.list_star_arg,
+            dict_star_arg = self.dict_star_arg,
+            default_count = self.default_count
+        )
+
     def checkValid(self):
-        arg_names = self.getAllNames()
+        arg_names = self.getParameterNames()
 
         # Check for duplicate arguments, could happen.
         for arg_name in arg_names:
             if arg_names.count(arg_name) != 1:
                 return "duplicate argument '%s' in function definition" % arg_name
+
         return None
 
     def __repr__(self):
-        parts = [ str(normal_arg) for normal_arg in self.normal_args ]
+        parts = [
+            str(normal_arg)
+            for normal_arg
+            in self.normal_args
+        ]
 
         if self.list_star_arg is not None:
             parts.append("*%s" % self.list_star_arg)
@@ -201,65 +118,58 @@ class ParameterSpec(ParameterSpecTuple):
         if self.owner is not None:
             return
 
-        ParameterSpecTuple.setOwner(self, owner)
+        self.owner = owner
+        self.normal_variables = []
+
+        for normal_arg in self.normal_args:
+            if type(normal_arg) is str:
+                normal_variable = Variables.ParameterVariable(
+                    owner          = self.owner,
+                    parameter_name = normal_arg
+                )
+            else:
+                assert False, normal_arg
+
+            self.normal_variables.append(normal_variable)
 
         if self.list_star_arg:
-            self.list_star_variable = Variables.ParameterVariable(owner, self.list_star_arg, False)
+            self.list_star_variable = Variables.ParameterVariable(
+                owner          = owner,
+                parameter_name = self.list_star_arg
+            )
         else:
             self.list_star_variable = None
 
         if self.dict_star_arg:
             self.dict_star_variable = Variables.ParameterVariable(
                 owner          = owner,
-                parameter_name = self.dict_star_arg,
-                kw_only        = False
+                parameter_name = self.dict_star_arg
             )
         else:
             self.dict_star_variable = None
 
         self.kw_only_variables = [
-            Variables.ParameterVariable(self.owner, kw_only_arg, True)
+            Variables.ParameterVariable(
+                owner          = self.owner,
+                parameter_name = kw_only_arg
+            )
             for kw_only_arg in
             self.kw_only_args
         ]
 
-    def isEmpty(self):
-        return len(self.normal_args) == 0 and self.list_star_arg is None and \
-               self.dict_star_arg is None and len(self.kw_only_args) == 0
-
-    def getDefaultParameterVariables(self):
-        result = ParameterSpecTuple.getTopLevelVariables(self)
-
-        return result[ len(self.normal_args) - self.default_count : ]
-
-    def getDefaultParameterNames(self):
-        return self.normal_args[ \
-            len(self.normal_args) - self.default_count : ]
-
     def getDefaultCount(self):
         return self.default_count
 
     def hasDefaultParameters(self):
         return self.getDefaultCount() > 0
 
-    def getVariables(self):
-        result = ParameterSpecTuple.getVariables(self)[:]
-
-        if self.list_star_variable is not None:
-            result.append(self.list_star_variable)
-
-        if self.dict_star_variable is not None:
-            result.append(self.dict_star_variable)
-
-        return result + self.kw_only_variables
-
     def getTopLevelVariables(self):
-        result = ParameterSpecTuple.getTopLevelVariables(self)
-
-        return result + self.kw_only_variables
+        return self.normal_variables + self.kw_only_variables
 
     def getAllVariables(self):
-        result = ParameterSpecTuple.getAllVariables(self)[:]
+        result = list(self.normal_variables)
+
+        result += self.kw_only_variables
 
         if self.list_star_variable is not None:
             result.append(self.list_star_variable)
@@ -267,10 +177,12 @@ class ParameterSpec(ParameterSpecTuple):
         if self.dict_star_variable is not None:
             result.append(self.dict_star_variable)
 
-        return result + self.kw_only_variables
+        return result
 
-    def getAllNames(self):
-        result = ParameterSpecTuple.getAllNames(self)[:]
+    def getParameterNames(self):
+        result = list(self.normal_args)
+
+        result += self.kw_only_args
 
         if self.list_star_arg is not None:
             result.append(self.list_star_arg)
@@ -278,7 +190,7 @@ class ParameterSpec(ParameterSpecTuple):
         if self.dict_star_arg is not None:
             result.append(self.dict_star_arg)
 
-        return result + list(self.kw_only_args)
+        return result
 
     def getStarListArgumentName(self):
         return self.list_star_arg
@@ -311,30 +223,13 @@ class ParameterSpec(ParameterSpecTuple):
     def getKwOnlyParameterCount(self):
         return len(self.kw_only_args)
 
-    def getCoArgNames(self):
-        result = []
-
-        for count, variable in enumerate(self.getTopLevelVariables()):
-            if variable.isNestedParameterVariable():
-                result.append(".%d" % count)
-            else:
-                result.append(variable.getName())
-
-        if self.list_star_variable is not None:
-            result.append(self.list_star_arg)
-
-        if self.dict_star_variable is not None:
-            result.append(self.dict_star_arg)
-
-        return result
-
 
 # Note: Based loosely on "inspect.getcallargs" with corrections.
 def matchCall(func_name, args, star_list_arg, star_dict_arg, num_defaults,
               positional, pairs, improved = False):
     # This is of incredible code complexity, but there really is no other way to
     # express this with less statements, branches, or variables.
-    # pylint: disable=R0914,R0912,R0915
+    # pylint: disable=R0912,R0914,R0915
 
     assert type(positional) is tuple, positional
     assert type(pairs) in (tuple, list), pairs
@@ -354,7 +249,7 @@ def matchCall(func_name, args, star_list_arg, star_dict_arg, num_defaults,
             # Tuple argument case:
 
             assigned_tuple_params.append(arg)
-            value = iter(value)
+            value = iter(value.getIterationValues())
 
             for i, subarg in enumerate(arg):
                 try:
@@ -396,7 +291,7 @@ def matchCall(func_name, args, star_list_arg, star_dict_arg, num_defaults,
         assign(arg, value)
 
     # Python3 does this check earlier.
-    if Utils.python_version >= 300 and not star_dict_arg:
+    if python_version >= 300 and not star_dict_arg:
         for pair in pairs:
             if pair[0] not in args:
                 message = "'%s' is an invalid keyword argument for this function" % pair[0]
@@ -412,7 +307,16 @@ def matchCall(func_name, args, star_list_arg, star_dict_arg, num_defaults,
             assign(star_list_arg, ())
     elif 0 < num_args < num_total:
         if num_defaults == 0:
-            if num_args != 1:
+            if num_args == 1:
+                raise TooManyArguments(
+                    TypeError(
+                        "%s() takes exactly one argument (%d given)" % (
+                            func_name,
+                            num_total
+                        )
+                    )
+                )
+            else:
                 raise TooManyArguments(
                     TypeError(
                         "%s expected %d arguments, got %d" % (
@@ -423,15 +327,6 @@ def matchCall(func_name, args, star_list_arg, star_dict_arg, num_defaults,
                     )
                 )
 
-            raise TooManyArguments(
-                TypeError(
-                    "%s() takes exactly %s (%d given)" % (
-                        func_name,
-                        "one argument" if num_args == 1 else "%d arguments" % num_args,
-                        num_total
-                    )
-                )
-            )
         else:
             raise TooManyArguments(
                 TypeError(
@@ -542,7 +437,7 @@ def matchCall(func_name, args, star_list_arg, star_dict_arg, num_defaults,
                 )
 
             if num_required == 1:
-                arg_desc = "1 argument"
+                arg_desc = "1 argument" if python_version < 350 else "one argument"
             else:
                 arg_desc = "%d arguments" % num_required
 
@@ -561,7 +456,7 @@ def matchCall(func_name, args, star_list_arg, star_dict_arg, num_defaults,
                 TypeError(
                     "%s expected %s%s, got %d" % (
                         func_name,
-                        ( "at least " if Utils.python_version < 300 else "" )
+                        ( "at least " if python_version < 300 else "" )
                             if num_defaults > 0
                         else "exactly ",
                         "%d arguments" % num_required,
