@@ -1,4 +1,4 @@
-#     Copyright 2015, Kay Hayen, mailto:kay.hayen@gmail.com
+#     Copyright 2016, Kay Hayen, mailto:kay.hayen@gmail.com
 #
 #     Part of "Nuitka", an optimizing Python compiler that is compatible and
 #     integrates with CPython, but also works on its own.
@@ -18,7 +18,7 @@
 """ The type1 node.
 
 This one just determines types. It's great for optimization. We may be able to
-predict its value, but knowing it. In that case, we have a builtin name
+predict its value, but knowing it. In that case, we have a built-in name
 reference for that type to convert to, or when checking the result of it, we
 will then know it's limited after the fact.
 
@@ -26,11 +26,11 @@ will then know it's limited after the fact.
 
 from nuitka.Builtins import builtin_names
 
+from .BuiltinRefNodes import ExpressionBuiltinAnonymousRef, ExpressionBuiltinRef
 from .NodeBases import (
     ExpressionBuiltinSingleArgBase,
     ExpressionChildrenHavingBase
 )
-from .NodeMakingHelpers import getComputationResult
 
 
 class ExpressionBuiltinType1(ExpressionBuiltinSingleArgBase):
@@ -44,10 +44,6 @@ class ExpressionBuiltinType1(ExpressionBuiltinSingleArgBase):
 
             type_name = value.__class__.__name__
 
-            from .BuiltinRefNodes import (
-                ExpressionBuiltinAnonymousRef,
-                ExpressionBuiltinRef
-            )
 
             if type_name in builtin_names:
                 new_node = ExpressionBuiltinRef(
@@ -82,6 +78,12 @@ class ExpressionBuiltinType1(ExpressionBuiltinSingleArgBase):
         return result, "new_statements", """\
 Removed type taking for unused result."""
 
+    def mayRaiseException(self, exception_type):
+        return self.getValue().mayRaiseException(exception_type)
+
+    def mayHaveSideEffects(self):
+        return self.getValue().mayHaveSideEffects()
+
 
 class ExpressionBuiltinSuper(ExpressionChildrenHavingBase):
     kind = "EXPRESSION_BUILTIN_SUPER"
@@ -105,6 +107,8 @@ class ExpressionBuiltinSuper(ExpressionChildrenHavingBase):
     getObject = ExpressionChildrenHavingBase.childGetter("object")
 
     def computeExpression(self, constraint_collection):
+        constraint_collection.onExceptionRaiseExit(BaseException)
+
         # TODO: Quite some cases should be possible to predict.
         return self, None, None
 
@@ -139,15 +143,19 @@ class ExpressionBuiltinIsinstance(ExpressionChildrenHavingBase):
         # TODO: Should be possible to query run time type instead, but we don't
         # have that method yet. Later this will be essential.
         if not instance.isCompileTimeConstant():
+            constraint_collection.onExceptionRaiseExit(BaseException)
+
             return self, None, None
 
         cls = self.getCls()
 
         if not cls.isCompileTimeConstant():
+            constraint_collection.onExceptionRaiseExit(BaseException)
+
             return self, None, None
 
         # So if both are compile time constant, we are able to compute it.
-        return getComputationResult(
+        return constraint_collection.getCompileTimeComputationResult(
             node        = self,
             computation = lambda : isinstance(
                 instance.getCompileTimeConstant(),
