@@ -1,4 +1,4 @@
-#     Copyright 2015, Kay Hayen, mailto:kay.hayen@gmail.com
+#     Copyright 2016, Kay Hayen, mailto:kay.hayen@gmail.com
 #
 #     Part of "Nuitka", an optimizing Python compiler that is compatible and
 #     integrates with CPython, but also works on its own.
@@ -56,15 +56,17 @@ class ExpressionVariableRef(NodeBase, ExpressionMixin):
 
         self.variable_trace = None
 
+        self.global_trace = None
+
     def getDetails(self):
         if self.variable is None:
             return {
-                "name" : self.variable_name
+                "variable_name" : self.variable_name
             }
         else:
             return {
-                "name"     : self.variable_name,
-                "variable" : self.variable
+                "variable_name" : self.variable_name,
+                "variable"      : self.variable
             }
 
     def getDetail(self):
@@ -73,16 +75,6 @@ class ExpressionVariableRef(NodeBase, ExpressionMixin):
         else:
             return repr(self.variable)
 
-    def makeCloneAt(self, source_ref):
-        result = self.__class__(
-            variable_name = self.variable_name,
-            source_ref    = source_ref
-        )
-
-        result.variable = self.variable
-
-        return result
-
     @staticmethod
     def isTargetVariableRef():
         return False
@@ -95,7 +87,6 @@ class ExpressionVariableRef(NodeBase, ExpressionMixin):
 
     def setVariable(self, variable):
         assert isinstance(variable, Variables.Variable), repr(variable)
-        assert self.variable is None
 
         self.variable = variable
 
@@ -111,15 +102,30 @@ class ExpressionVariableRef(NodeBase, ExpressionMixin):
         replacement = self.variable_trace.getReplacementNode(self)
 
         if replacement is not None:
-            return replacement, "new_expression", "Value propagated for '%s'." % self.variable.getName()
+            constraint_collection.signalChange(
+                "new_expression",
+                self.source_ref,
+                "Value propagated for '%s' from '%s'." % (
+                    self.variable.getName(),
+                    replacement.getSourceReference().getAsString()
+                )
+            )
 
-        global_trace = VariableRegistry.getGlobalVariableTrace(variable)
+            # Need to compute the replacement still.
+            return replacement.computeExpression(constraint_collection)
 
+        if not self.variable_trace.mustHaveValue():
+            # TODO: This could be way more specific surely.
+            constraint_collection.onExceptionRaiseExit(
+                BaseException
+            )
+
+        self.global_trace = VariableRegistry.getGlobalVariableTrace(variable)
 
         # TODO: Maybe local variables are factored into this strangely.
-        if global_trace is None and variable.isModuleVariable():
+        if self.global_trace is None and variable.isModuleVariable():
             constraint_collection.assumeUnclearLocals()
-        elif (variable.isModuleVariable() and not global_trace.hasDefiniteWrites() ) or \
+        elif (variable.isModuleVariable() and not self.global_trace.hasDefiniteWrites() ) or \
              variable.isMaybeLocalVariable():
             if self.variable_name in Builtins.builtin_exception_names:
                 from .BuiltinRefNodes import ExpressionBuiltinExceptionRef
@@ -168,6 +174,8 @@ Replaced read-only module attribute '__name__' with constant value."""
                 change_desc = """\
 Replaced read-only module attribute '__package__' with constant value."""
             else:
+                self.variable_trace.addUsage()
+
                 # Probably should give a warning once about it.
                 new_node = self
                 change_tags = None
@@ -175,8 +183,25 @@ Replaced read-only module attribute '__package__' with constant value."""
 
             return new_node, change_tags, change_desc
 
+        self.variable_trace.addUsage()
+
         return self, None, None
 
+    def computeExpressionCall(self, call_node, call_args, call_kw,
+                              constraint_collection):
+
+        constraint_collection.onExceptionRaiseExit(BaseException)
+
+        constraint_collection.onControlFlowEscape(self)
+
+        if self.global_trace is None and \
+           self.variable_name in ("dir", "eval", "exec", "execfile", "locals", "vars") and \
+           self.variable.isModuleVariable():
+            # Just inform the collection that all escaped.
+            constraint_collection.onLocalsUsage()
+
+        return call_node, None, None
+
     def onContentEscapes(self, constraint_collection):
         constraint_collection.onVariableContentEscapes(self.variable)
 
@@ -196,25 +221,26 @@ class ExpressionTempVariableRef(NodeBase, ExpressionMixin):
     kind = "EXPRESSION_TEMP_VARIABLE_REF"
 
     def __init__(self, variable, source_ref):
+        assert variable.isTempVariable()
+
         NodeBase.__init__(self, source_ref = source_ref)
 
         self.variable = variable
         self.variable_trace = None
 
-    def getDetails(self):
+    def getDetailsForDisplay(self):
         return {
             "name" : self.variable.getName()
         }
 
+    def getDetails(self):
+        return {
+            "variable" : self.variable
+        }
+
     def getDetail(self):
         return self.variable.getName()
 
-    def makeCloneAt(self, source_ref):
-        return self.__class__(
-            variable   = self.variable,
-            source_ref = source_ref
-        )
-
     def getVariableName(self):
         return self.variable.getName()
 
@@ -235,6 +261,7 @@ class ExpressionTempVariableRef(NodeBase, ExpressionMixin):
         if replacement is not None:
             return replacement, "new_expression", "Value propagated for temp '%s'." % self.variable.getName()
 
+        self.variable_trace.addUsage()
         # Nothing to do here.
         return self, None, None
 
@@ -260,83 +287,3 @@ class ExpressionTempVariableRef(NodeBase, ExpressionMixin):
     # Python3 only, it updates temporary variables that are closure variables.
     def setVariable(self, variable):
         self.variable = variable
-
-
-class ExpressionTargetVariableRef(ExpressionVariableRef):
-    kind = "EXPRESSION_TARGET_VARIABLE_REF"
-
-    # TODO: Remove default and correct argument order later.
-    def __init__(self, variable_name, source_ref, variable = None):
-        ExpressionVariableRef.__init__(self, variable_name, source_ref)
-
-        self.variable_version = None
-
-        # TODO: Remove setVariable, once not needed anymore and in-line to
-        # here.
-        if variable is not None:
-            self.setVariable(variable)
-            assert variable.getName() == variable_name
-
-    def getDetails(self):
-        if self.variable is None:
-            return {
-                "name" : self.variable_name
-            }
-        else:
-            return {
-                "name"     : self.variable_name,
-                "variable" : self.variable,
-                "version"  : self.variable_version
-            }
-
-    def makeCloneAt(self, source_ref):
-        result = self.__class__(
-            variable_name = self.variable_name,
-            source_ref    = source_ref,
-            variable      = self.variable
-        )
-
-        return result
-
-    def computeExpression(self, constraint_collection):
-        assert False
-
-    @staticmethod
-    def isTargetVariableRef():
-        return True
-
-    def getVariableVersion(self):
-        assert self.variable_version is not None, self
-
-        return self.variable_version
-
-    def setVariable(self, variable):
-        ExpressionVariableRef.setVariable(self, variable)
-
-        self.variable_version = variable.allocateTargetNumber()
-        assert self.variable_version is not None
-
-
-class ExpressionTargetTempVariableRef(ExpressionTempVariableRef):
-    kind = "EXPRESSION_TARGET_TEMP_VARIABLE_REF"
-
-    def __init__(self, variable, source_ref):
-        ExpressionTempVariableRef.__init__(self, variable, source_ref)
-
-        self.variable_version = variable.allocateTargetNumber()
-
-    def computeExpression(self, constraint_collection):
-        assert False, self.parent
-
-    @staticmethod
-    def isTargetVariableRef():
-        return True
-
-    def getVariableVersion(self):
-        return self.variable_version
-
-    # Python3 only, it updates temporary variables that are closure variables.
-    def setVariable(self, variable):
-        ExpressionTempVariableRef.setVariable(self, variable)
-
-        self.variable_version = self.variable.allocateTargetNumber()
