@@ -1,4 +1,4 @@
-#     Copyright 2014, Kay Hayen, mailto:kay.hayen@gmail.com
+#     Copyright 2016, Kay Hayen, mailto:kay.hayen@gmail.com
 #
 #     Part of "Nuitka", an optimizing Python compiler that is compatible and
 #     integrates with CPython, but also works on its own.
@@ -22,15 +22,17 @@ source code comments with developer manual sections.
 
 """
 
-from nuitka import Utils
 from nuitka.nodes.AssignNodes import (
+    ExpressionTargetTempVariableRef,
+    ExpressionTempVariableRef,
     StatementAssignmentVariable,
-    StatementDelVariable
+    StatementReleaseVariable
 )
 from nuitka.nodes.BuiltinIteratorNodes import (
     ExpressionBuiltinIter1,
     ExpressionBuiltinNext1
 )
+from nuitka.nodes.CodeObjectSpecs import CodeObjectSpec
 from nuitka.nodes.ConditionalNodes import StatementConditional
 from nuitka.nodes.ConstantRefNodes import ExpressionConstantRef
 from nuitka.nodes.ContainerOperationNodes import (
@@ -38,142 +40,55 @@ from nuitka.nodes.ContainerOperationNodes import (
     ExpressionListOperationAppend,
     ExpressionSetOperationAdd
 )
+from nuitka.nodes.FrameNodes import StatementsFrame
 from nuitka.nodes.FunctionNodes import (
     ExpressionFunctionBody,
     ExpressionFunctionCall,
     ExpressionFunctionCreation,
     ExpressionFunctionRef
 )
-from nuitka.nodes.LoopNodes import StatementBreakLoop, StatementLoop
+from nuitka.nodes.GeneratorNodes import (
+    ExpressionGeneratorObjectBody,
+    ExpressionMakeGeneratorObject
+)
+from nuitka.nodes.LoopNodes import StatementLoop, StatementLoopBreak
+from nuitka.nodes.NodeMakingHelpers import (
+    makeVariableRefNode,
+    makeVariableTargetRefNode
+)
+from nuitka.nodes.OutlineNodes import ExpressionOutlineBody
 from nuitka.nodes.ParameterSpecs import ParameterSpec
 from nuitka.nodes.ReturnNodes import StatementReturn
 from nuitka.nodes.StatementNodes import (
     StatementExpressionOnly,
     StatementGeneratorEntry,
-    StatementsFrame,
     StatementsSequence
 )
-from nuitka.nodes.VariableRefNodes import (
-    ExpressionTargetTempVariableRef,
-    ExpressionTempVariableRef,
-    ExpressionVariableRef
-)
 from nuitka.nodes.YieldNodes import ExpressionYield
+from nuitka.PythonVersions import python_version
 
 from .Helpers import (
     buildNode,
     buildNodeList,
     getKind,
     makeStatementsSequenceFromStatement,
-    makeTryFinallyExpression,
-    makeTryFinallyStatement,
-    mergeStatements,
-    wrapTryFinallyLater
+    mergeStatements
 )
 from .ReformulationAssignmentStatements import buildAssignmentStatements
 from .ReformulationBooleanExpressions import buildAndNode
 from .ReformulationTryExceptStatements import makeTryExceptSingleHandlerNode
+from .ReformulationTryFinallyStatements import makeTryFinallyStatement
 
 
 def buildListContractionNode(provider, node, source_ref):
     # List contractions are dealt with by general code.
 
-    if Utils.python_version < 300:
-        temp_scope = provider.allocateTempScope("listcontr")
-
-        outer_iter_var = provider.allocateTempVariable(
-            temp_scope = temp_scope,
-            name       = "listcontr_iter"
-        )
-
-        outer_iter_ref = ExpressionTempVariableRef(
-            variable      = outer_iter_var,
-            source_ref    = source_ref
-        )
-
-        container_tmp   = provider.allocateTempVariable(
-            temp_scope = temp_scope,
-            name       = "listcontr_result"
-        )
-
-        statements, del_statements = _buildContractionBodyNode(
-            provider        = provider,
-            node            = node,
-            emit_class      = ExpressionListOperationAppend,
-            start_value     = ExpressionConstantRef(
-                constant   = [],
-                source_ref = source_ref
-            ),
-            outer_iter_ref  = outer_iter_ref,
-            container_tmp   = container_tmp,
-            temp_scope      = temp_scope,
-            assign_provider = True,
-            source_ref      = source_ref,
-            function_body   = provider
-        )
-
-        statements.insert(
-            0,
-            StatementAssignmentVariable(
-                variable_ref = ExpressionTargetTempVariableRef(
-                    variable   = outer_iter_var,
-                    source_ref = source_ref
-                ),
-                source       = ExpressionBuiltinIter1(
-                    value      = buildNode(
-                        provider   = provider,
-                        node       = node.generators[0].iter,
-                        source_ref = source_ref
-                    ),
-                    source_ref = source_ref
-                ),
-                source_ref  = source_ref
-            )
-        )
-
-        result = makeTryFinallyExpression(
-            expression = ExpressionTempVariableRef(
-                variable   = container_tmp,
-                source_ref = source_ref
-            ),
-            tried      = statements,
-            final      = del_statements,
-            source_ref = source_ref
-        )
-
-        final = StatementsSequence(
-            statements = (
-                StatementDelVariable(
-                    variable_ref = ExpressionTargetTempVariableRef(
-                        variable   = container_tmp,
-                        source_ref = source_ref
-                    ),
-                    tolerant     = True,
-                    source_ref   = source_ref
-                ),
-                StatementDelVariable(
-                    variable_ref = ExpressionTargetTempVariableRef(
-                        variable   = outer_iter_var,
-                        source_ref = source_ref
-                    ),
-                    tolerant     = True,
-                    source_ref   = source_ref
-                ),
-            ),
-            source_ref = source_ref
-        )
-
-        wrapTryFinallyLater(
-            node  = result,
-            final = final
-        )
-
-        return result
-
     return _buildContractionNode(
         provider        = provider,
         node            = node,
-        name            = "<listcontraction>",
+        name            = "list_contraction"
+                            if python_version < 300 else
+                          "<listcontraction>",
         emit_class      = ExpressionListOperationAppend,
         start_value     = ExpressionConstantRef(
             constant   = [],
@@ -181,7 +96,7 @@ def buildListContractionNode(provider, node, source_ref):
         ),
         # Note: For Python3, the list contractions no longer assign to the outer
         # scope.
-        assign_provider = Utils.python_version < 300,
+        assign_provider = python_version < 300,
         source_ref      = source_ref
     )
 
@@ -207,16 +122,16 @@ def buildDictContractionNode(provider, node, source_ref):
     # Dict contractions are dealt with by general code.
 
     return _buildContractionNode(
-        provider         = provider,
-        node             = node,
-        name             = "<dictcontraction>",
-        emit_class       = ExpressionDictOperationSet,
-        start_value      = ExpressionConstantRef(
+        provider        = provider,
+        node            = node,
+        name            = "<dictcontraction>",
+        emit_class      = ExpressionDictOperationSet,
+        start_value     = ExpressionConstantRef(
             constant   = {},
             source_ref = source_ref
         ),
-        assign_provider  = False,
-        source_ref       = source_ref
+        assign_provider = False,
+        source_ref      = source_ref
     )
 
 
@@ -237,28 +152,54 @@ def buildGeneratorExpressionNode(provider, node, source_ref):
 
 
 def _buildContractionBodyNode(provider, node, emit_class, start_value,
-                              container_tmp, outer_iter_ref, temp_scope,
-                              assign_provider, source_ref, function_body):
+                              container_tmp, iter_tmp, temp_scope,
+                              assign_provider, function_body, source_ref):
+
+    # This uses lots of variables and branches. There is no good way
+    # around that, and we deal with many cases, due to having generator
+    # expressions sharing this code, pylint: disable=R0912,R0914
+    tmp_variables = []
+
+    if assign_provider:
+        tmp_variables.append(iter_tmp)
+
+    if container_tmp is not None:
+        tmp_variables.append(container_tmp)
+
+    # First assign the iterator if we are an outline.
+    if assign_provider:
+        statements = [
+            StatementAssignmentVariable(
+                variable_ref = makeVariableTargetRefNode(
+                    variable   = iter_tmp,
+                    source_ref = source_ref
+                ),
+                source       = ExpressionBuiltinIter1(
+                    value      = buildNode(
+                        provider   = provider,
+                        node       = node.generators[0].iter,
+                        source_ref = source_ref
+                    ),
+                    source_ref = source_ref
+                ),
+                source_ref   = source_ref.atInternal()
+            )
+        ]
+    else:
+        statements = []
 
     if start_value is not None:
-        statements = [
+        statements.append(
             StatementAssignmentVariable(
                 variable_ref = ExpressionTargetTempVariableRef(
                     variable   = container_tmp,
                     source_ref = source_ref
                 ),
-                source     = start_value,
-                source_ref = source_ref.atInternal()
+                source       = start_value,
+                source_ref   = source_ref.atInternal()
             )
-        ]
+        )
 
-        if assign_provider:
-            tmp_variables = []
-        else:
-            tmp_variables = [container_tmp]
-    else:
-        statements = []
-        tmp_variables = []
 
     if hasattr(node, "elt"):
         if start_value is not None:
@@ -277,8 +218,6 @@ def _buildContractionBodyNode(provider, node, emit_class, start_value,
         else:
             assert emit_class is ExpressionYield
 
-            function_body.markAsGenerator()
-
             current_body = emit_class(
                 buildNode(
                     provider   = function_body,
@@ -290,17 +229,17 @@ def _buildContractionBodyNode(provider, node, emit_class, start_value,
     else:
         assert emit_class is ExpressionDictOperationSet
 
-        current_body = emit_class(
-            ExpressionTempVariableRef(
+        current_body = ExpressionDictOperationSet(
+            dict_arg   = ExpressionTempVariableRef(
                 variable   = container_tmp,
                 source_ref = source_ref
             ),
-            key = buildNode(
+            key        = buildNode(
                 provider   = function_body,
                 node       = node.key,
                 source_ref = source_ref,
             ),
-            value = buildNode(
+            value      = buildNode(
                 provider   = function_body,
                 node       = node.value,
                 source_ref = source_ref,
@@ -313,7 +252,7 @@ def _buildContractionBodyNode(provider, node, emit_class, start_value,
         source_ref = source_ref
     )
 
-    for count, qual in enumerate(reversed( node.generators)):
+    for count, qual in enumerate(reversed(node.generators)):
         tmp_value_variable = function_body.allocateTempVariable(
             temp_scope = temp_scope,
             name       = "iter_value_%d" % count
@@ -324,9 +263,12 @@ def _buildContractionBodyNode(provider, node, emit_class, start_value,
         # The first iterated value is to be calculated outside of the function
         # and will be given as a parameter "_iterated", the others are built
         # inside the function.
+
         if qual is node.generators[0]:
-            def makeIteratorRef():
-                return outer_iter_ref.makeCloneAt(source_ref)
+            iterator_ref = makeVariableRefNode(
+                variable   = iter_tmp,
+                source_ref = source_ref
+            )
 
             tmp_iter_variable = None
 
@@ -355,39 +297,33 @@ def _buildContractionBodyNode(provider, node, emit_class, start_value,
                         variable   = tmp_iter_variable,
                         source_ref = source_ref
                     ),
-                    source     = value_iterator,
-                    source_ref = source_ref
+                    source       = value_iterator,
+                    source_ref   = source_ref
                 )
             ]
 
-            def makeIteratorRef():
-                return ExpressionTempVariableRef(
-                    variable   = tmp_iter_variable,
-                    source_ref = source_ref
-                )
+            iterator_ref = ExpressionTempVariableRef(
+                variable   = tmp_iter_variable,
+                source_ref = source_ref
+            )
 
         loop_statements = [
             makeTryExceptSingleHandlerNode(
-                tried          = makeStatementsSequenceFromStatement(
-                    statement = StatementAssignmentVariable(
-                        variable_ref = ExpressionTargetTempVariableRef(
-                            variable   = tmp_value_variable,
-                            source_ref = source_ref
-                        ),
-                        source     = ExpressionBuiltinNext1(
-                            value      = makeIteratorRef(),
-                            source_ref = source_ref
-                        ),
+                tried          = StatementAssignmentVariable(
+                    variable_ref = ExpressionTargetTempVariableRef(
+                        variable   = tmp_value_variable,
                         source_ref = source_ref
-                    )
+                    ),
+                    source       = ExpressionBuiltinNext1(
+                        value      = iterator_ref,
+                        source_ref = source_ref
+                    ),
+                    source_ref   = source_ref
                 ),
                 exception_name = "StopIteration",
-                handler_body   = makeStatementsSequenceFromStatement(
-                    statement = StatementBreakLoop(
-                        source_ref = source_ref.atInternal()
-                    )
+                handler_body   = StatementLoopBreak(
+                    source_ref = source_ref.atInternal()
                 ),
-                public_exc     = False,
                 source_ref     = source_ref
             ),
             buildAssignmentStatements(
@@ -408,22 +344,10 @@ def _buildContractionBodyNode(provider, node, emit_class, start_value,
             source_ref = source_ref
         )
 
-        if len(conditions) == 1:
+        if len(conditions) >= 1:
             loop_statements.append(
                 StatementConditional(
-                    condition  = conditions[0],
-                    yes_branch = makeStatementsSequenceFromStatement(
-                        statement = current_body
-                    ),
-                    no_branch  = None,
-                    source_ref = source_ref
-                )
-            )
-        elif len(conditions) > 1:
-            loop_statements.append(
-                StatementConditional(
-                    condition = buildAndNode(
-                        provider   = function_body,
+                    condition  = buildAndNode(
                         values     = conditions,
                         source_ref = source_ref
                     ),
@@ -449,18 +373,14 @@ def _buildContractionBodyNode(provider, node, emit_class, start_value,
 
         if tmp_iter_variable is not None:
             nested_statements.append(
-                StatementDelVariable(
-                    variable_ref = ExpressionTargetTempVariableRef(
-                        variable   = tmp_iter_variable,
-                        source_ref = source_ref
-                    ),
-                    tolerant   = False,
+                StatementReleaseVariable(
+                    variable   = tmp_iter_variable,
                     source_ref = source_ref
                 )
             )
 
         current_body = StatementsSequence(
-            statements = nested_statements,
+            statements = mergeStatements(nested_statements, False),
             source_ref = source_ref
         )
 
@@ -475,70 +395,116 @@ def _buildContractionBodyNode(provider, node, emit_class, start_value,
             )
         )
 
-    del_statements = []
-    for tmp_variable in tmp_variables:
-        del_statements.append(
-            StatementDelVariable(
-                variable_ref = ExpressionTargetTempVariableRef(
-                    variable   = tmp_variable,
-                    source_ref = source_ref
-                ),
-                tolerant   = True,
-                source_ref = source_ref.atInternal()
-            )
+    release_statements = [
+        StatementReleaseVariable(
+            variable   = tmp_variable,
+            source_ref = source_ref
         )
+        for tmp_variable in
+        tmp_variables
+    ]
 
-    return statements, del_statements
+    return statements, release_statements
 
 
 def _buildContractionNode(provider, node, name, emit_class, start_value,
                           assign_provider, source_ref):
     # The contraction nodes are reformulated to function bodies, with loops as
     # described in the developer manual. They use a lot of temporary names,
-    # nested blocks, etc. and so a lot of variable names. There is no good way
-    # around that, and we deal with many cases, due to having generator
-    # expressions sharing this code, pylint: disable=R0912,R0914
+    # nested blocks, etc. and so a lot of variable names.
 
     # Note: The assign_provider is only to cover Python2 list contractions,
     # assigning one of the loop variables to the outside scope.
+    if assign_provider:
+        function_body = ExpressionOutlineBody(
+            provider   = provider,
+            name       = name,
+            source_ref = source_ref
+        )
 
-    assert provider.isParentVariableProvider(), provider
+        iter_tmp = function_body.allocateTempVariable(
+            temp_scope = None,
+            name       = ".0"
+        )
+    else:
+        # TODO: No function ought to be necessary.
 
-    function_body = ExpressionFunctionBody(
-        provider   = provider,
-        name       = name,
-        doc        = None,
-        parameters = ParameterSpec(
-            name          = "contraction",
-            normal_args   = ("__iterator",),
-            list_star_arg = None,
-            dict_star_arg = None,
-            default_count = 0,
-            kw_only_args  = ()
-        ),
-        source_ref = source_ref
+        function_body = ExpressionFunctionBody(
+            provider   = provider,
+            name       = name,
+            doc        = None,
+            parameters = ParameterSpec(
+                name          = name,
+                normal_args   = (".0",),
+                list_star_arg = None,
+                dict_star_arg = None,
+                default_count = 0,
+                kw_only_args  = ()
+            ),
+            flags      = set(),
+            source_ref = source_ref
+        )
+
+        iter_tmp = function_body.getVariableForAssignment(
+            variable_name = ".0"
+        )
+        assert iter_tmp.isParameterVariable()
+
+    code_object = CodeObjectSpec(
+        code_name     = name,
+        code_kind     = "Generator" if emit_class is ExpressionYield else "Function",
+        arg_names     = () if assign_provider else function_body.getParameters().getParameterNames(),
+        kw_only_count = 0,
+        has_starlist  = False,
+        has_stardict  = False,
     )
 
+    if emit_class is ExpressionYield:
+        code_body = ExpressionGeneratorObjectBody(
+            provider   = function_body,
+            name       = "<genexpr>",
+            flags      = set(),
+            source_ref = source_ref
+        )
+
+        iter_tmp = code_body.getVariableForReference(
+            variable_name = ".0"
+        )
+        assert iter_tmp.isLocalVariable()
+
+        function_body.setBody(
+            makeStatementsSequenceFromStatement(
+                statement = StatementReturn(
+                    expression = ExpressionMakeGeneratorObject(
+                        generator_ref = ExpressionFunctionRef(
+                            function_body = code_body,
+                            source_ref    = source_ref
+                        ),
+                        code_object   = code_object,
+                        source_ref    = source_ref
+                    ),
+                    source_ref = source_ref
+                )
+            )
+        )
+    else:
+        code_body = function_body
+
     if start_value is not None:
-        container_tmp = function_body.allocateTempVariable(
+        container_tmp = code_body.allocateTempVariable(
             temp_scope = None,
             name       = "contraction_result"
         )
     else:
         container_tmp = None
 
-    outer_iter_ref = ExpressionVariableRef(
-        variable_name = "__iterator",
-        source_ref    = source_ref
-    )
-
-    statements, del_statements = _buildContractionBodyNode(
-        function_body   = function_body,
+    statements, release_statements = _buildContractionBodyNode(
+        function_body   = code_body,
         assign_provider = assign_provider,
         provider        = provider,
         node            = node,
         emit_class      = emit_class,
-        outer_iter_ref  = outer_iter_ref,
+        iter_tmp        = iter_tmp,
         temp_scope      = None,
         start_value     = start_value,
         container_tmp   = container_tmp,
@@ -558,48 +524,50 @@ def _buildContractionNode(provider, node, name, emit_class, start_value,
 
     statements = (
         makeTryFinallyStatement(
+            provider   = function_body,
             tried      = statements,
-            final      = del_statements,
+            final      = release_statements,
             source_ref = source_ref.atInternal()
         ),
     )
 
-    function_body.setBody(
-        StatementsFrame(
-            statements    = statements,
-            guard_mode    = "pass_through"
-                              if emit_class is not ExpressionYield else
-                            "generator",
-            var_names     = (),
-            arg_count     = 0,
-            kw_only_count = 0,
-            has_starlist  = False,
-            has_stardict  = False,
-            code_name     = "contraction",
-            source_ref    = source_ref
+    code_body.setBody(
+        makeStatementsSequenceFromStatement(
+            statement = StatementsFrame(
+                statements  = mergeStatements(statements, False),
+                guard_mode  = "pass_through"
+                                  if emit_class is not ExpressionYield else
+                                "generator",
+                code_object = code_object,
+                source_ref  = source_ref
+            )
         )
     )
 
-    return ExpressionFunctionCall(
-        function   = ExpressionFunctionCreation(
-            function_ref = ExpressionFunctionRef(
-                function_body = function_body,
-                source_ref    = source_ref
+    if not assign_provider:
+        return ExpressionFunctionCall(
+            function   = ExpressionFunctionCreation(
+                function_ref = ExpressionFunctionRef(
+                    function_body = function_body,
+                    source_ref    = source_ref
+                ),
+                code_object  = code_object,
+                defaults     = (),
+                kw_defaults  = None,
+                annotations  = None,
+                source_ref   = source_ref
             ),
-            defaults     = (),
-            kw_defaults  = None,
-            annotations  = None,
-            source_ref   = source_ref
-        ),
-        values     = (
-            ExpressionBuiltinIter1(
-                value      = buildNode(
-                    provider   = provider,
-                    node       = node.generators[0].iter,
+            values     = (
+                ExpressionBuiltinIter1(
+                    value      = buildNode(
+                        provider   = provider,
+                        node       = node.generators[0].iter,
+                        source_ref = source_ref
+                    ),
                     source_ref = source_ref
                 ),
-                source_ref = source_ref
             ),
-        ),
-        source_ref = source_ref
-    )
+            source_ref = source_ref
+        )
+    else:
+        return function_body
