@@ -1,4 +1,4 @@
-#     Copyright 2015, Kay Hayen, mailto:kay.hayen@gmail.com
+#     Copyright 2016, Kay Hayen, mailto:kay.hayen@gmail.com
 #
 #     Part of "Nuitka", an optimizing Python compiler that is compatible and
 #     integrates with CPython, but also works on its own.
@@ -23,6 +23,8 @@ source code comments with developer manual sections.
 """
 
 from nuitka.nodes.AssignNodes import (
+    ExpressionTargetTempVariableRef,
+    ExpressionTempVariableRef,
     StatementAssignmentVariable,
     StatementReleaseVariable
 )
@@ -44,12 +46,7 @@ from nuitka.nodes.GlobalsLocalsNodes import (
     ExpressionBuiltinGlobals,
     ExpressionBuiltinLocals
 )
-from nuitka.nodes.TryFinallyNodes import StatementTryFinally
 from nuitka.nodes.TypeNodes import ExpressionBuiltinIsinstance
-from nuitka.nodes.VariableRefNodes import (
-    ExpressionTargetTempVariableRef,
-    ExpressionTempVariableRef
-)
 
 from .Helpers import (
     buildNode,
@@ -58,6 +55,14 @@ from .Helpers import (
     makeStatementsSequenceFromStatement,
     makeStatementsSequenceFromStatements
 )
+from .ReformulationTryFinallyStatements import makeTryFinallyStatement
+
+
+def _getLocalsClassNode(provider):
+    if provider.isCompiledPythonModule():
+        return ExpressionBuiltinGlobals
+    else:
+        return ExpressionBuiltinLocals
 
 
 def wrapEvalGlobalsAndLocals(provider, globals_node, locals_node,
@@ -92,7 +97,7 @@ def wrapEvalGlobalsAndLocals(provider, globals_node, locals_node,
 
     post_statements = []
 
-    if provider.isExpressionFunctionBody() and provider.isClassDictCreation():
+    if provider.isExpressionClassBody():
         post_statements.append(
             StatementLocalsDictSync(
                 locals_arg = ExpressionTempVariableRef(
@@ -106,12 +111,10 @@ def wrapEvalGlobalsAndLocals(provider, globals_node, locals_node,
     post_statements += [
         StatementReleaseVariable(
             variable   = globals_keeper_variable,
-            tolerant   = False,
             source_ref = source_ref
         ),
         StatementReleaseVariable(
             variable   = locals_keeper_variable,
-            tolerant   = False,
             source_ref = source_ref
         )
     ]
@@ -129,11 +132,11 @@ def wrapEvalGlobalsAndLocals(provider, globals_node, locals_node,
             ),
             source_ref = source_ref
         ),
-        no_expression  = ExpressionTempVariableRef(
+        expression_no  = ExpressionTempVariableRef(
             variable   = globals_keeper_variable,
             source_ref = source_ref
         ),
-        yes_expression = ExpressionBuiltinLocals(
+        expression_yes = _getLocalsClassNode(provider)(
             source_ref = source_ref
         ),
         source_ref     = source_ref
@@ -271,7 +274,7 @@ exec: arg 1 must be a string, file, or code object""",
                 source_ref      = source_ref
             )
 
-    if provider.isExpressionFunctionBody():
+    if not provider.isCompiledPythonModule():
         provider.markAsExecContaining()
 
         if orig_globals is None:
@@ -317,7 +320,7 @@ exec: arg 1 must be a string, file, or code object""",
         name       = "plain"
     )
 
-    tried = makeStatementsSequenceFromStatements(
+    tried = (
         # First evaluate the source code expressions.
         StatementAssignmentVariable(
             variable_ref = ExpressionTargetTempVariableRef(
@@ -399,7 +402,7 @@ exec: arg 1 must be a string, file, or code object""",
                                 variable   = locals_keeper_variable,
                                 source_ref = source_ref
                             ),
-                            source       = ExpressionBuiltinLocals(
+                            source       = _getLocalsClassNode(provider)(
                                 source_ref = source_ref
                             ),
                             source_ref   = source_ref,
@@ -489,84 +492,76 @@ exec: arg 1 must be a string, file, or code object""",
             no_branch  = None,
             source_ref = source_ref
         ),
-        StatementTryFinally(
-            tried      = makeStatementsSequenceFromStatement(
-                statement = StatementExec(
-                    source_code = ExpressionTempVariableRef(
-                        variable   = source_variable,
-                        source_ref = source_ref
-                    ),
-                    globals_arg = ExpressionTempVariableRef(
-                        variable   = globals_keeper_variable,
-                        source_ref = source_ref
-                    ),
-                    locals_arg  = ExpressionTempVariableRef(
-                        variable   = locals_keeper_variable,
-                        source_ref = source_ref
-                    ),
-                    source_ref  = source_ref
-                )
-            ),
-            final      = makeStatementsSequenceFromStatements(
-                StatementConditional(
-                    condition  = ExpressionComparisonIs(
-                        left       = ExpressionTempVariableRef(
-                            variable   = plain_indicator_variable,
-                            source_ref = source_ref
-                        ),
-                        right      = ExpressionConstantRef(
-                            constant   = True,
-                            source_ref = source_ref
-                        ),
-                        source_ref = source_ref
-                    ),
-                    yes_branch = makeStatementsSequenceFromStatement(
-                        statement = StatementLocalsDictSync(
-                            locals_arg = ExpressionTempVariableRef(
-                                variable   = locals_keeper_variable,
-                                source_ref = source_ref,
-                            ),
-                            source_ref = source_ref.atInternal()
-                        )
-                    ),
-                    no_branch  = None,
+        makeTryFinallyStatement(
+            provider   = provider,
+            tried      = StatementExec(
+                source_code = ExpressionTempVariableRef(
+                    variable   = source_variable,
                     source_ref = source_ref
                 ),
+                globals_arg = ExpressionTempVariableRef(
+                    variable   = globals_keeper_variable,
+                    source_ref = source_ref
+                ),
+                locals_arg  = ExpressionTempVariableRef(
+                    variable   = locals_keeper_variable,
+                    source_ref = source_ref
+                ),
+                source_ref  = source_ref
+            ),
+            final      = StatementConditional(
+                condition  = ExpressionComparisonIs(
+                    left       = ExpressionTempVariableRef(
+                        variable   = plain_indicator_variable,
+                        source_ref = source_ref
+                    ),
+                    right      = ExpressionConstantRef(
+                        constant   = True,
+                        source_ref = source_ref
+                    ),
+                    source_ref = source_ref
+                ),
+                yes_branch = makeStatementsSequenceFromStatement(
+                    statement = StatementLocalsDictSync(
+                        locals_arg = ExpressionTempVariableRef(
+                            variable   = locals_keeper_variable,
+                            source_ref = source_ref,
+                        ),
+                        source_ref = source_ref.atInternal()
+                    )
+                ),
+                no_branch  = None,
+                source_ref = source_ref
             ),
-            public_exc = False,
             source_ref = source_ref
         )
     )
 
-    final = makeStatementsSequenceFromStatements(
+    final = (
         StatementReleaseVariable(
             variable   = source_variable,
-            tolerant   = True,
             source_ref = source_ref
         ),
         StatementReleaseVariable(
             variable   = globals_keeper_variable,
-            tolerant   = True,
             source_ref = source_ref
         ),
         StatementReleaseVariable(
             variable   = locals_keeper_variable,
-            tolerant   = True,
             source_ref = source_ref
         ),
         StatementReleaseVariable(
             variable   = plain_indicator_variable,
-            tolerant   = True,
             source_ref = source_ref
         ),
     )
 
-    return StatementTryFinally(
+    return makeTryFinallyStatement(
+        provider   = provider,
         tried      = tried,
         final      = final,
-        public_exc = False,
         source_ref = source_ref
     )
 
 # This is here, to make sure it can register, pylint: disable=W0611
-import nuitka.optimizations.OptimizeBuiltinCalls # isort:skip
+import nuitka.optimizations.OptimizeBuiltinCalls # isort:skip @UnusedImport
