@@ -1,4 +1,4 @@
-#     Copyright 2015, Kay Hayen, mailto:kay.hayen@gmail.com
+#     Copyright 2016, Kay Hayen, mailto:kay.hayen@gmail.com
 #
 #     Part of "Nuitka", an optimizing Python compiler that is compatible and
 #     integrates with CPython, but also works on its own.
@@ -24,18 +24,21 @@ source code comments with developer manual sections.
 
 from nuitka import Options
 from nuitka.nodes.AssignNodes import (
+    ExpressionTargetTempVariableRef,
+    ExpressionTempVariableRef,
     StatementAssignmentVariable,
     StatementReleaseVariable
 )
 from nuitka.nodes.AttributeNodes import (
     ExpressionAttributeLookup,
-    ExpressionSpecialAttributeLookup
+    ExpressionAttributeLookupSpecial
 )
 from nuitka.nodes.CallNodes import ExpressionCallEmpty, ExpressionCallNoKeywords
 from nuitka.nodes.ComparisonNodes import ExpressionComparisonIs
 from nuitka.nodes.ConditionalNodes import StatementConditional
 from nuitka.nodes.ConstantRefNodes import ExpressionConstantRef
 from nuitka.nodes.ContainerMakingNodes import ExpressionMakeTuple
+from nuitka.nodes.CoroutineNodes import ExpressionAsyncWait
 from nuitka.nodes.ExceptionNodes import (
     ExpressionCaughtExceptionTracebackRef,
     ExpressionCaughtExceptionTypeRef,
@@ -45,12 +48,8 @@ from nuitka.nodes.StatementNodes import (
     StatementExpressionOnly,
     StatementsSequence
 )
-from nuitka.nodes.VariableRefNodes import (
-    ExpressionTargetTempVariableRef,
-    ExpressionTempVariableRef
-)
+from nuitka.PythonVersions import python_version
 from nuitka.tree.Helpers import makeReraiseExceptionStatement
-from nuitka.utils import Utils
 
 from .Helpers import (
     buildNode,
@@ -61,14 +60,14 @@ from .Helpers import (
     makeStatementsSequenceFromStatement
 )
 from .ReformulationAssignmentStatements import buildAssignmentStatements
-from .ReformulationTryExceptStatements import makeTryExceptSingleHandlerNode
+from . \
+    ReformulationTryExceptStatements import makeTryExceptSingleHandlerNodeWithPublish
 from .ReformulationTryFinallyStatements import makeTryFinallyStatement
 
 
 def _buildWithNode(provider, context_expr, assign_target, body, body_lineno,
-                   source_ref):
+                   sync, source_ref):
     # Many details, pylint: disable=R0914
-
     with_source = buildNode(provider, context_expr, source_ref)
 
     if Options.isFullCompat():
@@ -114,16 +113,85 @@ def _buildWithNode(provider, context_expr, assign_target, body, body_lineno,
     )
 
     if Options.isFullCompat():
+        if body:
+            deepest = body
+
+            while deepest.getVisitableNodes():
+                deepest = deepest.getVisitableNodes()[-1]
+
+            body_lineno = deepest.getCompatibleSourceReference().getLineNumber()
+
         with_exit_source_ref = source_ref.atLineNumber(body_lineno)
     else:
         with_exit_source_ref = source_ref
 
     # The "__enter__" and "__exit__" were normal attribute lookups under
     # CPython2.6, but that changed with CPython2.7.
-    if Utils.python_version < 270:
+    if python_version < 270:
         attribute_lookup_class = ExpressionAttributeLookup
     else:
-        attribute_lookup_class = ExpressionSpecialAttributeLookup
+        attribute_lookup_class = ExpressionAttributeLookupSpecial
+
+    enter_value = ExpressionCallEmpty(
+        called     = attribute_lookup_class(
+            source         = ExpressionTempVariableRef(
+                variable   = tmp_source_variable,
+                source_ref = source_ref
+            ),
+            attribute_name = "__enter__" if sync else "__aenter__",
+            source_ref     = source_ref
+        ),
+        source_ref = source_ref
+    )
+
+    exit_value_exception = ExpressionCallNoKeywords(
+        called     = ExpressionTempVariableRef(
+            variable   = tmp_exit_variable,
+            source_ref = with_exit_source_ref
+        ),
+        args       = ExpressionMakeTuple(
+            elements   = (
+                ExpressionCaughtExceptionTypeRef(
+                    source_ref = with_exit_source_ref
+                ),
+                ExpressionCaughtExceptionValueRef(
+                    source_ref = with_exit_source_ref
+                ),
+                ExpressionCaughtExceptionTracebackRef(
+                    source_ref = source_ref
+                ),
+            ),
+            source_ref = source_ref
+        ),
+        source_ref = with_exit_source_ref
+    )
+
+    exit_value_no_exception = ExpressionCallNoKeywords(
+        called     = ExpressionTempVariableRef(
+            variable   = tmp_exit_variable,
+            source_ref = source_ref
+        ),
+        args       = ExpressionConstantRef(
+            constant   = (None, None, None),
+            source_ref = source_ref
+        ),
+        source_ref = with_exit_source_ref
+    )
+
+    # For "async with", await the entered value and exit value must be awaited.
+    if not sync:
+        enter_value = ExpressionAsyncWait(
+            expression = enter_value,
+            source_ref = source_ref
+        )
+        exit_value_exception = ExpressionAsyncWait(
+            expression = exit_value_exception,
+            source_ref = source_ref
+        )
+        exit_value_no_exception = ExpressionAsyncWait(
+            expression = exit_value_no_exception,
+            source_ref = source_ref
+        )
 
     statements = [
         # First assign the with context to a temporary variable.
@@ -147,7 +215,7 @@ def _buildWithNode(provider, context_expr, assign_target, body, body_lineno,
                     variable   = tmp_source_variable,
                     source_ref = source_ref
                 ),
-                attribute_name = "__exit__",
+                attribute_name = "__exit__" if sync else "__aexit__",
                 source_ref     = source_ref
             ),
             source_ref   = source_ref
@@ -157,17 +225,7 @@ def _buildWithNode(provider, context_expr, assign_target, body, body_lineno,
                 variable   = tmp_enter_variable,
                 source_ref = source_ref
             ),
-            source       = ExpressionCallEmpty(
-                called     = attribute_lookup_class(
-                    source         = ExpressionTempVariableRef(
-                        variable   = tmp_source_variable,
-                        source_ref = source_ref
-                    ),
-                    attribute_name = "__enter__",
-                    source_ref     = source_ref
-                ),
-                source_ref = source_ref
-            ),
+            source       = enter_value,
             source_ref   = source_ref
         ),
         StatementAssignmentVariable(
@@ -186,7 +244,7 @@ def _buildWithNode(provider, context_expr, assign_target, body, body_lineno,
     statements += [
         makeTryFinallyStatement(
             provider   = provider,
-            tried      = makeTryExceptSingleHandlerNode(
+            tried      = makeTryExceptSingleHandlerNodeWithPublish(
                 provider       = provider,
                 tried          = with_body,
                 exception_name = "BaseException",
@@ -206,27 +264,7 @@ def _buildWithNode(provider, context_expr, assign_target, body, body_lineno,
                             source_ref   = source_ref
                         ),
                         makeConditionalStatement(
-                            condition  = ExpressionCallNoKeywords(
-                                called     = ExpressionTempVariableRef(
-                                    variable   = tmp_exit_variable,
-                                    source_ref = with_exit_source_ref
-                                ),
-                                args       = ExpressionMakeTuple(
-                                    elements   = (
-                                        ExpressionCaughtExceptionTypeRef(
-                                            source_ref = with_exit_source_ref
-                                        ),
-                                        ExpressionCaughtExceptionValueRef(
-                                            source_ref = with_exit_source_ref
-                                        ),
-                                        ExpressionCaughtExceptionTracebackRef(
-                                            source_ref = source_ref
-                                        ),
-                                    ),
-                                    source_ref = source_ref
-                                ),
-                                source_ref = with_exit_source_ref
-                            ),
+                            condition  = exit_value_exception,
                             no_branch  = makeReraiseExceptionStatement(
                                 source_ref = with_exit_source_ref
                             ),
@@ -236,7 +274,7 @@ def _buildWithNode(provider, context_expr, assign_target, body, body_lineno,
                     ),
                     source_ref = source_ref
                 ),
-                public_exc     = Utils.python_version >= 270,
+                public_exc     = python_version >= 270,
                 source_ref     = source_ref
             ),
             final      = StatementConditional(
@@ -253,17 +291,7 @@ def _buildWithNode(provider, context_expr, assign_target, body, body_lineno,
                 ),
                 yes_branch = makeStatementsSequenceFromStatement(
                     statement = StatementExpressionOnly(
-                        expression = ExpressionCallNoKeywords(
-                            called     = ExpressionTempVariableRef(
-                                variable   = tmp_exit_variable,
-                                source_ref = source_ref
-                            ),
-                            args       = ExpressionConstantRef(
-                                constant   = (None, None, None),
-                                source_ref = source_ref
-                            ),
-                            source_ref = with_exit_source_ref
-                        ),
+                        expression = exit_value_no_exception,
                         source_ref = source_ref
                     )
                 ),
@@ -280,19 +308,19 @@ def _buildWithNode(provider, context_expr, assign_target, body, body_lineno,
         final      = (
             StatementReleaseVariable(
                 variable   = tmp_source_variable,
-                source_ref = source_ref
+                source_ref = with_exit_source_ref
             ),
             StatementReleaseVariable(
                 variable   = tmp_enter_variable,
-                source_ref = source_ref
+                source_ref = with_exit_source_ref
             ),
             StatementReleaseVariable(
                 variable   = tmp_exit_variable,
-                source_ref = source_ref
+                source_ref = with_exit_source_ref
             ),
             StatementReleaseVariable(
                 variable   = tmp_indicator_variable,
-                source_ref = source_ref
+                source_ref = with_exit_source_ref
             ),
         ),
         source_ref = source_ref
@@ -326,7 +354,7 @@ def buildWithNode(provider, node, source_ref):
     # For compatibility, we need to gather a line number for the body here
     # already, but only the full compatibility mode will use it.
     terminal_statement = node.body[-1]
-    while getKind(terminal_statement) == "With":
+    while getKind(terminal_statement) in ("With", "AsyncWith"):
         terminal_statement = terminal_statement.body[-1]
     body_lineno = terminal_statement.lineno
 
@@ -337,6 +365,46 @@ def buildWithNode(provider, node, source_ref):
             body_lineno   = body_lineno,
             context_expr  = context_expr,
             assign_target = assign_target,
+            sync          = True,
+            source_ref    = source_ref
+        )
+
+    return body
+
+
+def buildAsyncWithNode(provider, node, source_ref):
+    # "with" statements are re-formulated as described in the developer
+    # manual. Catches exceptions, and provides them to "__exit__", while making
+    # the "__enter__" value available under a given name.
+
+    # Before Python3.3, multiple context managers are not visible in the parse
+    # tree, now we need to handle it ourselves.
+    context_exprs = [item.context_expr for item in node.items]
+    assign_targets = [item.optional_vars for item in node.items]
+
+    # The body for the first context manager is the other things.
+    body = buildStatementsNode(provider, node.body, source_ref)
+
+    assert len(context_exprs) > 0 and len(context_exprs) == len(assign_targets)
+
+    context_exprs.reverse()
+    assign_targets.reverse()
+
+    # For compatibility, we need to gather a line number for the body here
+    # already, but only the full compatibility mode will use it.
+    terminal_statement = node.body[-1]
+    while getKind(terminal_statement) in ("With", "AsyncWith"):
+        terminal_statement = terminal_statement.body[-1]
+    body_lineno = terminal_statement.lineno
+
+    for context_expr, assign_target in zip(context_exprs, assign_targets):
+        body = _buildWithNode(
+            provider      = provider,
+            body          = body,
+            body_lineno   = body_lineno,
+            context_expr  = context_expr,
+            assign_target = assign_target,
+            sync          = False,
             source_ref    = source_ref
         )
 
