@@ -1,4 +1,4 @@
-#     Copyright 2014, Kay Hayen, mailto:kay.hayen@gmail.com
+#     Copyright 2016, Kay Hayen, mailto:kay.hayen@gmail.com
 #
 #     Python tests originally created or extracted from other peoples work. The
 #     parts were too small to be protected.
@@ -16,8 +16,7 @@
 #     limitations under the License.
 #
 class SimpleClass:
-    """ The class documentation."""
-
+    " The class documentation." # Leading space on purpose.
 
     # TODO: Doesn't work with Python3, because we don't yet make our own dict
     # visible.  print locals()
@@ -42,7 +41,7 @@ print "Simple class:", SimpleClass
 print "Lives in", SimpleClass.__module__
 print "Documentation", SimpleClass.__doc__
 print "Instantiate simple class:", SimpleClass(14)
-print "Call simple class normal method:", SimpleClass( 11 ).normal_method(1, 2)
+print "Call simple class normal method:", SimpleClass(11).normal_method(1, 2)
 print "Call simple class static method:", SimpleClass(11).static_method()
 
 class MetaClass(type):
@@ -55,9 +54,9 @@ print MetaClass
 class ComplexClass:
     __metaclass__ = MetaClass
 
-print ComplexClass, dir( ComplexClass )
+print ComplexClass, dir(ComplexClass)
 
-print ComplexClass, hasattr( ComplexClass, "addedin" ) and ComplexClass.addedin
+print ComplexClass, hasattr(ComplexClass, "addedin") and ComplexClass.addedin
 
 
 def function():
@@ -99,16 +98,20 @@ print "Strange class with __new__ and __del__ overloads", strangeClassBehaviour(
 class ClosureLocalizer:
     function = function
 
-    def deco(f):
+    # Using what looks like a method as a decorator.
+    def deco(f):  # @NoSelf
+        f.decorated = True
+
         return f
 
     @deco
-    def x():
+    def x(self):
         pass
 
 print "Class with a name from module level renamed to local", ClosureLocalizer.function
+print "Class method decorated", ClosureLocalizer().x.decorated
 
-print "Class with decorator"
+print "Class with decorator and meta class:"
 
 def classdecorator(cls):
     print "cls decorator", cls.addedin
@@ -124,7 +127,7 @@ print "Class that updates its locals:",
 class DictUpdating:
     a = 1
 
-    locals().update( { "b" : 2 } )
+    locals().update({ 'b' : 2 })
 
     for f in range(6):
         locals()[ "test_%s" % f ] = f
@@ -156,3 +159,18 @@ class ClassesWithNestedClass:
             return { 'a':2 }
 
 print ClassesWithNestedClass, ClassesWithNestedClass().NestedClass, ClassesWithNestedClass().NestedClass().getDict()
+
+secondary = "global closure wins"
+
+class ClassWithModuleVariableCollisionMain():
+    secondary = None
+
+    def __init__(self):
+        self.secondary = self.Child()
+        self.value = self.secondary.attr
+
+    class Child():
+        def __init__(self):
+            self.attr = secondary
+
+print ClassWithModuleVariableCollisionMain, ClassWithModuleVariableCollisionMain().value
