@@ -1,4 +1,4 @@
-#     Copyright 2015, Kay Hayen, mailto:kay.hayen@gmail.com
+#     Copyright 2016, Kay Hayen, mailto:kay.hayen@gmail.com
 #
 #     Python tests originally created or extracted from other peoples work. The
 #     parts were too small to be protected.
@@ -37,22 +37,22 @@ def b():
 
 from collections import OrderedDict
 
+def displayable(dictionary):
+    return sorted(dictionary.items())
+
 def m():
     class M(type):
-        # @classmethod
         def __new__(cls, class_name, bases, attrs, **over):
-            print("Metaclass M.__new__ cls", cls, "name", class_name, "bases", bases, "dict", attrs, "extra class defs", over)
+            print("Metaclass M.__new__ cls", cls, "name", class_name, "bases", bases, "dict", displayable(attrs), "extra class defs", displayable(over))
 
             return type.__new__(cls, class_name, bases, attrs)
 
         def __init__(self, name, bases, attrs, **over):
-            print("Metaclass M.__init__", name, bases, attrs, over)
+            print("Metaclass M.__init__", name, bases, displayable(attrs), displayable(over))
             super().__init__(name, bases, attrs)
 
-        # TODO: Removing this
-        # @classmethod
-        def __prepare__(name, bases, **over):
-            print("Metaclass M.__prepare__", name, bases, over)
+        def __prepare__(metacls, bases, **over):  # @NoSelf
+            print("Metaclass M.__prepare__", metacls, bases, displayable(over))
             return OrderedDict()
 
     print("Called", m)
@@ -71,7 +71,9 @@ def e():
 
 
 class C1(a(), b(), other = d(), metaclass = m(), yet_other = e()):
-    pass
+    import sys
+    # TODO: Enable this.
+    # print("C1 locals type is", type(sys._getframe().f_locals))
 
 print("OK, class created", C1)
 
@@ -107,4 +109,4 @@ class someClassWithPrivateArgumentNames:
     def f(self, *, __kw:1):
         pass
 print(someClassWithPrivateArgumentNames.f.__annotations__)
-print("OK.")
\ No newline at end of file
+print("OK.")
