@@ -1,4 +1,4 @@
-#     Copyright 2013, Kay Hayen, mailto:kay.hayen@gmail.com
+#     Copyright 2016, Kay Hayen, mailto:kay.hayen@gmail.com
 #
 #     Python tests originally created or extracted from other peoples work. The
 #     parts were too small to be protected.
@@ -15,12 +15,11 @@
 #     See the License for the specific language governing permissions and
 #     limitations under the License.
 #
-
 def raisy():
-   raise ValueError() from None
+    raise ValueError() from None
 
 try:
-   print( "Raising exception in a function 'from None'" )
-   raisy()
+    print("Raising exception in a function 'from None':")
+    raisy()
 except (ValueError,TypeError) as e:
-   print( "Caught as", repr(e) )
+    print("Caught as", repr(e))
