@@ -1,5 +1,5 @@
 # -*- coding: utf-8 -*-
-#     Copyright 2014, Kay Hayen, mailto:kay.hayen@gmail.com
+#     Copyright 2016, Kay Hayen, mailto:kay.hayen@gmail.com
 #
 #     Python tests originally created or extracted from other peoples work. The
 #     parts were too small to be protected.
@@ -16,10 +16,10 @@
 #     See the License for the specific language governing permissions and
 #     limitations under the License.
 #
-print "Hello World from Module main Code"
+print "Hello World from module main code"
 
 def printHelloWorld():
-    print "Hello World from Function main Code"
+    print "Hello World from function main code"
 
 print printHelloWorld
 
@@ -30,12 +30,12 @@ def printHelloWorld2(arg):
 
 print printHelloWorld2
 
-printHelloWorld2( "Hello World from Function positional argument" )
-printHelloWorld2( arg = "Hello World from Function keyword argument" )
+printHelloWorld2("Hello World from function positional argument")
+printHelloWorld2(arg = "Hello World from function keyword argument")
 
-def printHelloWorld3(arg = "Hello World from Function default argument"):
+def printHelloWorld3(arg = "Hello World from function default argument"):
     print arg
 
 print printHelloWorld3
 
-printHelloWorld3()
\ No newline at end of file
+printHelloWorld3()
