@@ -1,8 +1,8 @@
 #!/usr/bin/env python
-#     Copyright 2015, Kay Hayen, mailto:kay.hayen@gmail.com
+#     Copyright 2016, Kay Hayen, mailto:kay.hayen@gmail.com
 #
 #     Python test originally created or extracted from other peoples work. The
-#     parts from me are licensed as below. It is at least Free Softwar where
+#     parts from me are licensed as below. It is at least Free Software where
 #     it's copied from other people. In these cases, that will normally be
 #     indicated.
 #
@@ -19,7 +19,7 @@
 #     limitations under the License.
 #
 
-import os, sys
+import os, sys, subprocess
 
 # The test runner needs "lxml" itself.
 try:
@@ -166,9 +166,26 @@ for filename in sorted(os.listdir('.')):
             if "PYTHONHASHSEED" not in os.environ:
                 os.environ["PYTHONHASHSEED"] = '0'
 
+            # Coverage modules hates Nuitka to re-execute, and so we must avoid
+            # that.
+            python_path = subprocess.check_output(
+                [
+                    os.environ["PYTHON"],
+                    "-c"
+                    "import sys, os; print(os.pathsep.join(sys.path))"
+                ]
+            )
+
+            if sys.version_info >= (3,):
+                python_path = python_path.decode("utf8")
+
+            os.environ["PYTHONPATH"] = python_path.strip()
+
+
             command.insert(2, "--must-not-re-execute")
 
             command = command[0:1] + [
+                "-S",
                 "-m",
                 "coverage",
                 "run",
