@@ -1,4 +1,4 @@
-#     Copyright 2014, Kay Hayen, mailto:kay.hayen@gmail.com
+#     Copyright 2016, Kay Hayen, mailto:kay.hayen@gmail.com
 #
 #     Python tests originally created or extracted from other peoples work. The
 #     parts were too small to be protected.
@@ -21,5 +21,5 @@ class BaseOptions(object):
     _save_attr = ()
 
 if __debug__:
-    if __name__=='__main__':
+    if __name__ == "__main__":
         o = BaseOptions()
